@@ -1,3 +1,43 @@
+/*
+ *  Project   : tin - a Usenet reader
+ *  Module    : tinrc.h
+ *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
+ *  Created   :
+ *  Updated   : 2000-01-03
+ *  Notes     :
+ *
+ * Copyright (c) 1999-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Jason Faultless.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+
 /*
  * These are the tin defaults read from the tinrc file
  * If you change this, ensure you change the initial values in init.c
@@ -6,6 +46,10 @@
  *        and could be nuked if tin comes with a prefilled .inputhistory
  *        which is installed automatically if no .inputhistory is found.
  */
+
+#ifndef TINRC_H
+#	define TINRC_H 1
+
 struct t_config {
 	/*
 	 * Chars used to show article status
@@ -14,7 +58,11 @@ struct t_config {
 	char art_marked_inrange;
 	char art_marked_return;
 	char art_marked_selected;
+	char art_marked_recent;
 	char art_marked_unread;
+	char art_marked_read;
+	char art_marked_killed;
+	char art_marked_read_selected;
 	char editor_format[PATH_LEN];		/* editor + parameters  %E +%N %F */
 	char default_goto_group[HEADER_LEN];		/* default for the 'g' command */
 	char default_mail_address[HEADER_LEN];
@@ -42,12 +90,20 @@ struct t_config {
 	char default_shell_command[LEN];
 	char mail_quote_format[LEN];
 	char maildir[PATH_LEN];				/* mailbox dir where = saves are stored */
+	int mailbox_format;					/* format of the mailbox (mboxo, mboxrd, mmdf, ...) */
 	char mail_address[HEADER_LEN];				/* user's mail address */
-	char mm_charset[LEN];				/* MIME charset: moved from rfc1522.c */
+#ifndef CHARSET_CONVERSION
+	char mm_charset[LEN];				/* MIME charset */
+#else
+	int mm_network_charset;				/* MIME charset */
+#endif /* !CHARSET_CONVERSION */
+	char mm_local_charset[LEN];		/* display charset, not a rc/Menu-option anymore -> should be moved elsewhere */
+#ifdef HAVE_ICONV_OPEN_TRANSLIT
+	t_bool translit;						/* use //TRANSLIT */
+#endif /* HAVE_ICONV_OPEN_TRANSLIT */
 	char news_headers_to_display[LEN];	/* which headers to display */
 	char news_headers_to_not_display[LEN];	/* which headers to not display */
 	char news_quote_format[LEN];
-	char post_process_command[PATH_LEN];		/* Post processing command */
 	char quote_chars[LEN];			/* quote chars for posting/mails ": " (size matches prefixbuf in copy_body() */
 #ifdef HAVE_COLOR
 	char quote_regex[LEN];				/* regex used to determine quoted lines */
@@ -66,6 +122,7 @@ struct t_config {
 	int default_move_group;
 	int default_save_mode;				/* Append/Overwrite existing file when saving */
 	int getart_limit;					/* number of article to get */
+	int recent_time;				/* Time limit when article is "fresh" */
 	int groupname_max_length;			/* max len of group names to display on screen */
 	int kill_level;						/* Define how killed articles are shown */
 	int mail_mime_encoding;
@@ -74,6 +131,7 @@ struct t_config {
 	int reread_active_file_secs;		/* reread active file interval in seconds */
 	int show_author;					/* show_author value from 'M' menu in tinrc */
 	int sort_article_type;				/* method used to sort arts[] */
+	int sort_threads_type;				/* method used to sort base[] */
 	int strip_bogus;
 	int thread_articles;				/* threading system for viewing articles */
 	int wildcard;						/* 0=wildmat, 1=regex */
@@ -116,8 +174,6 @@ struct t_config {
 	t_bool catchup_read_groups;			/* ask if read groups are to be marked read */
 	t_bool confirm_action;
 	t_bool confirm_to_quit;
-	t_bool display_mime_allheader_asis;	/* rfc 1522/2047 all heades (^H) will not be decoded by default */
-	t_bool display_mime_header_asis;	/* rfc 1522/2047 news_headers_to_display will be decoded by default */
 	t_bool draw_arrow;					/* draw -> or highlighted bar */
 	t_bool force_screen_redraw;			/* force screen redraw after external (shell) commands */
 	t_bool full_page_scroll;			/* page half/full screen of articles/groups */
@@ -126,11 +182,13 @@ struct t_config {
 	t_bool inverse_okay;
 	t_bool keep_dead_articles;			/* keep all dead articles in dead.articles */
 	t_bool keep_posted_articles;		/* keep all posted articles in ~/Mail/posted */
+	char keep_posted_articles_file[LEN];		/* file, to keep posted articles */
 	t_bool mail_8bit_header;			/* allow 8bit chars. in header of mail message */
 	t_bool mark_saved_read;				/* mark saved article/thread as read */
 	t_bool pgdn_goto_next;
 	t_bool pos_first_unread;			/* position cursor at first/last unread article */
 	t_bool post_8bit_header;			/* allow 8bit chars. in header when posting to newsgroup */
+	t_bool post_process_view;			/* set TRUE to invoke mailcap viewer app */
 #ifndef DISABLE_PRINTING
 	t_bool print_header;				/* print all of mail header or just Subject: & From lines */
 #endif /* !DISABLE_PRINTING */
@@ -138,7 +196,6 @@ struct t_config {
 	t_bool prompt_followupto;			/* display empty Followup-To header in editor */
 	t_bool quote_empty_lines;			/* quote empty lines, too */
 	t_bool quote_signatures;			/* quote signatures */
-	t_bool save_to_mmdf_mailbox;		/* save mail to MMDF/mbox format mailbox */
 	t_bool show_description;
 	t_bool show_last_line_prev_page;	/* set TRUE to see last line of prev page (ala nn) */
 	t_bool show_lines;
@@ -146,7 +203,7 @@ struct t_config {
 	t_bool show_only_unread_groups;		/* set TRUE to see only subscribed groups with new news */
 	t_bool show_score;
 	t_bool show_signatures;				/* show signatures when displaying articles */
-	t_bool show_xcommentto;				/* set TRUE to show X-Comment-To in upper right corner */
+	t_bool hide_uue;					/* set TRUE to treat uuencoded data as an attachment */
 	t_bool sigdashes;					/* set TRUE to prepend every signature with dashes */
 	t_bool signature_repost;			/* set TRUE to add signature when reposting articles */
 	t_bool space_goto_next_unread;
@@ -155,9 +212,10 @@ struct t_config {
 	t_bool strip_newsrc;
 	t_bool tab_after_X_selection;		/* set TRUE if you want auto TAB after X */
 	t_bool tab_goto_next_unread;
+	t_bool tex2iso_conv;			/* convert "a to Umlaut-a */
 	t_bool thread_catchup_on_exit;		/* catchup thread with left arrow key or not */
 	t_bool unlink_article;
-	t_bool use_builtin_inews;
+	char inews_prog[PATH_LEN];
 	t_bool use_getart_limit;
 	t_bool use_mailreader_i;			/* invoke user's mailreader earlier to use more of its features (i = interactive) */
 	t_bool use_mouse;					/* enables/disables mouse support under xterm */
@@ -175,3 +233,5 @@ struct t_config {
 	t_bool default_filter_select_expire;
 	t_bool default_filter_select_global;
 };
+
+#endif /* !TINRC_H */
