@@ -3,33 +3,61 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 1998-05-02
+ *  Updated   : 1999-11-19
  *  Notes     :
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1992-2002 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
-#	include	"tin.h"
+#	include "tin.h"
 #endif /* !TIN_H */
 #ifndef TCURSES_H
-#	include	"tcurses.h"
+#	include "tcurses.h"
 #endif /* !TCURSES_H */
 
 /*
  * List of allowed seperator chars in active file
  * unsed in parse_active_line()
  */
-#define ACTIVE_SEP		" \n"
+#define ACTIVE_SEP	" \n"
 
 t_bool force_reread_active_file = FALSE;
 
-static char acSaveActiveFile[PATH_LEN];
 static time_t active_timestamp;	/* time active file read (local) */
 
+/*
+ * if you are using C-News nntpd set NUM_SIMULTANEOUS_GROUP_COMMAND to 1
+ */
 #ifdef NNTP_ABLE
 #	define NUM_SIMULTANEOUS_GROUP_COMMAND 50
 #endif /* NNTP_ABLE */
@@ -40,25 +68,24 @@ static time_t active_timestamp;	/* time active file read (local) */
  */
 static int find_newnews_index (char *cur_newnews_host);
 static void active_add (struct t_group *ptr, long count, long max, long min, const char *moderated);
+static void append_group_line (char *active_file, char *group_path, long art_max, long art_min, char *base_dir);
 static void check_for_any_new_groups (void);
+static void make_group_list (char *active_file, char *base_dir, char *group_path);
+static void make_group_name (char *base_dir, char *group_name, char *group_path);
 static void read_active_file (void);
 static void read_newsrc_active_file (void);
-static void set_active_timestamp (void);
 static void subscribe_new_group (char *group, char *autosubscribe, char *autounsubscribe);
-static void vAppendGrpLine (char *pcActiveFile, char *pcGrpPath, long lArtMax, long lArtMin, char *pcBaseDir);
-static void vInitVariables (void);
-static void vMakeGrpList (char *pcActiveFile, char *pcBaseDir, char *pcGrpPath);
 
 
 /*
- *  Get default array size for active[] from environment (AmigaDOS)
- *  or just return the standard default.
+ * Get default array size for active[] from environment (AmigaDOS)
+ * or just return the standard default.
  */
 int
 get_active_num (
 	void)
 {
-#ifdef ENV_VAR_GROUPS /* M_AMIGA && M_OS2 && WIN32 only */
+#ifdef ENV_VAR_GROUPS /* M_AMIGA only */
 	char *ptr;
 	int num;
 
@@ -69,17 +96,8 @@ get_active_num (
 }
 
 
-static void
-set_active_timestamp (
-	void)
-{
-	(void) time (&active_timestamp);
-	force_reread_active_file = FALSE;
-}
-
-
 t_bool
-reread_active_file (
+need_reread_active_file (
 	void)
 {
 	return (force_reread_active_file || (tinrc.reread_active_file_secs != 0 &&
@@ -90,6 +108,7 @@ reread_active_file (
 /*
  * Resync active file when reread_active_file_secs have passed or
  * force_reread_actve_file is set.
+ * Return TRUE if a reread was performed
  */
 t_bool
 resync_active_file (
@@ -98,19 +117,23 @@ resync_active_file (
 	char old_group[HEADER_LEN];
 	t_bool command_line = FALSE;
 
-	if (!reread_active_file ())
+	if (!need_reread_active_file ())
 		return FALSE;
 
 	reread_active_for_posted_arts = FALSE;
 
-	if (cur_groupnum >= 0 && group_top)
+	if (selmenu.curr >= 0 && selmenu.max)
 		strcpy (old_group, CURR_GROUP.name);
 	else
 		old_group[0] = '\0';
 
-	vWriteNewsrc ();
+	write_newsrc ();
 	read_news_active_file ();
 
+#ifdef HAVE_MH_MAIL_HANDLING
+	read_mail_active_file ();
+#endif /* HAVE_MH_MAIL_HANDLING */
+
 	if (read_cmd_line_groups ())
 		command_line = TRUE;
 
@@ -142,7 +165,7 @@ active_add (
 	const char *moderated)
 {
 	/* name - pre-initialised when group is made */
-	ptr->aliasedto = ((moderated[0] == '=') ? my_strdup(moderated+1) : (char *) 0);
+	ptr->aliasedto = ((moderated[0] == '=') ? my_strdup(moderated + 1) : (char *) 0);
 	ptr->description = (char *) 0;
 	/* spool - see below */
 	ptr->moderated = moderated[0];
@@ -160,17 +183,9 @@ active_add (
 	ptr->newsrc.xbitmap = (t_bitmap *) 0;
 	ptr->attribute = (struct t_attribute *) 0;
 	ptr->glob_filter = &glob_filter;
-	vSetDefaultBitmap (ptr);
-#ifdef INDEX_DAEMON
-	ptr->last_updated_time = (time_t) 0;
-#endif /* INDEX_DAEMON */
+	set_default_bitmap (ptr);
 
-#ifdef WIN32				/* Paths are in form - x:\a\b\c */
-	if (strchr(moderated, '\\'))
-#else
-	if (moderated[0] == '/')
-#endif /* WIN32 */
-	{
+	if (moderated[0] == '/') {
 		ptr->type = GROUP_TYPE_SAVE;
 		ptr->spooldir = my_strdup(moderated);
 	} else {
@@ -184,7 +199,8 @@ active_add (
  * Decide how to handle a bogus groupname.
  * If we process them interactively, create an empty active[] for this
  * group and mark it bogus for display in the group selection page
- * Otherwise, bogus groups are dealt with when newsrc is written.
+ * Otherwise, bogus groups are not displayed and are dealt with when newsrc
+ * is written.
  */
 t_bool
 process_bogus (
@@ -192,10 +208,10 @@ process_bogus (
 {
 	struct t_group *ptr;
 
-	if (read_saved_news)
+	if (read_saved_news || tinrc.strip_bogus != BOGUS_ASK)
 		return FALSE;
 
-	if ((ptr = psGrpAdd(name)) == NULL)
+	if ((ptr = group_add(name)) == NULL)
 		return FALSE;
 
 	active_add (ptr, 0L, 1L, 0L, "n");
@@ -229,7 +245,7 @@ parse_active_line (
 	r = strtok((char *)0, ACTIVE_SEP);	/* mod status or path to mailgroup */
 
 	if (!p || !q || !r) {
-		error_message (txt_bad_active_file, line);
+		error_message (_(txt_bad_active_file), line);
 		return FALSE;
 	}
 
@@ -254,7 +270,7 @@ static void
 read_newsrc_active_file (
 	void)
 {
-	FILE *fp = (FILE *) 0;
+	FILE *fp;
 	char *ptr;
 	char moderated[PATH_LEN];
 	int window = 0;
@@ -275,13 +291,13 @@ read_newsrc_active_file (
 	if ((fp = fopen (newsrc, "r")) == (FILE *) 0)
 		return;
 
-	if (file_size(newsrc) <= 0) {
+	if (file_size(newsrc) <= 0L) {
 		fclose(fp);
 		return;
 	}
 
-	if (INTERACTIVE)
-		wait_message (0, txt_reading_news_newsrc_file);
+	if (!batch_mode)
+		wait_message (0, _(txt_reading_news_newsrc_file));
 
 	while ((ptr = tin_fgets (fp, FALSE)) != (char *)0 || window != 0) {
 		if (ptr) {
@@ -301,20 +317,20 @@ read_newsrc_active_file (
 
 		if (read_news_via_nntp) {
 #ifdef NNTP_ABLE
-			char acBuf[NNTP_STRLEN];
-			char acLine[NNTP_STRLEN];
+			char buf[NNTP_STRLEN];
+			char line[NNTP_STRLEN];
 			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr) {
 				ngnames[index_i] = my_strdup(ptr);
-				sprintf (acBuf, "GROUP %s", ngnames[index_i]);
+				sprintf (buf, "GROUP %s", ngnames[index_i]);
 #	ifdef DEBUG
-				debug_nntp ("read_newsrc_active_file", acBuf);
+				debug_nntp ("read_newsrc_active_file", buf);
 #	endif /* DEBUG */
-				put_server (acBuf);
+				put_server (buf);
 				index_i = (index_i + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 				window++;
 			}
 			if (window == NUM_SIMULTANEOUS_GROUP_COMMAND || ptr == NULL) {
-				int respcode = get_respcode(acLine);
+				int respcode = get_respcode(line);
 
 				if (reconnected_in_last_get_server) {
 					/*
@@ -326,11 +342,11 @@ read_newsrc_active_file (
 					int i;
 					int j = index_o;
 					for (i = 0; i < window - 1; i++) {
-						sprintf (acBuf, "GROUP %s", ngnames[j]);
+						sprintf (buf, "GROUP %s", ngnames[j]);
 #	ifdef DEBUG
-						debug_nntp ("read_newsrc_active_file", acBuf);
+						debug_nntp ("read_newsrc_active_file", buf);
 #	endif /* DEBUG */
-						put_server (acBuf);
+						put_server (buf);
 						j = (j + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 					}
 					if (--index_o < 0)
@@ -349,10 +365,10 @@ read_newsrc_active_file (
 
 							sprintf(fmt, "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
 
-							if (sscanf (acLine, fmt, &count, &min, &max, ngname) != 4)
-								error_message("Invalid response to GROUP command, %s", acLine); /* FIXME: -> lang.c*/
+							if (sscanf (line, fmt, &count, &min, &max, ngname) != 4)
+								error_message(_("Invalid response to GROUP command, %s"), line); /* FIXME: -> lang.c */
 							if (strcmp(ngname, ngnames[index_o]) != 0)
-								error_message("Wrong newsgroup name in response of GROUP command, %s for %s", acLine, ngnames[index_o]); /* FIXME: -> lang.c */
+								error_message(_("Wrong newsgroup name in response of GROUP command, %s for %s"), line, ngnames[index_o]); /* FIXME: -> lang.c */
 							ptr = ngname;
 							free(ngnames[index_o]);
 							index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -367,13 +383,13 @@ read_newsrc_active_file (
 						continue;
 
 					case ERR_ACCESS:
-						error_message (cCRLF "%s", acLine);
+						error_message (cCRLF "%s", line);
 						tin_done (NNTP_ERROR_EXIT);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
 					default:
 #	ifdef DEBUG
-						debug_nntp ("NOT_OK", acLine);
+						debug_nntp ("NOT_OK", line);
 #	endif /* DEBUG */
 						free(ngnames[index_o]);
 						index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -384,7 +400,7 @@ read_newsrc_active_file (
 				continue;
 #endif /* NNTP_ABLE */
 		} else {
-			if (vGrpGetArtInfo (spooldir, ptr, GROUP_TYPE_NEWS, &count, &max, &min))
+			if (group_get_art_info (spooldir, ptr, GROUP_TYPE_NEWS, &count, &max, &min))
 				continue;
 		}
 
@@ -398,10 +414,10 @@ read_newsrc_active_file (
 		 * NULL means group already present, so we just fixup the counters
 		 * This call may implicitly ++num_active
 		 */
-		if ((grpptr = psGrpAdd(ptr)) == NULL) {
+		if ((grpptr = group_add(ptr)) == NULL) {
 			t_bool changed = FALSE;
 
-			if ((grpptr = psGrpFind(ptr)) == NULL)
+			if ((grpptr = group_find(ptr)) == NULL)
 				continue;
 
 			if (max > grpptr->xmax) {
@@ -414,7 +430,7 @@ read_newsrc_active_file (
 			}
 			if (changed) {
 				grpptr->count = count;
-				expand_bitmap(grpptr, 0); /* expand_bitmap(grpptr,grpptr->xmin) should be enought*/
+				expand_bitmap(grpptr, 0); /* expand_bitmap(grpptr,grpptr->xmin) should be enough */
 			}
 			continue;
 		}
@@ -428,14 +444,14 @@ read_newsrc_active_file (
 	fclose(fp);
 
 	/*
-	 *  Exit if active file wasn't read correctly or is empty
+	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
-		error_message (txt_active_file_is_empty, (read_news_via_nntp ? txt_servers_active : news_active_file));
+		error_message (_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
 		tin_done (EXIT_FAILURE);
 	}
 
-	if (INTERACTIVE)
+	if (!batch_mode)
 		my_fputs("\n", stdout);
 }
 
@@ -454,21 +470,21 @@ read_active_file (
 	long processed = 0L;
 	struct t_group *grpptr;
 
-	if (INTERACTIVE)
-		wait_message (0, txt_reading_news_active_file);
+	if (!batch_mode)
+		wait_message (0, _(txt_reading_news_active_file));
 
 	if ((fp = open_news_active_fp ()) == (FILE *) 0) {
 
-		if (cmd_line)
+		if (cmd_line && !batch_mode)
 			my_fputc ('\n', stderr);
 
 #if defined(NNTP_ABLE) || defined(NNTP_ONLY)
 		if (read_news_via_nntp)
 #endif /* NNTP_ABLE || NNTP_ONLY */
-			error_message (txt_cannot_open, news_active_file);
+			error_message (_(txt_cannot_open), news_active_file);
 #if defined(NNTP_ABLE) || defined(NNTP_ONLY)
 		else
-			error_message (txt_cannot_open_active_file, news_active_file, tin_progname);
+			error_message (_(txt_cannot_open_active_file), news_active_file, tin_progname);
 #endif /* NNTP_ABLE || NNTP_ONLY */
 
 		tin_done (EXIT_FAILURE);
@@ -486,9 +502,9 @@ read_active_file (
 		 * NULL means group already present, so we just fixup the counters
 		 * This call may implicitly ++num_active
 		 */
-		if ((grpptr = psGrpAdd(ptr)) == NULL) {
+		if ((grpptr = group_add(ptr)) == NULL) {
 
-			if ((grpptr = psGrpFind(ptr)) == NULL)
+			if ((grpptr = group_find(ptr)) == NULL)
 				continue;
 
 			if (max > grpptr->xmax) {
@@ -513,14 +529,14 @@ read_active_file (
 	TIN_FCLOSE (fp);
 
 	/*
-	 *  Exit if active file wasn't read correctly or is empty
+	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
-		error_message (txt_active_file_is_empty, (read_news_via_nntp ? txt_servers_active : news_active_file));
+		error_message (_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
 		tin_done (EXIT_FAILURE);
 	}
 
-	if (INTERACTIVE)
+	if (!batch_mode)
 		my_fputs("\n", stdout);
 }
 
@@ -545,7 +561,7 @@ read_news_active_file (
 		} else
 			fclose(fp);
 
-		if (file_size(newsrc) <= 0) {
+		if (file_size(newsrc) <= 0L) {
 			list_active = TRUE;
 			newsrc_active = FALSE;
 		}
@@ -559,7 +575,8 @@ read_news_active_file (
 	if (newsrc_active)
 		read_newsrc_active_file ();
 
-	set_active_timestamp ();
+	(void) time (&active_timestamp);
+	force_reread_active_file = FALSE;
 
 	check_for_any_new_groups ();
 }
@@ -587,10 +604,14 @@ check_for_any_new_groups (
 	time_t old_newnews_time;
 	time_t new_newnews_time;
 
-	if (!check_for_new_newsgroups || !INTERACTIVE)
+	if (!check_for_new_newsgroups /* || !batch_mode */)
 		return;
 
-	wait_message (0, txt_checking_new_groups);
+#if 0
+	if (!batch_mode /* || (batch_mode && verbose) */)
+#endif /* 0 */
+		wait_message (0, _(txt_checking_new_groups));
+
 	(void) time (&new_newnews_time);
 	strcpy (new_newnews_host, (read_news_via_nntp ? nntp_server : "local")); /* What if nntp server called local ? */
 
@@ -634,15 +655,15 @@ check_for_any_new_groups (
 				*ptr = '\0';
 			}
 
-			free_attributes_array ();
-			read_attributes_file (global_attributes_file, TRUE);
-			read_attributes_file (local_attributes_file, FALSE);
-
 			subscribe_new_group (line, autosubscribe, autounsubscribe);
 		}
 
 		TIN_FCLOSE (fp);
 
+		free_attributes_array ();
+		read_attributes_file (global_attributes_file, TRUE);
+		read_attributes_file (local_attributes_file, FALSE);
+
 		if (tin_errno)
 			return;				/* Don't update the time if we quit */
 	}
@@ -658,6 +679,7 @@ check_for_any_new_groups (
 		sprintf (buf, "%s %lu", new_newnews_host, (unsigned long int) new_newnews_time);
 		load_newnews_info (buf);
 	}
+	my_fputc('\n', stdout);
 }
 
 
@@ -697,7 +719,7 @@ subscribe_new_group (
 			return;
 		}
 
-		if ((ptr = psGrpAdd(group)) != NULL)
+		if ((ptr = group_add(group)) != NULL)
 			active_add(ptr, 0L, 1L, 0L, "y");
 
 		if ((idx = my_group_add(group)) < 0)
@@ -705,7 +727,7 @@ subscribe_new_group (
 	}
 
 	if (!no_write && (autosubscribe != (char *) 0) && match_group_list (group, autosubscribe)) {
-		my_printf (txt_autosubscribed, group);
+		my_printf (_(txt_autosubscribed), group);
 
 		subscribe (&active[my_group[idx]], SUBSCRIBED);
 		/*
@@ -713,7 +735,7 @@ subscribe_new_group (
 		 * effectively loses the group, and it has now been subscribed to and
 		 * so will be reread later by read_newsrc()
 		 */
-		group_top--;
+		selmenu.max--;
 	} else
 		active[my_group[idx]].newgroup = TRUE;
 }
@@ -726,8 +748,8 @@ subscribe_new_group (
  */
 t_bool
 match_group_list (
-	char *group,
-	char *group_list)
+	const char *group,
+	const char *group_list)
 {
 	char *separator;
 	char pattern[HEADER_LEN];
@@ -739,7 +761,7 @@ match_group_list (
 	/*
 	 * walk through comma-separated entries in list
 	 */
-	while (list_len > 0) {
+	while (list_len != 0) {
 		/*
 		 * find end/length of this entry
 		 */
@@ -763,7 +785,7 @@ match_group_list (
 		 * case-insensitive wildcard match
 		 */
 		if (GROUP_MATCH(group, pattern, TRUE))
-			accept = !negate;	/* matched!*/
+			accept = !negate;	/* matched! */
 
 		/*
 		 * now examine next entry if any
@@ -774,7 +796,7 @@ match_group_list (
 		group_list += group_len;
 		list_len -= group_len;
 	}
-	return (accept);
+	return accept;
 }
 
 
@@ -810,8 +832,8 @@ load_newnews_info (
 	if ((ptr = strchr (info, ' ')) == (char *) 0)
 		return;
 
+	*ptr++ = '\0';
 	new_time = (time_t) atol (ptr);
-	*ptr = '\0';
 
 	/*
 	 * If this is a new host entry, set it up
@@ -844,7 +866,7 @@ find_newnews_index (
 
 	for (i = 0; i < num_newnews; i++) {
 		if (STRCMPEQ(cur_newnews_host, newnews[i].host))
-			return(i);
+			return i;
 	}
 
 	return -1;
@@ -857,7 +879,7 @@ find_newnews_index (
  */
 char
 group_flag (
-	int ch)
+	char ch)
 {
 	switch (ch) {
 		case 'm':
@@ -878,235 +900,130 @@ void
 create_save_active_file (
 	void)
 {
-	char acGrpPath[PATH_LEN];
+	char group_path[PATH_LEN];
+	char local_save_active_file[PATH_LEN];
 
-	vInitVariables ();
+	snprintf (local_save_active_file, sizeof(local_save_active_file) - 1, "%s/%s/%s", homedir, RCDIR, ACTIVE_SAVE_FILE);
 
-	if (no_write && file_size (acSaveActiveFile) != -1)
+	if (no_write && file_size (local_save_active_file) != -1L)
 		return;
 
-	my_printf (txt_creating_active);
+	my_printf (_(txt_creating_active));
 
-	vPrintActiveHead (acSaveActiveFile);
-	strcpy (acGrpPath, tinrc.savedir);
-	vMakeGrpList (acSaveActiveFile, tinrc.savedir, acGrpPath);
+	print_active_head (local_save_active_file);
+	strcpy (group_path, tinrc.savedir);
+	make_group_list (local_save_active_file, tinrc.savedir, group_path);
 }
 
 
 static void
-vInitVariables (
-	void)
+make_group_list (
+	char *active_file,
+	char *base_dir,
+	char *group_path)
 {
-	char *pcPtr;
-	char acTempActiveFile[PATH_LEN];
-	char acMailActiveFile[PATH_LEN];
-	char acMailDir[PATH_LEN];
-	char acHomeDir[PATH_LEN];
-	char acSaveDir[PATH_LEN];
-#ifndef M_AMIGA
-	struct passwd *psPwd;
-	struct passwd sPwd;
+	DIR *dir;
+	DIR_BUF *direntry;
+	char *ptr;
+	char filename[PATH_LEN];
+	char path[PATH_LEN];
+	long art_max;
+	long art_min;
+	struct stat stat_info;
+	t_bool is_dir;
 
-	psPwd = (struct passwd *) 0;
-	if (((pcPtr = getlogin ()) != (char *) 0) && strlen (pcPtr))
-		psPwd = getpwnam (pcPtr);
+	if ((dir = opendir (group_path)) != (DIR *) 0) {
+		is_dir = FALSE;
+		while ((direntry = readdir (dir)) != (DIR_BUF *) 0) {
+			STRCPY(filename, direntry->d_name);
+			sprintf (path, "%s/%s", group_path, filename);
 
-	if (psPwd == (struct passwd *) 0)
-		psPwd = getpwuid (getuid ());
-
-	if (psPwd != (struct passwd *) 0) {
-		memcpy (&sPwd, psPwd, sizeof (struct passwd));
-		psPwd = &sPwd;
-	}
-#endif /* !M_AMIGA */
-
-	if ((pcPtr = getenv ("TIN_HOMEDIR")) != (char *) 0) {
-		strcpy (acHomeDir, pcPtr);
-	} else if ((pcPtr = getenv ("HOME")) != (char *) 0) {
-		strcpy (acHomeDir, pcPtr);
-#ifndef M_AMIGA
-	} else if (psPwd != (struct passwd *) 0) {
-		strcpy (acHomeDir, psPwd->pw_dir);
-	} else
-		strcpy (acHomeDir, "/tmp");
-#else
-	} else
-		strcpy (acHomeDir, "T:");
-#endif /* !M_AMIGA */
-
-#ifdef WIN32
-#	define DOTTINDIR "tin"
-#else
-#	define DOTTINDIR ".tin"
-#endif /* WIN32 */
-	sprintf (acTempActiveFile, "%s/%s/%ld.tmp", acHomeDir, DOTTINDIR, (long) getpid ());
-	sprintf (acMailActiveFile, "%s/%s/%s", acHomeDir, DOTTINDIR, ACTIVE_MAIL_FILE);
-	sprintf (acSaveActiveFile, "%s/%s/%s", acHomeDir, DOTTINDIR, ACTIVE_SAVE_FILE);
-	sprintf (acMailDir, "%s/Mail", acHomeDir);
-	sprintf (acSaveDir, "%s/News", acHomeDir);
-}
-
-
-static void
-vMakeGrpList (
-	char *pcActiveFile,
-	char *pcBaseDir,
-	char *pcGrpPath)
-{
-	DIR *tDirFile;
-	DIR_BUF *tFile;
-	char *pcPtr;
-	char acFile[PATH_LEN];
-	char acPath[PATH_LEN];
-	long lArtMax;
-	long lArtMin;
-	struct stat sStatInfo;
-	t_bool bIsDir;
-
-	if (access (pcGrpPath, R_OK))
-		return;
-
-	tDirFile = opendir (pcGrpPath);
-
-	if (tDirFile != (DIR *) 0) {
-		bIsDir = FALSE;
-		while ((tFile = readdir (tDirFile)) != (DIR_BUF *) 0) {
-			STRCPY(acFile, tFile->d_name);
-			sprintf (acPath, "%s/%s", pcGrpPath, acFile);
-
-			if (!(acFile[0] == '.' && acFile[1] == '\0') &&
-				!(acFile[0] == '.' && acFile[1] == '.' && acFile[2] == '\0')) {
-				if (stat (acPath, &sStatInfo) != -1) {
-					if (S_ISDIR(sStatInfo.st_mode))
-						bIsDir = TRUE;
+			if (!(filename[0] == '.' && filename[1] == '\0') &&
+				!(filename[0] == '.' && filename[1] == '.' && filename[2] == '\0')) {
+				if (stat (path, &stat_info) != -1) {
+					if (S_ISDIR(stat_info.st_mode))
+						is_dir = TRUE;
 				}
 			}
-			if (bIsDir) {
-				bIsDir = FALSE;
-				strcpy (pcGrpPath, acPath);
+			if (is_dir) {
+				is_dir = FALSE;
+				strcpy (group_path, path);
 
-				vMakeGrpList (pcActiveFile, pcBaseDir, pcGrpPath);
-				vFindArtMaxMin (pcGrpPath, &lArtMax, &lArtMin);
-				vAppendGrpLine (pcActiveFile, pcGrpPath, lArtMax, lArtMin, pcBaseDir);
+				make_group_list (active_file, base_dir, group_path);
+				find_art_max_min (group_path, &art_max, &art_min);
+				append_group_line (active_file, group_path, art_max, art_min, base_dir);
 
-				pcPtr = strrchr (pcGrpPath, '/');
-				if (pcPtr != (char *) 0)
-					*pcPtr = '\0';
+				ptr = strrchr (group_path, '/');
+				if (ptr != (char *) 0)
+					*ptr = '\0';
 			}
 		}
-		CLOSEDIR(tDirFile);
+		CLOSEDIR(dir);
 	}
 }
 
 
 static void
-vAppendGrpLine (
-	char *pcActiveFile,
-	char *pcGrpPath,
-	long lArtMax,
-	long lArtMin,
-	char *pcBaseDir)
+append_group_line (
+	char *active_file,
+	char *group_path,
+	long art_max,
+	long art_min,
+	char *base_dir)
 {
-	FILE *hFp;
-	char acGrpName[PATH_LEN];
+	FILE *fp;
+	char *file_tmp;
+	char group_name[PATH_LEN];
 
-	if (lArtMax == 0 && lArtMin == 1)
+	if (art_max == 0 && art_min == 1)
 		return;
 
-	if ((hFp = fopen (pcActiveFile, "a+")) != (FILE *) 0) {
-		vMakeGrpName (pcBaseDir, acGrpName, pcGrpPath);
-		my_printf ("Appending=[%s %ld %ld %s]\n", acGrpName, lArtMax, lArtMin, pcBaseDir);
-		vPrintGrpLine (hFp, acGrpName, lArtMax, lArtMin, pcBaseDir);
-		fclose (hFp);
+	file_tmp = get_tmpfilename(active_file);
+
+	if (!backup_file (active_file, file_tmp)) {
+		free (file_tmp);
+		return;
 	}
-}
 
-
-#ifdef INDEX_DAEMON
-void
-vMakeActiveMyGroup (
-	void)
-{
-	register int iNum;
-
-	group_top = 0;
-
-	for (iNum = 0; iNum < num_active; iNum++)
-		my_group[group_top++] = iNum;
+	if ((fp = fopen (active_file, "a+")) != (FILE *) 0) {
+		make_group_name (base_dir, group_name, group_path);
+		my_printf ("Appending=[%s %ld %ld %s]\n", group_name, art_max, art_min, base_dir);
+		print_group_line (fp, group_name, art_max, art_min, base_dir);
+		if (ferror(fp) || fclose (fp))
+			/* TODO: issue warning? */
+			rename (file_tmp, active_file);
+	}
+	unlink (file_tmp);
+	free (file_tmp);
 }
 
 
 /*
- *  Load the last updated time for each group in the active file so that
- *  tind is more efficient and only has to stat the group dir and compare
- *  the last changed time with the time read from the ~/.tin/group.times
- *  file to determine if the group needs updating.
- *
- *  alt.sources 71234589
- *  comp.archives 71234890
+ * Given an absolute pathname & a base pathname build a newsgroup name
+ * base = /usr/spool/news
+ * absolute path = /usr/spool/news/alt/sources
+ * newsgroup = alt.sources
  */
-void
-read_group_times_file (
-	void)
+static void
+make_group_name (
+	char *base_dir,
+	char *group_name,
+	char *group_path)
 {
-	FILE *fp;
-	char *p, *q;
-	char buf[HEADER_LEN];
-	char group[HEADER_LEN];
-	time_t updated_time;
-	struct t_group *psGrp;
+	char *base_ptr;
+	char *name_ptr;
+	char *path_ptr;
 
-	if ((fp = fopen (group_times_file, "r")) == (FILE *) 0)
-		return;
-
-	while (fgets (buf, (int) sizeof (buf), fp) != (char *) 0) {
-		/*
-		 * read the group name
-		 */
-		for (p = buf, q = group; *p && *p != ' ' && *p != '\t'; p++, q++)
-			*q = *p;
-
-		*q = '\0';
-
-		/*
-		 * read the last updated time
-		 */
-		updated_time = (time_t) atol (p);
-
-		/*
-		 * find the group in active[] and set updated time
-		 */
-		psGrp = psGrpFind (group);
-
-		if (psGrp)
-			psGrp->last_updated_time = updated_time;
-
-#	ifdef DEBUG
-		if (debug == 2)
-			my_printf ("group=[%-40.40s]  [%lu]\n", psGrp->name, (unsigned long int) psGrp->last_updated_time);
-#	endif /* DEBUG */
+	base_ptr = base_dir;
+	path_ptr = group_path;
 
+	while (*base_ptr && (*base_ptr == *path_ptr)) {
+		base_ptr++;
+		path_ptr++;
 	}
-	fclose (fp);
+	strcpy (group_name, ++path_ptr);
+
+	name_ptr = group_name;
+	while ((name_ptr = strchr (name_ptr, '/')) != (char *) 0)
+		*name_ptr = '.';
 }
-
-
-/*
- *  Save the last updated time for each group to ~/.tin/group.times
- */
-void
-write_group_times_file (
-	void)
-{
-	FILE *fp;
-	register int i;
-
-	if ((fp = fopen (group_times_file, "w")) == (FILE *) 0)
-		return;
-
-	for (i = 0; i < num_active; i++)
-		fprintf (fp, "%s %lu\n", active[i].name, (unsigned long int) active[i].last_updated_time);
-
-	fclose (fp);
-}
-#endif /* INDEX_DAEMON */
