@@ -4,11 +4,33 @@
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
  *  Updated   : 1997-03-19
- *  Copyright : (c) Copyright 1997-99 by Urs Janssen
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made.
+ *
+ * Copyright (c) 1997-2002 Urs Janssen <urs@tin.org>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifndef TIN_H
@@ -18,10 +40,8 @@
 #	include "tnntp.h"
 #endif /* !TNNTP_H */
 
-#ifndef INDEX_DAEMON
-	static const char * get_full_name (void);
-	static const char * get_user_name (void);
-#endif /* !INDEX_DAEMON */
+static const char * get_full_name (void);
+static const char * get_user_name (void);
 
 
 /* find hostname */
@@ -30,44 +50,40 @@ get_host_name (
 	void)
 {
 	char *ptr;
-	static char hostname[MAXHOSTNAMELEN+1]; /* need space for '\0' */
+	static char hostname[MAXHOSTNAMELEN + 1]; /* need space for '\0' */
 
-	hostname[0]='\0';
+	hostname[0] = '\0';
 
-#ifdef HAVE_GETHOSTBYNAME
+#ifdef HAVE_GETHOSTNAME
 	gethostname(hostname, sizeof(hostname));
 #else
-#	if defined(M_AMIGA) || defined(M_OS2)
+#	if defined(M_AMIGA)
 	if ((ptr = getenv("NodeName")) != (char *) 0)
 		strncpy(hostname, ptr, MAXHOSTNAMELEN);
-#	else
-#		if defined(WIN32)
-	if ((ptr = getenv("COMPUTERNAME")) != (char *) 0)
-		strncpy(hostname, ptr, MAXHOSTNAMELEN);
-#		endif /* WIN32 */
-#	endif /* M_AMIGA || M_OS2 */
-#endif /* HAVE_GETHOSTBYNAME */
+#	endif /* M_AMIGA */
+#endif /* HAVE_GETHOSTNAME */
 #ifdef HAVE_SYS_UTSNAME_H
-	if (! *hostname)
+	if (!*hostname)
 		strcpy(hostname, system_info.nodename);
 #endif /* HAVE_SYS_UTSNAME_H */
-	if (! *hostname) {
+	if (!*hostname) {
 		if ((ptr = getenv("HOST")) != (char *) 0)
 			strncpy (hostname, ptr, MAXHOSTNAMELEN);
 		else {
 			if ((ptr = getenv("HOSTNAME")) != (char *) 0)
 				strncpy (hostname, ptr, MAXHOSTNAMELEN);
 			else
-				hostname[0]='\0';
+				hostname[0] = '\0';
 		}
 	}
-	hostname[MAXHOSTNAMELEN]='\0';
-	return (hostname);
+	hostname[MAXHOSTNAMELEN] = '\0';
+	return hostname;
 }
 
 
 #ifdef DOMAIN_NAME
 /* find domainname - check DOMAIN_NAME */
+/* TODO: check /etc/defaultdomain as a last resort? */
 const char *
 get_domain_name (
 	void)
@@ -77,27 +93,28 @@ get_domain_name (
 	FILE *fp;
 	static char domain[8192];
 
-#	if defined(M_AMIGA)
+#	ifdef M_AMIGA
 /*
  * Damn compiler bugs...
  * Without this hack, SASC 6.55 produces a TST.B d16(pc),
  * which is illegal on a 68000
  */
 static const char *domain_name_hack = DOMAIN_NAME;
-#		undef	DOMAIN_NAME
+#		undef DOMAIN_NAME
 #		define DOMAIN_NAME domain_name_hack
 #	endif /* M_AMIGA */
 
-	domain[0]='\0';
+	domain[0] = '\0';
 
 	if (strlen(DOMAIN_NAME))
 		strcpy(domain, DOMAIN_NAME);
 
-#	if defined(M_AMIGA)
-	if (strchr(domain, ':')) { /* absolute AmigaOS paths contain one, RFC-hostnames don't */
+#	ifdef M_AMIGA
+	if (strchr(domain, ':')) /* absolute AmigaOS paths contain one, RFC-hostnames don't */
 #	else
-	if (domain[0] == '/' && domain[1]) {
+	if (domain[0] == '/' && domain[1])
 #	endif /* M_AMIGA */
+	{
 		/* If 1st letter is '/' read domainname from specified file */
 		if ((fp = fopen (domain, "r")) != (FILE *) 0) {
 			while (fgets (buff, (int) sizeof (buff), fp) != (char *) 0) {
@@ -110,48 +127,51 @@ static const char *domain_name_hack = DOMAIN_NAME;
 				}
 			}
 			if (domain[0] == '/') /* file was empty */
-				domain[0]='\0';
+				domain[0] = '\0';
 
 			fclose (fp);
 		} else
-			domain[0]='\0';
+			domain[0] = '\0';
 	}
-	domain[MAXHOSTNAMELEN]='\0';
-	return (domain);
+	domain[MAXHOSTNAMELEN] = '\0';
+	return domain;
 }
 #endif /* DOMAIN_NAME */
 
 
 #ifdef HAVE_GETHOSTBYNAME
 #	define MAXLINELEN   1024
+#	define WS	" \f\t\v"
 /* find FQDN - gethostbyaddr() */
 const char *
 get_fqdn (
 	const char *host)
 {
-	FILE *inf;
-	char *cp, *domain;
-	char line[MAXLINELEN+1];
-	char name[MAXHOSTNAMELEN+2];
+	char *domain;
+	char line[MAXLINELEN + 1];
+	char name[MAXHOSTNAMELEN + 2];
 	static char fqdn[1024];
 	struct hostent *hp;
 	struct in_addr in;
 
 	*fqdn = '\0';
 	domain = NULL;
-
 	name[MAXHOSTNAMELEN] = '\0';
+
 	if (host) {
 		if (strchr(host, '.'))
 			return host;
 		(void) strncpy(name, host, MAXHOSTNAMELEN);
-	} else
+	} else {
+#	ifdef HAVE_GETHOSTNAME
 		if (gethostname(name, MAXHOSTNAMELEN))
-			return(NULL);
+#	endif /* HAVE_GETHOSTNAME */
+			return NULL;
+	}
 
 	if ('0' <= *name && *name <= '9') {
 		in.s_addr = inet_addr(name);
-		if ((hp = gethostbyaddr((char *)&in.s_addr, 4, AF_INET)))
+		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
 			in.s_addr = (*hp->h_addr);
 		return(hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
 	}
@@ -159,37 +179,46 @@ get_fqdn (
 		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
 			in.s_addr = (*hp->h_addr);
 
-	sprintf(fqdn,"%s", hp
+	sprintf(fqdn, "%s", hp
 		? strchr(hp->h_name, '.')
 			? hp->h_name
 			: inet_ntoa(in)
 		: "");
-	if (!*fqdn || (fqdn[strlen(fqdn)-1] <= '9')) {
+	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
+		FILE *inf;
+
 		*fqdn = '\0';
-		inf = fopen("/etc/resolv.conf", "r");
-		if (inf) {
+
+		if ((inf = fopen("/etc/resolv.conf", "r")) != NULL) {
+			char *eos;
+
 			while(fgets(line, MAXLINELEN, inf)) {
+
+				if (line[0] == '#' || line[0] == '\n')
+					continue;
+
 				line[MAXLINELEN] = '\0';
-				(void) str_trim(line);
-				if (strncmp(line,"domain ", 7) == 0) {
-					domain = line + 7;
-					break;
-				}
-				if (strncmp(line, "search ", 7) == 0) {
-					domain = line + 7;
-					cp = strchr(domain, ' ');
-					if (cp)
-						*cp = '\0';
-					break;
+
+				if ((eos = strpbrk(line, WS)) != NULL) {
+					int j = eos - line;
+
+					if (j) {
+						if ((strncmp(line, "domain", j) == 0) || (strncmp(line, "search", j) == 0)) {
+							domain = strtok(eos, WS);
+							break;
+						}
+					}
 				}
 			}
-			if (domain)
+			if (domain) {
+				strip_line(domain);
 				sprintf(fqdn, "%s.%s", name, domain);
+			}
+
 			fclose(inf);
 		}
 	}
-
-	return(fqdn);
+	return fqdn;
 }
 #endif /* HAVE_GETHOSTBYNAME */
 
@@ -197,7 +226,6 @@ get_fqdn (
 /*
  * Find username & fullname
  */
-#ifndef INDEX_DAEMON
 void
 get_user_info (
 	char *user_name,
@@ -213,10 +241,8 @@ get_user_info (
 	if ((ptr = get_user_name()))
 		strcpy(user_name, ptr);
 }
-#endif /* !INDEX_DAEMON */
 
 
-#ifndef INDEX_DAEMON
 static const char *
 get_user_name (
 	void)
@@ -230,62 +256,68 @@ get_user_name (
 	username[0] = '\0';
 #	if defined (M_AMIGA) || defined (VMS)
 	if ((p = getenv ("USER"))) {
-		STRCPY (username, p);
+		STRCPY(username, p);
 #		ifdef VMS
 		lower (username);
 #		endif /* VMS */
 	}
 #	else
 	pw = getpwuid (getuid ());
-	strcpy (username, pw->pw_name);
+
+	if (pw != (struct passwd *) 0)
+		strcpy (username, pw->pw_name);
+	else {
+		error_message (_(txt_error_passwd_missing));
+		tin_done (EXIT_FAILURE);
+	}
 #	endif /* M_AMIGA || VMS */
-	return(username);
+	return username;
 }
-#endif /* !INDEX_DAEMON */
 
 
-#ifndef INDEX_DAEMON
 static const char *
 get_full_name (
 	void)
 {
 	char *p;
+	static char fullname[128];
+#	if !defined(VMS) && !defined(DONT_HAVE_PW_GECOS)
 	char buf[128];
 	char tmp[128];
-	static char fullname[128];
 	struct passwd *pw;
+#	endif /* !VMS && !DONT_HAVE_PW_GECOS */
 
 	fullname[0] = '\0';
 
 	if ((p = getenv ("NAME")) != (char *) 0) {
 		strncpy (fullname, p, sizeof (fullname));
-		return (fullname);
+		return fullname;
 	}
-	if ((p =  getenv ("REALNAME")) != (char *) 0) {
+	if ((p = getenv ("REALNAME")) != (char *) 0) {
 		strncpy (fullname, p, sizeof (fullname));
-		return (fullname);
+		return fullname;
 	}
 
 #	ifdef VMS
-	strncpy (fullname, fix_fullname(get_uaf_fullname()), sizeof (fullname));
+	STRCPY(fullname, fix_fullname(get_uaf_fullname()));
 #	else
+#		ifndef DONT_HAVE_PW_GECOS
 	pw = getpwuid (getuid ());
-	strncpy (buf, pw->pw_gecos, sizeof (fullname));
+	STRCPY(buf, pw->pw_gecos);
 	if ((p = strchr (buf, ',')))
 		*p = '\0';
 	if ((p = strchr (buf, '&'))) {
 		*p++ = '\0';
-		strcpy (tmp, pw->pw_name);
-		/* strcpy(tmp, get_user_name()); */
-		if (*tmp && *tmp >= 'a' && *tmp <= 'z')
-			*tmp = *tmp - 32;
-		sprintf (fullname, "%s%s%s", buf, tmp, p);
+		STRCPY(tmp, pw->pw_name);
+		if (*tmp && isalpha((int) *tmp) && islower((int) *tmp))
+			*tmp = toupper((int) *tmp);
+		snprintf (fullname, sizeof(fullname) -1, "%s%s%s", buf, tmp, p);
 	} else
-		strcpy (fullname, buf);
+		STRCPY(fullname, buf);
+#		endif /* !DONT_HAVE_PW_GECOS */
 #	endif /* VMS */
-	return (fullname);
+	return fullname;
 }
-#endif /* !INDEX_DAEMON */
 
 
 /*
@@ -295,7 +327,6 @@ get_full_name (
 /*
  * build From: in 'name <user@host.doma.in>' format
  */
-#ifndef INDEX_DAEMON
 void
 get_from_name (
 	char *from_name,
@@ -310,31 +341,26 @@ get_from_name (
 	if (!(fromhost && *fromhost))
 		fromhost = domain_name;
 
-	if (thisgrp && (thisgrp->attribute->from != (char *) 0)) {
+	if (thisgrp && *thisgrp->attribute->from != '\0') {
 		strcpy(from_name, thisgrp->attribute->from);
 		return;
 	}
 
-	if (*tinrc.mail_address) {
-		strcpy(from_name, tinrc.mail_address);
-		return;
-	}
+	sprintf (from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), get_full_name(), get_user_name(), fromhost);
 
-	sprintf (from_name, ((strchr(get_full_name(), '.')) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), get_full_name(), get_user_name(), fromhost);
 #	ifdef DEBUG
 	if (debug == 2)
 		error_message ("FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, get_user_name(), domain_name, get_full_name());
 #	endif /* DEBUG */
 
 }
-#endif /* !INDEX_DAEMON */
 
 
 /*
  * build_sender()
  * returns *(Full_Name <user@fq.domainna.me>)
  */
-#if !defined(INDEX_DAEMON) && !defined(FORGERY)
+#ifndef FORGERY
 char *
 build_sender (
 	void)
@@ -367,4 +393,4 @@ build_sender (
 
 	return sender;
 }
-#endif /* !INDEX_DAEMON && !FORGERY */
+#endif /* !FORGERY */
