@@ -5,11 +5,36 @@
  *  Created   : 1992-03-17
  *  Updated   : 1997-12-31
  *  Notes     : NNTP builtin version of inews
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 
@@ -25,7 +50,7 @@
  * local prototypes
  */
 #ifdef NNTP_INEWS
-	static t_bool submit_inews (char *name);
+	static t_bool submit_inews (char *name, char *a_message_id);
 #endif /* NNTP_INEWS */
 
 #if defined(NNTP_INEWS) && !defined(FORGERY)
@@ -43,7 +68,7 @@
 #		endif /* MULTINET */
 #	else
 #		ifdef HAVE_NETDB_H
-#			include	<netdb.h>
+#			include <netdb.h>
 #		endif /* HAVE_NETDB_H */
 #	endif /* VMS */
 
@@ -55,7 +80,7 @@
 #	endif /* HAVE_NETINET_IN_H */
 #endif /* 0 */
 
-#define	PATHMASTER	"not-for-mail"
+#define PATHMASTER	"not-for-mail"
 
 /*
  * Submit an article using the NNTP POST command
@@ -63,13 +88,14 @@
 #ifdef NNTP_INEWS
 static t_bool
 submit_inews (
-	char *name)
+	char *name,
+	char *a_message_id)
 {
 	FILE *fp;
-	char *ptr;
+	char *ptr, *ptr2;
 	char from_name[HEADER_LEN];
 	char message_id[HEADER_LEN];
-	char line[NNTP_STRLEN];
+	char line[HEADER_LEN];
 	int auth_error = 0;
 	int respcode;
 	t_bool leave_loop = FALSE;
@@ -82,7 +108,7 @@ submit_inews (
 #	endif /* !FORGERY */
 
 	if ((fp = fopen (name, "r")) == (FILE *) 0) {
-		perror_message (txt_cannot_open, name);
+		perror_message (_(txt_cannot_open), name);
 		return ret_code;
 	}
 
@@ -98,7 +124,7 @@ submit_inews (
 					*ptr = '\0';
 			}
 			if (ptr - line == 10 && !strncasecmp (line, "Message-ID", 10)) {
-				strcpy(message_id, ptr+2);
+				strcpy(message_id, ptr + 2);
 				id_in_article = TRUE;
 				if ((ptr = strchr(message_id, '\n')))
 					*ptr = '\0';
@@ -109,22 +135,26 @@ submit_inews (
 
 	if ((from_name[0] == '\0') || (from_name[6] == '\0')) {
 		/* we could silently add a From: line here if we want to... */
-		error_message (txt_error_no_from);
+		error_message (_(txt_error_no_from));
 		fclose (fp);
 		return ret_code;
 	}
-	/*
-	 * check for valid From: line
-	 *
-	 * this will be done once again in sender_needed!?!
-	 */
+
 #ifndef FORGERY
-	if (GNKSA_OK != gnksa_check_from(rfc1522_encode(from_name, FALSE) + 6)) { /* error in address */
-		error_message (txt_invalid_from, from_name);
+	/*
+	 * we should only skip the gnksa_check_from() test if we are going to post
+	 * a forged cancel, but inews.c doesn't know anything about the message
+	 * type, so we skip the test if FORGERY is set
+	 *
+	 * check for valid From: line
+	 */
+	if (GNKSA_OK != gnksa_check_from(from_name + 6)) { /* error in address */
+		error_message (_(txt_invalid_from), from_name + 6);
 		fclose (fp);
 		return ret_code;
 	}
 #endif /* !FORGERY */
+
 	do {
 		rewind(fp);
 
@@ -134,21 +164,25 @@ submit_inews (
 			sender = sender_needed (from_name + 6, ptr);
 			switch (sender) {
 				case -2: /* can't build Sender: */
-					error_message (txt_invalid_sender, ptr);
+					error_message (_(txt_invalid_sender), ptr);
 					fclose (fp);
 					return ret_code;
 					/* NOTREACHED */
 					break;
 
-				case -1: /* illegal From: (can't happen as check is done above allready) */
-					error_message (txt_invalid_from, from_name + 6);
+				case -1: /* illegal From: (can't happen as check is done above already) */
+					error_message (_(txt_invalid_from), from_name + 6);
 					fclose (fp);
 					return ret_code;
 					/* NOTREACHED */
 					break;
 
 				case 1:	/* insert Sender */
-					sprintf (sender_hdr, "Sender: %s", rfc1522_encode(ptr, ismail));
+					snprintf (sender_hdr, sizeof(sender_hdr), "Sender: %s", ptr);
+#		if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
+					buffer_to_network(sender_hdr);
+#		endif /* LOCAL_CHARSET || MAC_OS_X || CHARSET_CONVERSION */
+					STRCPY (sender_hdr, rfc1522_encode (sender_hdr, ismail));
 					break;
 
 				case 0: /* no sender needed */
@@ -174,7 +208,6 @@ submit_inews (
 		 * if it's present: use it.
 		 */
 		if (message_id[0] == '\0') {
-			char * ptr2;
 
 			/* simple syntax check - locate last '<' */
 			if ((ptr = strrchr (line, '<')) != (char *) 0) {
@@ -255,16 +288,6 @@ submit_inews (
 		 * of the put_server(".") above a "." would be resent as the last
 		 * "command".
 		 */
-		/*
-		 * here we could add a check if the server returns the
-		 * Message-ID in the response string...
-		 * if it does so, compare it with message_id (if set)
-		 * and pass it to update_posted_info_file() and
-		 * quick_filter_select_posted_art() if tinrc.add_posted_to_filter
-		 * is set
-		 * make sure, that quick_filter_select_posted_art() and
-		 * update_posted_info_file() arn't called twice!
-		 */
 		respcode = get_only_respcode (line);
 		leave_loop = TRUE;
 
@@ -291,6 +314,35 @@ submit_inews (
 		return ret_code;
 	}
 
+	/*
+	 * scan line if it contains a Message-ID
+	 */
+	{
+		/* simple syntax check - locate last '<' */
+		if ((ptr = strrchr (line, '<')) != (char *) 0) {
+			/* search next '>' */
+			if ((ptr2 = strchr (ptr, '>')) != (char *) 0) {
+				/* terminate string */
+				*++ptr2 = '\0';
+				/* check for @ and no whitespaces */
+				if ((strchr(ptr, '@') != (char *) 0) && (strpbrk(ptr, " \t") == (char *) 0))
+					/* copy Message-ID */
+					strcpy(a_message_id, ptr);
+			}
+		}
+
+#if 0
+		if (*message_id && *a_message_id) { /* check if returned ID matches purposed ID */
+			if (strcmp(message_id, a_message_id)) {
+				; /* shouldn't happen - warn user? */
+			}
+		}
+#endif /* 0 */
+
+		if (*message_id && (id_in_article || !*a_message_id))
+			strcpy(a_message_id, message_id);
+	}
+
 	ret_code = TRUE;
 
 	return ret_code;
@@ -302,39 +354,63 @@ submit_inews (
  */
 t_bool
 submit_news_file (
-	char *name)
+	char *name,
+	char *a_message_id)
 {
 	char buf[LEN];
 	char *cp = buf;
 	t_bool ret_code;
 	t_bool ismail = FALSE;
 
+	a_message_id[0] = '\0';
+
 	checknadd_headers (name);
 
 	/* 7bit ISO-2022-KR is NEVER to be used in Korean news posting. */
-	if (!(strcasecmp(tinrc.mm_charset, "euc-kr") || strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)))
+#ifdef CHARSET_CONVERSION
+	if (!(strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR") || strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)))
+#else
+	if (!(strcasecmp(tinrc.mm_charset, "EUC-KR") || strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)))
+#endif /* CHARSET_CONVERSION */
 		tinrc.post_mime_encoding = 0;	/* FIXME: txt_8bit */
 
 	rfc15211522_encode(name, txt_mime_encodings[tinrc.post_mime_encoding], tinrc.post_8bit_header, ismail);
 
 #ifdef NNTP_INEWS
-	if (read_news_via_nntp && !read_saved_news && tinrc.use_builtin_inews) {
-		ret_code = submit_inews (name);
+	if (read_news_via_nntp && !read_saved_news && 0 == strcasecmp(tinrc.inews_prog, "--internal")) {
+		ret_code = submit_inews (name, a_message_id);
 	} else
 #endif /* NNTP_INEWS */
 		{
 #ifdef M_UNIX
+		/* use 'inewsdir/inews -h' or tinrc.inews_prog or 'inews -h' */
 		if (*inewsdir) {
 			strcpy (buf, inewsdir);
-			strcat (buf, "/");
-			cp += strlen (cp);
-		}
-		sh_format (cp, sizeof(buf) - (cp - buf), "inews -h < %s", name);
+			strcat (buf, "/inews -h");
+		} else if (0 != strcasecmp(tinrc.inews_prog, "--internal"))
+			strcpy (buf, tinrc.inews_prog);
+		else
+			strcpy (buf, "inews -h");
+
+		cp += strlen (cp);
+
+		sh_format (cp, sizeof(buf) - (cp - buf), " < %s", name);
 #else
 		make_post_cmd (cp, name);
 #endif /* M_UNIX */
 
 		ret_code = invoke_cmd (buf);
+#ifdef NNTP_INEWS
+		if (!ret_code && read_news_via_nntp && !read_saved_news && 0 != strcasecmp(tinrc.inews_prog, "--internal")) {
+			if (prompt_yn(cLINES, _(txt_post_via_builtin_inews), TRUE)) {
+				ret_code = submit_inews (name, a_message_id);
+				if (ret_code) {
+					if (prompt_yn(cLINES, _(txt_post_via_builtin_inews_only), TRUE) == 1)
+						strcpy(tinrc.inews_prog,"--internal");
+				}
+			}
+		}
+#endif /* NNTP_INEWS */
 	}
 
 	return ret_code;
@@ -351,7 +427,8 @@ submit_news_file (
  *               -2 = error (no '.' and/or '@' in Sender)
  */
 #if defined(NNTP_INEWS) && !defined(FORGERY)
-static int sender_needed (
+static int
+sender_needed (
 	char *from,
 	char *sender)
 {
@@ -359,27 +436,23 @@ static int sender_needed (
 	char *sender_at_pos;
 	char *sender_dot_pos;
 	char from_addr[HEADER_LEN];
-	char from_line[HEADER_LEN];
 	char from_name[HEADER_LEN];
 	char sender_addr[HEADER_LEN];
 	char sender_line[HEADER_LEN];
 	char sender_name[HEADER_LEN];
 
-#ifdef DEBUG
-	if (debug == 2)
-		wait_message (0, "sender_needed From:=[%s]", from);
-#endif /* DEBUG */
+#	ifdef DEBUG
+	if (debug == 2) {
+		wait_message (3, "sender_needed From:=[%s]", from);
+		wait_message (3, "sender_needed Sender:=[%s]", sender);
+	}
+#	endif /* DEBUG */
 
-	sprintf (from_line, "From: %s", from);
-	if (GNKSA_OK != gnksa_do_check_from(rfc1522_encode(from_line, FALSE) + 6, from_addr, from_name))
-		return -1;
+	/* split From: line into address & comment */
 
-#ifdef DEBUG
-	if (debug == 2)
-		wait_message (0, "sender_needed Sender:=[%s]", sender);
-#endif /* DEBUG */
+	gnksa_do_check_from(from, from_addr, from_name);
 
-	sprintf (sender_line, "Sender: %s", sender);
+	snprintf (sender_line, sizeof(sender_line), "Sender: %s", sender);
 	if (GNKSA_OK != gnksa_do_check_from(rfc1522_encode(sender_line, FALSE) + 8, sender_addr, sender_name))
 		return -2;
 
