@@ -3,15 +3,41 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-28
+ *  Updated   : 2001-11-10
  *  Notes     :
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -22,13 +48,13 @@
 #	include "trace.h"
 #endif /* !included_trace_h */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 #ifndef VERSION_H
-#	include  "version.h"
+#	include "version.h"
 #endif /* !VERSION_H */
 #ifndef BUGREP_H
-#	include	"bugrep.h"
+#	include "bugrep.h"
 #endif /* !BUGREP_H */
 
 /*
@@ -42,8 +68,7 @@ static int read_site_config(void);
 
 
 char active_times_file[PATH_LEN];
-char article[PATH_LEN];			/* ~/.article file */
-char bug_addr[LEN];			/* address to add send bug reports to */
+char article[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
 char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
 char bug_nntpserver2[PATH_LEN];		/* welcome message of NNTP server used */
 char cmdline_nntpserver[PATH_LEN];
@@ -55,7 +80,6 @@ char default_signature[PATH_LEN];
 char domain_name[MAXHOSTNAMELEN];
 char global_attributes_file[PATH_LEN];
 char global_config_file[PATH_LEN];
-char global_filter_file[PATH_LEN];
 char homedir[PATH_LEN];
 char host_name[MAXHOSTNAMELEN];
 char index_maildir[PATH_LEN];
@@ -65,7 +89,7 @@ char inewsdir[PATH_LEN];
 char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
 char local_attributes_file[PATH_LEN];
 char local_config_file[PATH_LEN];
-char local_filter_file[PATH_LEN];
+char filter_file[PATH_LEN];
 char local_input_history_file[PATH_LEN];
 char local_newsgroups_file[PATH_LEN];	/* local copy of NNTP newsgroups file */
 char local_newsrctable_file[PATH_LEN];
@@ -75,8 +99,6 @@ char mail_news_user[LEN];		/* mail new news to this user address */
 char mailbox[PATH_LEN];			/* system mailbox for each user */
 char mailer[PATH_LEN];			/* mail program */
 char mailgroups_file[PATH_LEN];
-char msg_headers_file[PATH_LEN];	/* $HOME/.tin/headers */
-char my_distribution[LEN];		/* Distribution: */
 char **news_headers_to_display_array;	/* array of which headers to display */
 char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char newnewsrc[PATH_LEN];
@@ -91,38 +113,27 @@ char posted_msgs_file[PATH_LEN];
 char postponed_articles_file[PATH_LEN];	/* ~/.tin/postponed.articles file */
 char tin_progname[PATH_LEN];		/* program name */
 char rcdir[PATH_LEN];
-char reply_to[LEN];			/* Reply-To: address */
 char save_active_file[PATH_LEN];
 char spooldir[PATH_LEN];		/* directory where news is */
 char subscriptions_file[PATH_LEN];
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
 
-#ifdef INDEX_DAEMON
-	char group_times_file[PATH_LEN];
-#endif /* INDEX_DAEMON */
+char proc_ch_default;			/* set in change_config_file () */
 
 #ifdef VMS
 	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
 #endif /* VMS */
 
-#ifdef M_OS2
-	char TMPDIR[PATH_LEN];
-#endif /* M_OS2 */
-
-int NOTESLINES;				/* set in set_win_size () */
 int groupname_len = 0;			/* 'runtime' copy of groupname_max_len */
-int hist_last[HIST_MAXNUM+1];
-int hist_pos[HIST_MAXNUM+1];
+int hist_last[HIST_MAXNUM + 1];
+int hist_pos[HIST_MAXNUM + 1];
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int num_headers_to_display;		/* num headers to display -- swp */
 int num_headers_to_not_display;		/* num headers to not display -- swp */
 int num_of_killed_arts;
 int num_of_selected_arts;		/* num articles marked 'hot' */
-int num_of_tagged_arts;
 int system_status;
-int top = 0;
-int top_base;				/* Highest numbered thread */
 int xmouse, xrow, xcol;			/* xterm button pressing information */
 
 #ifdef HAVE_COLOR
@@ -130,46 +141,28 @@ int xmouse, xrow, xcol;			/* xterm button pressing information */
 	t_bool word_highlight;		/* word highlighting on/off */
 #endif /* HAVE_COLOR */
 
-pid_t process_id;
-uid_t real_uid;
-gid_t real_gid;
-gid_t tin_gid;
-uid_t tin_uid;
+pid_t process_id;			/* Useful to have around for .suffixes */
 mode_t real_umask;
 
 t_bool no_write = FALSE;		/* do not write newsrc on quit (-X cmd-line flag) */
-t_bool catchup = FALSE;			/* mark all arts read in all subscribed groups */
-t_bool check_any_unread = FALSE;
 t_bool check_for_new_newsgroups;	/* don't check for new newsgroups */
 t_bool cmd_line;			/* batch / interactive mode */
 t_bool created_rcdir;			/* checks if first time tin is started */
 t_bool dangerous_signal_exit;		/* no get_respcode() in nntp_command when dangerous signal exit */
-#ifdef INDEX_DAEMON
-	t_bool delete_index_file;		/* delete index file before indexing (tind only) */
-#endif /* INDEX_DAEMON */
 t_bool disable_gnksa_domain_check;	/* disable checking TLD in From: etc. */
 t_bool disable_sender;			/* disable generation of Sender: header */
-t_bool global_filtered_articles;	/* globally killed / auto-selected articles */
 t_bool got_sig_pipe = FALSE;
-t_bool in_headers;			/* color in headers */
-t_bool local_filtered_articles;		/* locally killed / auto-selected articles */
-t_bool local_index;			/* do private indexing? */
-t_bool mail_news;			/* mail all arts to specified user */
+t_bool filtered_articles;		/* locally killed / auto-selected articles */
 t_bool list_active;
 t_bool newsrc_active;
 t_bool post_article_and_exit;		/* quick post of an article then exit (elm like) */
 t_bool post_postponed_and_exit;		/* post postponed articles and exit */
-t_bool purge_index_files;		/* stat all articles to see if they still exist */
 t_bool reread_active_for_posted_arts;
 t_bool read_local_newsgroups_file;	/* read newsgroups file locally or via NNTP */
 t_bool read_news_via_nntp = FALSE;	/* read news locally or via NNTP */
 t_bool read_saved_news = FALSE;		/* tin -R read saved news from tin -S */
-t_bool save_news;			/* save all arts to savedir structure */
 t_bool show_description = TRUE;		/* current copy of tinrc flag */
-t_bool start_any_unread = FALSE;
-t_bool tex2iso_supported;		/* Convert german style TeX to ISO-Latin1 */
 t_bool batch_mode;			/* update index files only mode */
-t_bool update_fork = FALSE;		/* update index files by forked tin -u */
 t_bool verbose = FALSE;			/* update index files only mode */
 t_bool (*wildcard_func) (const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
 t_bool xover_supported = FALSE;
@@ -181,8 +174,11 @@ t_bool xref_supported = TRUE;
 	t_bool force_auth_on_conn_open = FALSE;	/* authenticate on connection startup */
 #endif /* NNTP_ABLE */
 
+/* Currently active menu parameters */
+t_menu *currmenu;
+
 /* History entries */
-char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
+char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 
 #ifdef HAVE_SYS_UTSNAME_H
 	struct utsname system_info;
@@ -193,7 +189,11 @@ char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 	static struct passwd pwdentry;
 #endif /* !M_AMIGA */
 
-	struct regex_cache strip_re_regex, strip_was_regex
+struct regex_cache
+					strip_re_regex, strip_was_regex,
+					uubegin_regex, uubody_regex,
+					url_regex, mail_regex, news_regex,
+					shar_regex
 #ifdef HAVE_COLOR
 		, quote_regex, quote_regex2, quote_regex3
 #endif /* HAVE_COLOR */
@@ -207,7 +207,11 @@ struct t_config tinrc = {
 	MARK_INRANGE,			/* art_marked_inrange */
 	ART_MARK_RETURN,		/* art_marked_return */
 	ART_MARK_SELECTED,		/* art_marked_selected */
+	ART_MARK_RECENT,		/* art_marked_recent */
 	ART_MARK_UNREAD,		/* art_marked_unread */
+	ART_MARK_READ,			/* art_marked_read */
+	ART_MARK_KILLED,		/* art_marked_killed */
+	ART_MARK_READ_SELECTED,		/* art_marked_read_selected */
 	"",		/* editor_format */
 	"",		/* default_goto_group */
 	"",		/* default_mail_address */
@@ -235,16 +239,24 @@ struct t_config tinrc = {
 	"",		/* default_shell_command */
 	"In article %M you wrote:",		/* mail_quote_format */
 	"",		/* maildir */
+	0,			/* mailbox_format */
 	"",		/* mail_address */
-	"",		/* mm_charset */
-	"Newsgroups Followup-To Summary Keywords",		/* news_headers_to_display */
+#ifndef CHARSET_CONVERSION
+	"",		/* mm_charset, defaults to $MM_CHARSET */
+#else
+	0,			/* mm_network_charset, defaults to US-ASCII */
+#endif /* !CHARSET_CONVERSION */
+	"", 		/* mm_local_charset, display charset */
+#ifdef HAVE_ICONV_OPEN_TRANSLIT
+	FALSE,	/* translit */
+#endif /* HAVE_ICONV_OPEN_TRANSLIT */
+	"Newsgroups Followup-To Summary Keywords X-Comment-To",		/* news_headers_to_display */
 	"",		/* news_headers_to_not_display */
 	"%F wrote:",		/* news_quote_format */
-	"",					/* post_process_command */
 	DEFAULT_COMMENT,	/* quote_chars */
 #ifdef HAVE_COLOR
 	"",		/* quote_regex */
-	"",		/* quote_regex 2nd level*/
+	"",		/* quote_regex 2nd level */
 	"",		/* quote_regex >= 3rd level */
 #endif /* HAVE_COLOR */
 	"",		/* sigfile */
@@ -259,6 +271,7 @@ struct t_config tinrc = {
 	0,		/* default_move_group */
 	iKeySaveAppendFile,		/* default_save_mode */
 	0,		/* getart_limit */
+	2,		/* recent_time */
 	32,		/* groupname_max_length */
 	KILL_READ,		/* kill_level */
 	MIME_ENCODING_7BIT,		/* mail_mime_encoding */
@@ -266,7 +279,8 @@ struct t_config tinrc = {
 	POST_PROC_NONE,			/* post_process */
 	REREAD_ACTIVE_FILE_SECS,	/* reread_active_file_secs */
 	SHOW_FROM_NAME,				/* show_author */
-	SORT_BY_DATE_ASCEND,		/* sort_article_type */
+	SORT_ARTICLES_BY_DATE_ASCEND,		/* sort_article_type */
+	SORT_THREADS_BY_SCORE_DESCEND,		/* sort_threads_type */
 	BOGUS_ASK,		/* strip_bogus */
 	THREAD_MAX,		/* thread_articles */
 	0,		/* Default to wildmat, not regex */
@@ -303,14 +317,12 @@ struct t_config tinrc = {
 	TRUE,		/* auto_list_thread */
 	FALSE,		/* auto_reconnect */
 	FALSE,		/* auto_save */
-	FALSE,		/* batch_save */
+	TRUE,		/* batch_save */
 	TRUE,		/* beginner_level */
 	FALSE,		/* cache_overview_files */
 	FALSE,		/* catchup_read_groups */
 	TRUE,		/* confirm_action */
 	TRUE,		/* confirm_to_quit */
-	TRUE,		/* display_mime_allheader_asis */
-	FALSE,		/* display_mime_header_asis */
 #ifdef USE_INVERSE_HACK
 	TRUE,		/* draw_arrow */
 #else
@@ -327,23 +339,20 @@ struct t_config tinrc = {
 #endif /* USE_INVERSE_HACK */
 	TRUE,		/* keep_dead_articles */
 	TRUE,		/* keep_posted_articles */
+	POSTED_FILE,	/* keep_posted_articles_file */
 	FALSE,		/* mail_8bit_header */
 	TRUE,		/* mark_saved_read */
 	TRUE,		/* pgdn_goto_next */
 	TRUE,		/* pos_first_unread */
 	FALSE,		/* post_8bit_header */
+	TRUE,		/* post_process_view */
 #ifndef DISABLE_PRINTING
 	FALSE,		/* print_header */
 #endif /* !DISABLE_PRINTING */
 	FALSE,		/* process_only_unread */
 	FALSE,		/* prompt_followupto */
-	FALSE,		/* quote_empty_lines */
-	TRUE,		/* quote_signatures */
-#ifdef HAVE_MMDF_MAILER
-	TRUE,		/* save_to_mmdf_mailbox */
-#else
-	FALSE,		/* save_to_mmdf_mailbox */
-#endif /* HAVE_MMDF_MAILER */
+	TRUE,		/* quote_empty_lines */
+	FALSE,		/* quote_signatures */
 	TRUE,		/* show_description */
 	FALSE,		/* show_last_line_prev_page */
 	TRUE,		/* show_lines */
@@ -351,7 +360,7 @@ struct t_config tinrc = {
 	FALSE,		/* show_only_unread_groups */
 	FALSE,		/* show_score */
 	TRUE,		/* show_signatures */
-	FALSE,		/* show_xcommentto */
+	FALSE,		/* hide_uue */
 	TRUE,		/* sigdashes */
 	TRUE,		/* signature_repost */
 	FALSE,		/* space_goto_next_unread */
@@ -364,9 +373,10 @@ struct t_config tinrc = {
 	FALSE,		/* strip_newsrc */
 	FALSE,		/* tab_after_X_selection */
 	TRUE,		/* tab_goto_next_unread */
+	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
-	TRUE,		/* use_builtin_inews */
+	"",		/* inews_prog */
 	FALSE,		/* use_getart_limit */
 	FALSE,		/* use_mailreader_i */
 	FALSE,		/* use_mouse */
@@ -374,8 +384,8 @@ struct t_config tinrc = {
 	FALSE,		/* use_keypad */
 #endif /* HAVE_KEYPAD */
 #ifdef HAVE_METAMAIL
-	TRUE,		/* ask_for_metamail */
-	TRUE,		/* use_metamail */
+	FALSE,		/* ask_for_metamail */
+	FALSE,		/* use_metamail */
 #endif /* HAVE_METAMAIL */
 	FALSE,		/* default_filter_kill_case */
 	FALSE,		/* default_filter_kill_expire */
@@ -447,7 +457,7 @@ postinit_colors (
 					break;
 			}
 		}
-		TRACE(("postinit_colors [%d] = %d", n, *(our_colors[n].colorp)))
+		TRACE(("postinit_colors [%d] = %d", n, *(our_colors[n].colorp)));
 	}
 }
 #endif /* HAVE_COLOR */
@@ -464,14 +474,13 @@ void
 init_selfinfo (
 	void)
 {
-	char nam[LEN];
 	char *ptr;
 	const char *cptr;
 	FILE *fp;
 	struct stat sb;
 
-	host_name[0]='\0';
-	domain_name[0]='\0';
+	host_name[0] = '\0';
+	domain_name[0] = '\0';
 
 #ifndef M_AMIGA
 #	ifdef HAVE_SYS_UTSNAME_H
@@ -501,72 +510,38 @@ init_selfinfo (
 
 	process_id = getpid ();
 
-#if defined(M_AMIGA) || defined(M_OS2)
-	tin_uid = tin_gid = 0;
-	real_uid = real_gid = (getenv ("TIND") ? 1 : 0);
-#else
-	tin_uid = geteuid ();
-	tin_gid = getegid ();
-	real_uid = getuid ();
-	real_gid = getgid ();
 	real_umask = umask (0);
 	(void) umask (real_umask);
-#endif /* M_AMIGA */
 
 #if defined(HAVE_SETLOCALE) && defined(LC_ALL) && !defined(NO_LOCALE)
-	setlocale (LC_ALL, "");
+	if (!setlocale(LC_ALL, "")) {
+		/*
+		 * TODO: issue a warning here like
+		 *       "Can't set the specified locale! Check $LANG, $LC_CTYPE, $LC_ALL"
+		 */
+		;
+	}
 #endif /* HAVE_SETLOCALE && LC_ALL && !NO_LOCALE */
 
 /* FIXME: move to get_user_name() [header.c] */
-#ifdef M_AMIGA
-	if ((ptr = getenv ("USERNAME")) != (char *) 0) {
-		my_strncpy (userid, ptr, sizeof (userid));
-	} else {
-		error_message (txt_env_var_not_found, "USERNAME");
-		tin_done (EXIT_FAILURE);
-	}
-	if ((ptr = getenv ("HOME")) != (char *) 0) {
-		my_strncpy (homedir, ptr, sizeof (homedir));
-	} else {
-		error_message (txt_env_var_not_found, "HOME");
-		tin_done (EXIT_FAILURE);
-	}
-#else
-	myentry = (struct passwd *) 0;
-
-	if (myentry == (struct passwd *) 0)
-		myentry = getpwuid (getuid ());
-
-	if (myentry != (struct passwd *) 0) {
+#ifndef M_AMIGA
+	if ((myentry = getpwuid (getuid ())) != (struct passwd *) 0) {
 		memcpy (&pwdentry, myentry, sizeof (struct passwd));
 		myentry = &pwdentry;
 	}
-
-/* we might add a check for $LOGNAME here */
-
-#	if defined(M_OS2) || defined(WIN32)
-	if (myentry == (struct passwd *) 0) {
-		fprintf (stderr, "Environment variable USER not set.\n");
-		giveup();
-	}
-#	else
-#		ifdef VMS
+#	ifdef VMS
+	/* TODO: get_user_name() entirely reles on $USER */
 	if (((ptr = getlogin ()) != (char *) 0) && strlen (ptr))
 		myentry = getpwnam (ptr);
-#		endif /* VMS */
-#	endif /* M_OS2 || WIN32 */
+#	endif /* VMS */
+
+	if (myentry != (struct passwd *) 0)
+		strcpy (userid, myentry->pw_name);
+	else {
+		error_message (_(txt_error_passwd_missing));
+		tin_done (EXIT_FAILURE);
+	}
 
-#	ifdef M_OS2
-	strcpy (TMPDIR, get_val ("TMP", "/tmp/"));
-	if ((TMPDIR[strlen(TMPDIR)-1] != '/') && (TMPDIR[strlen(TMPDIR)-1] != '\\'))
-		strcat(TMPDIR,"/");
-#	endif /* M_OS2 */
-#	ifdef WIN32
-	strcpy (TMPDIR, get_val ("TMP", "\\tmp\\"));
-	if (TMPDIR[strlen(TMPDIR)-1] != '\\')
-		strcat(TMPDIR,"\\");
-#	endif /* WIN32 */
-	strcpy (userid, myentry->pw_name);
 #	ifdef VMS
 	lower (userid);
 #	endif /* VMS */
@@ -579,30 +554,34 @@ init_selfinfo (
 		strcpy (homedir, "/tmp");
 	} else
 		my_strncpy (homedir, myentry->pw_dir, sizeof (homedir));
-#endif /* M_AMIGA */
 
-	/*
-	 * we're setuid, so index in $SPOOLDIR even if user root
-	 * This is quite essential if non local index files are
-	 * to be updated during the night from crontab by root.
-	 */
-	if (tin_uid != real_uid) {
-		local_index = FALSE;
-		set_real_uid_gid ();
-	} else {	/* index in users home directory ~/.tin/.news */
-		local_index = TRUE;
+#else
+	/* TODO: get_user_name() uses $USER, not $USERNAME */
+	if ((ptr = getenv ("USERNAME")) != (char *) 0) {
+		my_strncpy (userid, ptr, sizeof (userid));
+	} else {
+		error_message (_(txt_env_var_not_found), "USERNAME");
+		tin_done (EXIT_FAILURE);
 	}
+	/* TODO: why not also check for TIN_HOME? */
+	if ((ptr = getenv ("HOME")) != (char *) 0) {
+		my_strncpy (homedir, ptr, sizeof (homedir));
+	} else {
+		error_message (_(txt_env_var_not_found), "HOME");
+		tin_done (EXIT_FAILURE);
+	}
+#endif /* !M_AMIGA */
 
 	cmdline_nntpserver[0] = '\0';
 	created_rcdir = FALSE;
 	dangerous_signal_exit = FALSE;
-#ifdef INDEX_DAEMON
-	delete_index_file = FALSE;
-#endif /* INDEX_DAEMON */
 	disable_gnksa_domain_check = FALSE;
+#ifdef MAC_OS_X	/* usualy they don't have a valid FQDN */
+	disable_sender = TRUE;
+#else
 	disable_sender = FALSE;
-	global_filtered_articles = FALSE;
-	local_filtered_articles = FALSE;
+#endif /* MAC_OS_X */
+	filtered_articles = FALSE;
 	iso2asc_supported = atoi (get_val ("ISO2ASC", DEFAULT_ISO2ASC));
 	if (iso2asc_supported > NUM_ISO_TABLES)
 		iso2asc_supported = 0;
@@ -612,20 +591,12 @@ init_selfinfo (
 	num_headers_to_not_display = 0;
 	num_of_selected_arts = 0;
 	num_of_killed_arts = 0;
-	num_of_tagged_arts = 0;
 	post_article_and_exit = FALSE;
 	post_postponed_and_exit = FALSE;
-	purge_index_files = FALSE;
 	read_local_newsgroups_file = FALSE;
 	force_reread_active_file = TRUE;
 	reread_active_for_posted_arts = TRUE;
-	save_news = FALSE;
-	tex2iso_supported = (atoi (get_val ("TEX2ISO", "0")) != 0) ? TRUE : FALSE;
-#ifdef INDEX_DAEMON
-	batch_mode = TRUE;
-#else
 	batch_mode = FALSE;
-#endif /* INDEX_DAEMON */
 	check_for_new_newsgroups = !batch_mode;
 	wildcard_func = wildmat;
 #ifdef HAVE_METAMAIL
@@ -646,9 +617,8 @@ init_selfinfo (
 	index_savedir[0] = '\0';
 	newsrc[0] = '\0';
 
-	sprintf (page_header, "%s %s release %s (\"%s\") [%s%s%s]",
+	sprintf (page_header, "%s %s release %s (\"%s\") [%s%s]",
 		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
-		(tex2iso_supported ? " TeX2ISO" : ""),
 		(iso2asc_supported >= 0 ? " ISO2ASC" : ""));
 	sprintf (cvers, txt_copyright_notice, page_header);
 
@@ -657,7 +627,9 @@ init_selfinfo (
 	news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
 	news_headers_to_not_display_array = NULL;
 
-	strcpy (bug_addr, BUG_REPORT_ADDRESS);
+	/* TODO: nuke $BUG_ADDRESS entirely? */
+	strcpy (bug_addr, get_val("BUG_ADDRESS", BUG_REPORT_ADDRESS));
+
 	bug_nntpserver1[0] = '\0';
 	bug_nntpserver2[0] = '\0';
 
@@ -671,13 +643,13 @@ init_selfinfo (
 
 #ifdef INEWSDIR
 	strcpy (inewsdir, INEWSDIR);
-#else /* INEWSDIR */
+#else
 	inewsdir[0] = '\0';
 #endif /* INEWSDIR */
 
 #ifdef apollo
 	strcpy(default_organization, get_val("NEWSORG", ""));
-#else /* apollo */
+#else
 	strcpy(default_organization, get_val("ORGANIZATION", ""));
 #endif /* apollo */
 
@@ -699,6 +671,11 @@ init_selfinfo (
 	newsgroups_file[0] = '\0';
 	subscriptions_file[0] = '\0';
 
+	/*
+	 * Setup default keymaps
+	 */
+	build_keymaps ();
+
 	/*
 	 * read the global site config file to override some default
 	 * values given at compile time
@@ -707,9 +684,9 @@ init_selfinfo (
 
 	/*
 	 * the site_confog-file was the last chance to set the domainname
-	 *  if it's still unset exit tin.
+	 * if it's still unset exit tin.
 	 */
-	if (domain_name[0]=='\0') {
+	if (domain_name[0] == '\0') {
 		error_message (txt_error_no_domain_name);
 		tin_done(EXIT_FAILURE);
 	}
@@ -741,34 +718,49 @@ init_selfinfo (
 		}
 	}
 
+	/*
+	 * Formerly get_mm_charset(), read_site_config() may set mm_charset
+	 */
+#ifndef CHARSET_CONVERSION
+	STRCPY(tinrc.mm_charset, get_val("MM_CHARSET", MM_CHARSET));
+#else
+	{
+		char *foo;
+		size_t space = 255;
+
+		foo = my_malloc(space);
+		strcpy (foo, "mm_network_charset=");
+		space -= strlen(foo);
+		strncat(foo, get_val("MM_CHARSET", MM_CHARSET), space);
+		if ((space -= strlen(foo)) > 0) {
+			strncat(foo, "\n", space);
+			match_list(foo, "mm_network_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset);
+		}
+	}
+#endif /* !CHARSET_CONVERSION */
 	/* read_site_config() might have changed the value of libdir */
 	/* FIXME: we'd better use TIN_DEFAULTS_DIR instead of TIN_LIBDIR here */
 	joinpath (global_attributes_file, libdir, ATTRIBUTES_FILE);
 	joinpath (global_config_file, libdir, CONFIG_FILE);
-	/*
-	 * FIXME: as we don't know which patternmatzching style the user
-	 * has defined a global filter file is useless
-	 */
-	joinpath (global_filter_file, libdir, FILTER_FILE);
 
 #ifdef VMS
 	joindir (rcdir, homedir, RCDIR); /* we're naming a directory here */
 	joinpath (rcdir_asfile, homedir, RCDIR);	/* for stat() */
 	strcat(rcdir_asfile, ".DIR");
 	if (stat (rcdir_asfile, &sb) == -1)
-#else /* VMS */
+#else
 	joinpath (rcdir, homedir, RCDIR);
 	if (stat (rcdir, &sb) == -1)
 #endif /* VMS */
 	{
 		created_rcdir = TRUE;
-		my_mkdir (rcdir, (mode_t)(S_IRWXU/*|S_IRUGO|S_IXUGO*/));
+		my_mkdir (rcdir, (mode_t)(S_IRWXU));
 	}
 #if defined(M_UNIX) || defined (M_AMIGA) || defined(VMS)
 	strcpy (tinrc.mailer_format, MAILER_FORMAT);
-#else /* M_UNIX ... */
+#else
 	strcpy (tinrc.mailer_format, mailer);
-#endif /* M_UNIX ... */
+#endif /* M_UNIX || M_AMIGA || VMS */
 #ifndef DISABLE_PRINTING
 	strcpy (tinrc.printer, DEFAULT_PRINTER);
 #	ifdef M_AMIGA
@@ -776,39 +768,31 @@ init_selfinfo (
 		strcpy(tinrc.printer, DEFAULT_BBS_PRINTER);
 #	endif /* M_AMIGA */
 #endif /* !DISABLE_PRINTING */
+#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+	strcpy (tinrc.inews_prog, DEFAULT_INEWS_PROG);
+#endif /* NNTP_ABLE) || NNTP_ONLY */
 	strcpy (mailer, get_val (ENV_VAR_MAILER, DEFAULT_MAILER));
-#ifdef VMS
-	joinpath (article, homedir, "article.");
-#else /* VMS */
-	joinpath (article, homedir, ".article");
-#endif /* VMS */
+	joinpath (article, homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
-	sprintf (article+strlen(article), ".%d", (int) process_id);
+	sprintf (article + strlen(article), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 	joinpath (dead_article, homedir, "dead.article");
 	joinpath (dead_articles, homedir, "dead.articles");
-#ifdef VMS
-	joindir (tinrc.maildir, homedir, DEFAULT_MAILDIR);
-	joindir (tinrc.savedir, homedir, DEFAULT_SAVEDIR);
-#else /* VMS */
-	joinpath (tinrc.maildir, homedir, DEFAULT_MAILDIR);
-	joinpath (tinrc.savedir, homedir, DEFAULT_SAVEDIR);
-#endif /* VMS */
+	JOINPATH(tinrc.maildir, homedir, DEFAULT_MAILDIR);
+	JOINPATH(tinrc.savedir, homedir, DEFAULT_SAVEDIR);
 	joinpath (tinrc.sigfile, homedir, ".Sig");
 	joinpath (default_signature, homedir, ".signature");
 
 	if (!index_newsdir[0])
-#ifdef VMS
-		joindir (index_newsdir, get_val ("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
-#else /* VMS */
-		joinpath (index_newsdir, get_val ("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
-#endif /* VMS */
+		JOINPATH(index_newsdir, get_val ("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
+#if 0
+	 else {
+		if (stat (index_newsdir, &sb) == -1)
+			my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);
+	}
+#endif /* 0 */
 
-#ifdef VMS
-	joindir (index_maildir, get_val ("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
-#else /* VMS */
-	joinpath (index_maildir, get_val ("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
-#endif /* VMS */
+	JOINPATH(index_maildir, get_val ("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
 	if (stat (index_maildir, &sb) == -1)
 		my_mkdir (index_maildir, (mode_t)S_IRWXUGO);
 	joinpath (index_savedir, get_val ("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
@@ -817,63 +801,36 @@ init_selfinfo (
 		my_mkdir (index_savedir, (mode_t)S_IRWXUGO);
 	joinpath (local_attributes_file, rcdir, ATTRIBUTES_FILE);
 	joinpath (local_config_file, rcdir, CONFIG_FILE);
-	joinpath (local_filter_file, rcdir, FILTER_FILE);
+	joinpath (filter_file, rcdir, FILTER_FILE);
 	joinpath (local_input_history_file, rcdir, INPUT_HISTORY_FILE);
 	joinpath (local_newsrctable_file, rcdir, NEWSRCTABLE_FILE);
 	joinpath (local_newsgroups_file, rcdir, NEWSGROUPS_FILE);
 	joinpath (mail_active_file, rcdir, ACTIVE_MAIL_FILE);
 #ifdef VMS
 	joinpath (mailbox, DEFAULT_MAILBOX, "MAIL.TXT");
-#else /* VMS */
+#else
 	joinpath (mailbox, DEFAULT_MAILBOX, userid);
 #endif /* VMS */
-	joinpath (msg_headers_file, rcdir, MSG_HEADERS_FILE);
 	joinpath (mailgroups_file, rcdir, MAILGROUPS_FILE);
-#ifdef WIN32
-	joinpath (newsrc, rcdir, NEWSRC_FILE);
-	joinpath (newnewsrc, rcdir, NEWNEWSRC_FILE);
-#else /* WIN32 */
 	joinpath (newsrc, homedir, NEWSRC_FILE);
 	joinpath (newnewsrc, homedir, NEWNEWSRC_FILE);
-#	ifdef APPEND_PID
-	sprintf(newnewsrc+strlen(newnewsrc), "%d", (int) process_id);
-#	endif /* APPEND_PID */
-#endif /* WIN32 */
+#ifdef APPEND_PID
+	sprintf(newnewsrc + strlen(newnewsrc), "%d", (int) process_id);
+#endif /* APPEND_PID */
 	joinpath (posted_info_file, rcdir, POSTED_FILE);
 	joinpath (posted_msgs_file, tinrc.maildir, POSTED_FILE);
 	joinpath (postponed_articles_file, rcdir, POSTPONED_FILE);
 	joinpath (save_active_file, rcdir, ACTIVE_SAVE_FILE);
 
-#ifdef INDEX_DAEMON
-	joinpath (lock_file, TMPDIR, LOCK_FILE);
-	strcpy (newsrc, news_active_file);	/* default so all groups are indexed */
-	joinpath (group_times_file, rcdir, GROUP_TIMES_FILE);
-	joinpath (index_newsdir, get_val ("TIN_INDEX_NEWSDIR", novrootdir), INDEX_NEWSDIR);
-
-	if (stat (index_newsdir, &sb) == -1)
-		my_mkdir (index_newsdir, (mode_t)(S_IRWXU|S_IRUGO|S_IXUGO));
+#ifdef HAVE_LONG_FILE_NAMES
+	sprintf (lock_file, "%stin.%s.LCK", TMPDIR, userid);
 #else
-#	ifdef HAVE_LONG_FILE_NAMES
-		sprintf (lock_file, "%stin.%s.LCK", TMPDIR, userid);
-#	else
-		sprintf (lock_file, "%s%s.LCK", TMPDIR, userid);
-#	endif /* HAVE_LONG_FILE_NAMES */
+	sprintf (lock_file, "%s%s.LCK", TMPDIR, userid);
+#endif /* HAVE_LONG_FILE_NAMES */
 
-#	ifdef NNTP_ABLE
+#ifdef NNTP_ABLE
 	nntp_tcp_port = (unsigned short) atoi (get_val ("NNTPPORT", NNTP_TCP_PORT));
-#	endif /* NNTP_ABLE */
-
-	if (tin_uid != real_uid) {
-		joinpath (index_newsdir, get_val ("TIN_INDEX_NEWSDIR", spooldir), INDEX_NEWSDIR);
-		set_tin_uid_gid ();
-		if (stat (index_newsdir, &sb) == -1)
-			my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);
-
-		set_real_uid_gid ();
-	} else {
-		if (stat (index_newsdir, &sb) == -1)
-			my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);
-	}
+#endif /* NNTP_ABLE */
 
 	if (stat (posted_info_file, &sb) == -1) {
 		if ((fp = fopen (posted_info_file, "w")) != (FILE *) 0) {
@@ -882,107 +839,54 @@ init_selfinfo (
 		}
 	}
 
-	if (stat (msg_headers_file, &sb) == -1) {
-		if ((fp = fopen (msg_headers_file, "w")) != (FILE *) 0) {
-			fprintf (fp, txt_msg_headers_file);
-			fclose (fp);
-		}
-	}
-
 	if (stat (local_attributes_file, &sb) == -1)
 		write_attributes_file (local_attributes_file);
 
-#endif /* INDEX_DAEMON */
+	init_postinfo();
+	sprintf (txt_help_bug_report, _(txt_help_bug), bug_addr);
 
-	/*
-	 *  check enviroment for REPLYTO
-	 */
-	reply_to[0] = '\0';
-	if ((ptr = getenv ("REPLYTO")) != (char *) 0)
-		my_strncpy (reply_to, ptr, sizeof (reply_to));
-
-	/*
-	 *  check enviroment for DISTRIBUTION
-	 */
-	my_distribution[0] = '\0';
-	if ((ptr = getenv ("DISTRIBUTION")) != (char *) 0)
-		my_strncpy (my_distribution, ptr, sizeof (my_distribution));
-
-	/*
-	 * check enviroment for BUG_ADDRESS
-	 *
-	 * Argh! I think it's complete nonsense to have a _per_user_
-	 * configurable bug report address in $HOME/.tin/bug_address!
-	 * I think we should delete this whole section. Who sets a
-	 * bug report in environment anyway? XXXXXXXXXXXXXXXXXX
-	 */
-	/* I agree (obw) */
-	if ((ptr = getenv ("BUG_ADDRESS")) != (char *) 0) {
-		my_strncpy (bug_addr, ptr, sizeof (bug_addr));
-	} else {
-		joinpath (nam, rcdir, "bug_address");
-		if ((fp = fopen (nam, "r")) != (FILE *) 0) {
-			if (fgets (bug_addr, (int) sizeof (bug_addr), fp) != (char *) 0) {
-				ptr = strrchr (bug_addr, '\n');
-				if (ptr != (char *) 0)
-					*ptr = '\0';
-			}
-			fclose (fp);
-		}
-	}
-	sprintf (txt_help_bug_report, txt_help_bug, bug_addr);
-
-#ifdef HAVE_PGP
-	pgpopts = get_val("PGPOPTS", "");
-	if ((ptr = getenv("PGPPATH")) != (char *) 0)
-		my_strncpy (pgp_data, ptr, sizeof(pgp_data));
-	else
-		joinpath (pgp_data, homedir, ".pgp");
-#endif /* HAVE_PGP */
+#ifdef HAVE_PGP_GPG
+	init_pgp();
+#endif /* HAVE_PGP_GPG */
 }
 
 /*
- * If we're caching overview files (currently supported for non-setuid
- * Tin only; we don't handle updating of shared cache files!) and the
- * user specified an NNTP server with the '-g' option, make the directory
- * name specific to the NNTP server and make sure the directory exists.
+ * If we're caching overview files and the user specified an NNTP server
+ * with the '-g' option, make the directory name specific to the NNTP server
+ * and make sure the directory exists.
  */
-#ifndef INDEX_DAEMON
 void
 set_up_private_index_cache (
 	void)
 {
-	char *to;
-	char *from;
-	int c;
 	struct stat sb;
 
-	if (! tinrc.cache_overview_files)
-		return;
-	if (! local_index) {
-		error_message (txt_caching_disabled);
-		tinrc.cache_overview_files = FALSE;
+	if (xover_supported && !tinrc.cache_overview_files)
 		return;
+
+	if (cmdline_nntpserver[0] != 0) {
+		char *from;
+		char *to;
+		int c;
+
+		to = index_newsdir + strlen (index_newsdir);
+		*(to++) = '-';
+		for (from = cmdline_nntpserver; (c = *from) != 0; ++from)
+			*(to++) = tolower(c);
+		*to = 0;
 	}
-	if (cmdline_nntpserver[0] == 0)
-		return;
-	to = index_newsdir + strlen (index_newsdir);
-	*(to++) = '-';
-	for (from = cmdline_nntpserver; (c = *from) != 0; ++from)
-		*(to++) = tolower(c);
-	*to = 0;
+
 	if (stat (index_newsdir, &sb) == -1)
-		my_mkdir (index_newsdir, (mode_t)S_IRWXUGO); /* why not S_IRWXU ? */
+		my_mkdir (index_newsdir, (mode_t)S_IRWXU); /* was S_IRWXUGO */
+
 #	ifdef DEBUG
 	debug_nntp ("set_up_private_index_cache", index_newsdir);
 #	endif /* DEBUG */
 	joinpath (local_newsgroups_file, index_newsdir, NEWSGROUPS_FILE);
 	return;
 }
-#endif /* !INDEX_DAEMON */
 
 
-#ifndef INDEX_DAEMON
 /*
  * Create default mail & save directories if they do not exist
  */
@@ -994,7 +898,7 @@ create_mail_save_dirs (
 	char path[PATH_LEN];
 	struct stat sb;
 
-	if (!strfpath (tinrc.maildir, path, sizeof (path), homedir, (char *) 0, (char *) 0, (char *) 0))
+	if (!strfpath (tinrc.maildir, path, sizeof (path), NULL))
 		joinpath (path, homedir, DEFAULT_MAILDIR);
 
 	if (stat (path, &sb) == -1) {
@@ -1002,7 +906,7 @@ create_mail_save_dirs (
 		created = TRUE;
 	}
 
-	if (!strfpath (tinrc.savedir, path, sizeof (path), homedir, (char *) 0, (char *) 0, (char *) 0))
+	if (!strfpath (tinrc.savedir, path, sizeof (path), NULL))
 		joinpath (path, homedir, DEFAULT_SAVEDIR);
 
 	if (stat (path, &sb) == -1) {
@@ -1010,9 +914,8 @@ create_mail_save_dirs (
 		created = TRUE;
 	}
 
-	return (created);
+	return created;
 }
-#endif /* !INDEX_DAEMON */
 
 
 /*
@@ -1075,8 +978,13 @@ read_site_config (
 			continue;
 		if (match_string (buf, "organization=", default_organization, sizeof (default_organization)))
 			continue;
+#ifndef CHARSET_CONVERSION
 		if (match_string (buf, "mm_charset=", tinrc.mm_charset, sizeof (tinrc.mm_charset)))
 			continue;
+#else
+		if (match_list (buf, "mm_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset))
+			continue;
+#endif /* !CHARSET_CONVERSION */
 		if (match_list (buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
 			continue;
 		if (match_list (buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
@@ -1117,4 +1025,13 @@ postinit_regexp (
 		STRCPY(tinrc.quote_regex3, DEFAULT_QUOTE_REGEX3);
 	compile_regex (tinrc.quote_regex3, &quote_regex3, PCRE_CASELESS);
 #endif /* HAVE_COLOR */
+
+	compile_regex (UUBEGIN_REGEX, &uubegin_regex, PCRE_CASELESS|PCRE_ANCHORED);
+	compile_regex (UUBODY_REGEX, &uubody_regex, PCRE_ANCHORED);
+
+	compile_regex (URL_REGEX, &url_regex, PCRE_CASELESS);
+	compile_regex (MAIL_REGEX, &mail_regex, PCRE_CASELESS);
+	compile_regex (NEWS_REGEX, &news_regex, PCRE_CASELESS);
+
+	compile_regex (SHAR_REGEX, &shar_regex, PCRE_ANCHORED);
 }
