@@ -5,13 +5,39 @@
  *  Created   : 1991-04-01
  *  Updated   : 1997-12-28
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
- *  Copyright : (c) Copyright 1991-99 by Iain Lea & Rich Skrenta
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea, Rich Skrenta.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -19,7 +45,7 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-#define BITS_TO_BYTES(n)	(size_t)((n+NBITS-1)/NBITS)
+#define BITS_TO_BYTES(n)	(size_t)((n + NBITS - 1) / NBITS)
 
 static mode_t newsrc_mode = 0;
 
@@ -44,7 +70,7 @@ static void print_bitmap_seq (FILE *fp, struct t_group *group);
  * Any bogus groups will be handled accordingly. Bogus groups will _not_
  * be subscribed to as a design principle.
  *
- * Returns the numer of lines read (usefull for a check newsrc >= oldnewsrc)
+ * Returns the numer of lines read (useful for a check newsrc >= oldnewsrc)
  *    < 0 error
  *    >=0 number of lines read
  */
@@ -56,11 +82,11 @@ read_newsrc (
 	FILE *fp;
 	char *grp, *seq;
 	int sub, i;
-	signed long lines = 0;
+	signed long line_count = 0;
 	struct stat statbuf;
 
 	if (allgroups)
-		group_top = skip_newgroups();
+		selmenu.max = skip_newgroups();
 
 	/*
 	 * make a .newsrc if one doesn't exist & auto subscribe to set groups
@@ -73,12 +99,12 @@ read_newsrc (
 		newsrc_mode = (mode_t) statbuf.st_mode;
 
 	if ((fp = fopen (newsrc_file, "r")) != (FILE *) 0) {
-		if (INTERACTIVE)
-			wait_message (0, txt_reading_newsrc);
+		if (!batch_mode)
+			wait_message (0, _(txt_reading_newsrc));
 
 		while ((grp = tin_fgets (fp, FALSE)) != (char *) 0) {
 			seq = pcParseNewsrcLine (grp, &sub);
-			lines++;
+			line_count++;
 
 			if (sub == SUBSCRIBED) {
 				if ((i = my_group_add (grp)) >= 0) {
@@ -98,7 +124,7 @@ read_newsrc (
 			my_flush ();
 		}
 	}
-	return lines;
+	return line_count;
 }
 
 
@@ -122,7 +148,7 @@ iWriteNewsrcLine (
 
 	if (seq == NULL) {		/* line has no ':' or '!' in it */
 		if (tinrc.strip_bogus == BOGUS_REMOVE)
-			wait_message(2, txt_remove_bogus, line);
+			wait_message(2, _(txt_remove_bogus), line);
 		return 0;
 	}
 
@@ -131,11 +157,11 @@ iWriteNewsrcLine (
 	 * are set to auto removal. Also check for bogus flag just in case
 	 * strip_bogus was changed since tin started
 	 */
-	psGrp = psGrpFind (line);
+	psGrp = group_find (line);
 
 	if (tinrc.strip_bogus == BOGUS_REMOVE) {
 		if (psGrp == NULL || psGrp->bogus) { /* group dosen't exist */
-			wait_message(2, txt_remove_bogus, line);
+			wait_message(2, _(txt_remove_bogus), line);
 			return 0;
 		}
 	}
@@ -163,10 +189,9 @@ iWriteNewsrcLine (
  *         >=0 number of lines written
  */
 signed long int
-vWriteNewsrc (
+write_newsrc (
 	void)
 {
-#ifndef INDEX_DAEMON
 	FILE *fp_ip;
 	FILE *fp_op;
 	char *line;
@@ -199,7 +224,7 @@ vWriteNewsrc (
 		 * Don't rename if either fclose() fails or ferror() is set
 		 */
 		if (ferror (fp_op) || fclose (fp_op)) {
-			error_message (txt_filesystem_full, NEWSRC_FILE);
+			error_message (_(txt_filesystem_full), NEWSRC_FILE);
 			unlink (newnewsrc);
 		} else
 			write_ok = TRUE;
@@ -208,7 +233,7 @@ vWriteNewsrc (
 	fclose (fp_ip);
 
 	if (tot < 1) {
-		error_message (txt_newsrc_nogroups);
+		error_message (_(txt_newsrc_nogroups));
 		return 0L;		/* So we don't get prompted to try again */
 	}
 
@@ -216,9 +241,6 @@ vWriteNewsrc (
 		rename_file (newnewsrc, newsrc);
 
 	return tot;
-#else
-	return 1L;
-#endif /* !INDEX_DAEMON */
 }
 
 
@@ -233,13 +255,13 @@ create_newsrc (
 	register int i;
 
 	if ((fp = fopen (newsrc_file, "w")) != (FILE *) 0) {
-		wait_message (0, txt_creating_newsrc);
+		wait_message (0, _(txt_creating_newsrc));
 
 		for (i = 0; i < num_active; i++)
 			fprintf (fp, "%s!\n", active[i].name);
 
 		if (ferror (fp) || fclose (fp)) {
-			error_message (txt_filesystem_full, NEWSRC_FILE);
+			error_message (_(txt_filesystem_full), NEWSRC_FILE);
 			return FALSE;
 		}
 		return TRUE; /* newsrc created */
@@ -250,7 +272,7 @@ create_newsrc (
 
 /*
  * Automatically subscribe user to newsgroups specified in
- * NEWSLIBDIR/subscribe (locally) or same file but from NNTP
+ * NEWSLIBDIR/subscriptions (locally) or same file but from NNTP
  * server (LIST SUBSCRIPTIONS) and create .newsrc
  */
 static void
@@ -268,7 +290,7 @@ auto_subscribe_groups (
 	if ((fp_subs = open_subscription_fp ()) == (FILE *) 0)
 		return;
 
-	wait_message (0, txt_autosubscribing_groups);
+	wait_message (0, _(txt_autosubscribing_groups));
 
 	if ((fp_newsrc = fopen (newsrc_file, "w" FOPEN_OPTS)) == (FILE *) 0)
 		return;
@@ -276,10 +298,10 @@ auto_subscribe_groups (
 	if (newsrc_mode)
 		chmod (newsrc_file, newsrc_mode);
 
-/* TODO test me ! */
+	/* TODO test me ! */
 	while ((ptr = tin_fgets (fp_subs, FALSE)) != (char *) 0) {
 		if (ptr[0] != '#') {
-			if (psGrpFind (ptr) != 0)
+			if (group_find (ptr) != 0)
 				fprintf (fp_newsrc, "%s:\n", ptr);
 		}
 	}
@@ -287,7 +309,7 @@ auto_subscribe_groups (
 	/* We ignore user 'q'uits here. They will get them next time in any case */
 
 	if (ferror (fp_newsrc) || fclose (fp_newsrc))
-		error_message (txt_filesystem_full, NEWSRC_FILE);
+		error_message (_(txt_filesystem_full), NEWSRC_FILE);
 
 	TIN_FCLOSE (fp_subs);
 }
@@ -300,39 +322,21 @@ void
 backup_newsrc (
 	void)
 {
-#ifndef INDEX_DAEMON
-	FILE *fp_ip, *fp_op;
-	char *line;
 	char buf[HEADER_LEN];
 
-	if ((fp_ip = fopen (newsrc, "r")) != (FILE *) 0) {
-#	if defined(WIN32)
-		joinpath (buf, rcdir, OLDNEWSRC_FILE);
-#	else
-		joinpath (buf, homedir, OLDNEWSRC_FILE);
-#	endif /* WIN32 */
-		unlink (buf);	/* because rn makes a link of .newsrc -> .oldnewsrc */
+	joinpath (buf, homedir, OLDNEWSRC_FILE);
 
-		if ((fp_op = fopen (buf, "w" FOPEN_OPTS)) != (FILE *) 0) {
-			if (newsrc_mode)
-				chmod (buf, newsrc_mode);
+	if (!backup_file (newsrc, buf))
+		error_message (_(txt_filesystem_full_backup), NEWSRC_FILE);
 
-			while ((line = tin_fgets (fp_ip, FALSE)) != (char *) 0)
-				fprintf (fp_op, "%s\n", line);
-
-			if (ferror (fp_op) || fclose (fp_op))
-				error_message (txt_filesystem_full_backup, NEWSRC_FILE);
-
-		}
-		fclose (fp_ip);
-	}
-#endif /* !INDEX_DAEMON */
+	if (newsrc_mode)
+		chmod (buf, newsrc_mode);
 }
 
 
 /*
  * Subscribe/unsubscribe to a group in .newsrc.
- *	This involves rewriting the .newsrc with the new info
+ * This involves rewriting the .newsrc with the new info
  */
 void
 subscribe (
@@ -381,7 +385,7 @@ subscribe (
 		fclose (fp);
 
 		if (!found) {
-			wait_message (0, txt_subscribing);
+			wait_message (0, _(txt_subscribing));
 			group->subscribed = SUB_BOOL(sub_state);
 			if (sub_state == SUBSCRIBED) {
 				vGet1GrpArtInfo(group);
@@ -393,7 +397,7 @@ subscribe (
 	}
 
 	if (ferror (newfp) || fclose (newfp)) {
-		error_message (txt_filesystem_full, NEWSRC_FILE);
+		error_message (_(txt_filesystem_full), NEWSRC_FILE);
 		unlink (newnewsrc);
 	} else
 		rename_file (newnewsrc, newsrc);
@@ -423,14 +427,14 @@ reset_newsrc (
 			fclose (fp);
 		}
 		if (ferror (newfp) || fclose (newfp)) {
-			error_message (txt_filesystem_full, NEWSRC_FILE);
+			error_message (_(txt_filesystem_full), NEWSRC_FILE);
 			unlink (newnewsrc);
 		} else
 			rename_file (newnewsrc, newsrc);
 	}
 
-	for (i = 0; i < group_top; i++)
-		vSetDefaultBitmap (&active[my_group[i]]);
+	for (i = 0; i < selmenu.max; i++)
+		set_default_bitmap (&active[my_group[i]]);
 }
 
 
@@ -467,7 +471,7 @@ delete_group (
 		}
 
 		if (ferror (newfp) || fclose (newfp)) {
-			error_message (txt_filesystem_full, NEWSRC_FILE);
+			error_message (_(txt_filesystem_full), NEWSRC_FILE);
 			unlink (newnewsrc);
 		} else
 			rename_file (newnewsrc, newsrc);
@@ -493,7 +497,7 @@ grp_mark_read (
 #endif /* DEBUG_NEWSRC */
 
 	if (psArt != (struct t_article *) 0) {
-		for (i = 0; i < top; i++)
+		for (i = 0; i < top_art; i++)
 			art_mark_read (group, &psArt[i]);
 	}
 	if (group->newsrc.xbitmap != (t_bitmap *) 0) {
@@ -520,7 +524,7 @@ grp_mark_unread (
 	debug_print_comment ("Z command");
 #endif /* DEBUG_NEWSRC */
 
-	vGrpGetArtInfo (
+	group_get_art_info (
 		group->spooldir,
 		group->name,
 		group->type,
@@ -614,7 +618,7 @@ parse_bitmap_seq (
 #ifdef DEBUG_NEWSRC
 	{
 		char buf[NEWSRC_LINE];
-		sprintf (buf, "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE-strlen(group->name)-20), ptr);
+		sprintf (buf, "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 20), ptr);
 		debug_print_comment(buf);
 		debug_print_bitmap(group, NULL);
 	}
@@ -838,11 +842,11 @@ parse_unread_arts (
 		group->newsrc.xmax = group->xmax;
 
 	if (group->newsrc.xmax >= bitmin) {
-		newbitmap = (t_bitmap *)my_malloc(BITS_TO_BYTES(group->newsrc.xmax-bitmin+1));
+		newbitmap = (t_bitmap *) my_malloc(BITS_TO_BYTES(group->newsrc.xmax-bitmin + 1));
 		NSETRNG0(newbitmap, 0L, group->newsrc.xmax - bitmin);
 	}
 
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		if (arts[i].artnum < bitmin)
 			arts[i].status = ART_READ;
 		else if (arts[i].artnum > bitmax)
@@ -988,7 +992,7 @@ pos_group_in_newsrc (
 	joinpath (buf, TMPDIR, "unsubrc");
 	sprintf (unsub, "%s.%d", buf, (int) process_id);
 
-#else /* !VMS */
+#else
 	joinpath (buf, TMPDIR, ".subrc");
 	sprintf (sub, "%s.%d", buf, (int) process_id);
 
@@ -1028,7 +1032,7 @@ pos_group_in_newsrc (
 	}
 
 	if (ferror (fp_sub) || fclose (fp_sub) || ferror (fp_unsub) || fclose (fp_unsub)) {
-		error_message (txt_filesystem_full, NEWSRC_FILE);
+		error_message (_(txt_filesystem_full), NEWSRC_FILE);
 		fp_sub = fp_unsub = NULL;
 		goto rewrite_group_done;
 	}
@@ -1086,7 +1090,7 @@ pos_group_in_newsrc (
 	 * Try and cleanly close out the newnewsrc file
 	 */
 	if (ferror (fp_out) || fclose (fp_out))
-		error_message (txt_filesystem_full, NEWSRC_FILE);
+		error_message (_(txt_filesystem_full), NEWSRC_FILE);
 	else {
 		if (repositioned) {
 			rename_file (newnewsrc, newsrc);
@@ -1134,10 +1138,7 @@ catchup_newsrc_file (
 	register int i;
 	struct t_group *group;
 
-	if (!catchup)
-		return;
-
-	for (i = 0; i < group_top; i++) {
+	for (i = 0; i < selmenu.max; i++) {
 		group = &active[my_group[i]];
 		group->newsrc.present = TRUE;
 		if (group->newsrc.xbitmap != (t_bitmap *) 0) {
@@ -1150,8 +1151,6 @@ catchup_newsrc_file (
 		group->newsrc.num_unread = 0;
 		group->newsrc.xbitlen = 0;
 	}
-
-	tin_done (EXIT_SUCCESS);
 }
 
 
@@ -1170,7 +1169,7 @@ pcParseNewsrcLine (
 	*sub = UNSUBSCRIBED;				/* Default to no entry */
 
 	if ((ptr = strpbrk(line, " !:")) == NULL)			/* space|SUBSCRIBED|UNSUBSCRIBED */
-		return(NULL);
+		return NULL;
 
 	*sub = *ptr;						/* Save the subscription status */
 	tmp = ptr;							/* Keep this blank for later */
@@ -1178,11 +1177,11 @@ pcParseNewsrcLine (
 
 #if 0
 	if (ptr == NULL)					/* No seq info, so return a blank */
-		return(tmp);
+		return tmp;
 #endif /* 0 */
 
 	if ((ptr = strpbrk(ptr, " \t")) == NULL)
-		return(tmp);
+		return tmp;
 
 	return (ptr + 1);	/* Return pointer to sequence info. At worst this will be \0 */
 }
@@ -1217,11 +1216,11 @@ expand_bitmap (
 	if (min >= group->newsrc.xmin)
 		first = group->newsrc.xmin;
 	else
-		first = group->newsrc.xmin - ((group->newsrc.xmin - min + (NBITS-1)) & ~(NBITS-1));
+		first = group->newsrc.xmin - ((group->newsrc.xmin - min + (NBITS - 1)) & ~(NBITS - 1));
 
 	/* adjust first */
 	if (first > group->newsrc.xmax + 1)
-		first = first - ((first - (group->newsrc.xmax + 1) + (NBITS-1)) & ~(NBITS-1));
+		first = first - ((first - (group->newsrc.xmax + 1) + (NBITS - 1)) & ~(NBITS - 1));
 
 	/* check first */
 	if (first < 1) {
@@ -1426,7 +1425,6 @@ art_mark_will_return (
 }
 
 
-#if !defined(INDEX_DAEMON)
 void
 art_mark_deleted (
 	struct t_article *art)
@@ -1443,11 +1441,10 @@ art_mark_undeleted (
 	if (art != (struct t_article *) 0)
 		art->delete_it = FALSE;
 }
-#endif /* !INDEX_DAEMON */
 
 
 void
-vSetDefaultBitmap (
+set_default_bitmap (
 	struct t_group *group)
 {
 	if (group != (struct t_group *) 0) {
@@ -1489,6 +1486,11 @@ vNewsrcTestHarness (
 	long rng_min, rng_max;
 	struct t_group group;
 
+	/*
+	 * those env.-vars are not documented:
+	 * $TIN_RNG_MIN, $TIN_RNG_MAX, $TIN_MIN, $TIN_MAX, $TIN_COUNT
+	 * $TIN_SEQ
+	 */
 	rng_min = atoi (get_val ("TIN_RNG_MIN", "1"));
 	rng_max = atoi (get_val ("TIN_RNG_MAX", "1"));
 
@@ -1499,7 +1501,7 @@ vNewsrcTestHarness (
 	group.type = GROUP_TYPE_NEWS;
 	group.subscribed = TRUE;
 	group.newsrc.xbitmap = (t_bitmap *) 0;
-	vSetDefaultBitmap (&group);
+	set_default_bitmap (&group);
 
 	strcpy (seq, get_val ("TIN_SEQ", ""));
 
@@ -1518,10 +1520,10 @@ vNewsrcTestHarness (
 		while (retry) {
 		/* FIXME - this is secure now, but doesn't write any debug output */
 		/* (it didn't before too) */
-			if ((temp_file = my_tempnam ("","NEWSRC")) != (char *) 0) {
-				if ((fd = open (temp_file, (O_CREAT|O_EXCL), (S_IRUSR|S_IWUSR))) !=-1) {
+			if ((temp_file = my_tempnam ("", "NEWSRC")) != (char *) 0) {
+				if ((fd = open (temp_file, (O_CREAT|O_EXCL), (S_IRUSR|S_IWUSR))) != -1) {
 					if ((fp = fopen (temp_file, "w")) != (FILE *) 0) {
-						my_printf ("\n%d. PARSE Seq=[%s]\n", i+1, seq);
+						my_printf ("\n%d. PARSE Seq=[%s]\n", i + 1, seq);
 						parse_bitmap_seq (&group, seq);
 						debug_print_newsrc (&group.newsrc, stdout);
 						print_bitmap_seq (fp, &group);
@@ -1545,7 +1547,7 @@ vNewsrcTestHarness (
 		else {
 			fp = fopen (temp_file, "r");
 			fgets (seq, (int) sizeof(seq), fp);
-			seq[strlen(seq)-1] = '\0';
+			seq[strlen(seq) - 1] = '\0';
 			fclose (fp);
 		}
 	}
@@ -1556,7 +1558,6 @@ vNewsrcTestHarness (
 	set_bitmap_range_unread (&group.newsrc, rng_min, rng_max);
 	debug_print_newsrc (&group.newsrc, stdout);
 
-
 	NSETBLK0(group.newsrc.xbitmap, group.newsrc.xbitlen);
 	debug_print_newsrc (&group.newsrc, stdout);
 
@@ -1604,8 +1605,7 @@ set_bitmap_range_unread (
 		offset = beg - newsrc->xmin;
 		length = end - newsrc->xmin;
 
-my_printf ("\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n",
-newsrc->xmin, newsrc->xmax, beg, end, offset, length);
+my_printf ("\nRNG Min-Max=[%ld-%ld] Beg-End=[%ld-%ld] OFF=[%ld] LEN=[%ld]\n", newsrc->xmin, newsrc->xmax, beg, end, offset, length);
 
 		if (beg == end) {
 			NSET1(newsrc->xbitmap, offset);
@@ -1615,5 +1615,4 @@ newsrc->xmin, newsrc->xmax, beg, end, offset, length);
 	}
 }
 #	endif /* DEBUG_NEWSRC_FIXME */
-
 #endif /* DEBUG_NEWSRC */
