@@ -57,6 +57,7 @@ static TCP *nntp_wr_fp = NULL;
 		s_fclose (nntp_rd_fp);			\
 	nntp_rd_fp = nntp_wr_fp = NULL;
 
+
 /*
  * Return the actual fd in use for the nntp read-side socket
  * This is a bit of a leak of internal state, but it's use is very
@@ -69,6 +70,7 @@ get_nntp_fp (
 	return (fp == FAKE_NNTP_FP ? nntp_rd_fp : fp);
 }
 
+
 FILE *
 get_nntp_wr_fp (
 	FILE *fp)
@@ -76,6 +78,7 @@ get_nntp_wr_fp (
 	return (fp == FAKE_NNTP_FP ? nntp_wr_fp : fp);
 }
 
+
 /*
  * getserverbyfile(file)
  *
@@ -91,7 +94,6 @@ get_nntp_wr_fp (
  *
  *	Side effects: None.
  */
-
 char *
 getserverbyfile (
 	const char *file)
@@ -116,7 +118,9 @@ getserverbyfile (
 		FreeIfNeeded (old_env);
 		old_env = new_env;
 #	else
+#		ifdef HAVE_SETENV
 		setenv ("NNTPSERVER", buf, 1);
+#		endif /* HAVE_SETENV */
 #	endif /* HAVE_PUTENV */
 		return (buf);
 	}
@@ -166,6 +170,7 @@ getserverbyfile (
 	return (char *) 0;	/* No entry */
 }
 
+
 /*
  * server_init  Get a connection to the remote server.
  *
@@ -181,7 +186,6 @@ getserverbyfile (
  *			for reading and writing to server.
  *			"text" is updated to contain the rest of response string from server
  */
-
 #ifdef NNTP_ABLE
 int
 server_init (
@@ -190,18 +194,18 @@ server_init (
 	int port,
 	char *text)
 {
-#ifndef INET6
+#	ifndef INET6
 	char temp[256];
 	char *service = strncpy(temp, cservice, 255); /* ...calls non-const funcs */
-#endif /* !INET6 */
+#	endif /* !INET6 */
 #	ifndef VMS
 	int sockt_rd, sockt_wr;
 #	endif /* !VMS */
 
-#	if defined (M_AMIGA) || defined(WIN32)
+#	if defined (M_AMIGA)
 	if (!s_init())		/* some initialisation ... */
 		return -1;
-#	endif /* M_AMIGA || WIN32 */
+#	endif /* M_AMIGA */
 
 #	ifdef DECNET
 	char *cp;
@@ -227,30 +231,30 @@ server_init (
 #	ifndef VMS
 	/*
 	 * Now we'll make file pointers (i.e., buffered I/O) out of
-	 * the socket file descriptor.  Note that we can't just
+	 * the socket file descriptor. Note that we can't just
 	 * open a fp for reading and writing -- we have to open
 	 * up two separate fp's, one for reading, one for writing.
 	 */
 
 	if ((nntp_rd_fp = (TCP *) s_fdopen (sockt_rd, "r")) == NULL) {
-		perror ("server_init: fdopen #1"); /* FIXME: -> lang.c */
+		perror ("server_init: fdopen #1");
 		return (-errno);
 	}
 
 	if ((sockt_wr = s_dup (sockt_rd)) < 0) {
-		perror ("server_init: dup"); /* FIXME: -> lang.c */
+		perror ("server_init: dup");
 		return (-errno);
 	}
 
 #		ifdef TLI /* Transport Level Interface */
 	if (t_sync (sockt_rd) < 0) {	/* Sync up new fd with TLI */
-		t_error ("server_init: t_sync"); /* FIXME: -> lang.c */
+		t_error ("server_init: t_sync");
 		nntp_rd_fp = NULL;
 		return (-EPROTO);
 	}
 #		else
 	if ((nntp_wr_fp = (TCP *) s_fdopen (sockt_wr, "w")) == NULL) {
-		perror ("server_init: fdopen #2"); /* FIXME: -> lang.c */
+		perror ("server_init: fdopen #2");
 		nntp_rd_fp = NULL;
 		return (-errno);
 	}
@@ -316,7 +320,7 @@ get_tcp_socket (
 		return (-EPROTO);
 	}
 	if (t_bind (s, (struct t_bind *) 0, (struct t_bind *) 0) < 0) {
-		t_error ("t_bind"); /* FIXME: -> lang.c */
+		t_error ("t_bind");
 		t_close (s);
 		return (-EPROTO);
 	}
@@ -344,7 +348,7 @@ get_tcp_socket (
 	 * Let t_alloc() initialize the addr structure of the t_call structure.
 	 */
 	if ((callptr = (struct t_call *) t_alloc (s, T_CALL, T_ADDR)) == NULL){
-		t_error ("t_alloc"); /* FIXME: -> lang.c */
+		t_error ("t_alloc");
 		t_close (s);
 		return (-EPROTO);
 	}
@@ -361,9 +365,9 @@ get_tcp_socket (
 	if (t_connect (s, callptr, (struct t_call *) 0) < 0) {
 		save_errno = t_errno;
 		if (save_errno == TLOOK)
-			fprintf(stderr, "Server unavailable\n"); /* FIXME: -> lang.c */
+			fprintf(stderr, _("Server unavailable\n")); /* FIXME: -> lang.c */
 		else
-			t_error ("t_connect"); /* FIXME: -> lang.c */
+			t_error ("t_connect");
 		t_free((char *)callptr, T_CALL);
 		t_close (s);
 		return (-save_errno);
@@ -377,34 +381,34 @@ get_tcp_socket (
 
 	t_free((char *)callptr, T_CALL);
 
-	if (ioctl (s,  I_POP,  (char *) 0) < 0) {
-		perror ("I_POP(timod)"); /* FIXME: -> lang.c */
+	if (ioctl (s, I_POP, (char *) 0) < 0) {
+		perror ("I_POP(timod)");
 		t_close (s);
 		return (-EPROTO);
 	}
 
 	if (ioctl (s, I_PUSH, "tirdwr") < 0) {
-		perror ("I_PUSH(tirdwr)"); /* FIXME: -> lang.c */
+		perror ("I_PUSH(tirdwr)");
 		t_close (s);
 		return (-EPROTO);
 	}
 
-#	else /* not TLI */
+#	else
 #		ifndef EXCELAN
 	struct servent *sp;
 	struct hostent *hp;
 #			ifdef h_addr
 	int x = 0;
 	register char **cp;
-	static char *alist[2] = {0,0};
+	static char *alist[2] = {0, 0};
 #			endif /* h_addr */
 	static struct hostent def;
 	static struct in_addr defaddr;
 	static char namebuf[256];
 
 #			ifdef HAVE_GETSERVBYNAME
-	if ((sp = (struct servent *) getservbyname (service, "tcp")) ==  NULL) {
-		my_fprintf (stderr, "%s/tcp: Unknown service.\n", service); /* FIXME: -> lang.c */
+	if ((sp = (struct servent *) getservbyname (service, "tcp")) == NULL) {
+		my_fprintf (stderr, _("%s/tcp: Unknown service.\n"), service); /* FIXME: -> lang.c */
 		return (-EHOSTUNREACH);
 	}
 #			else
@@ -413,12 +417,13 @@ get_tcp_socket (
 #			endif /* HAVE_GETSERVBYNAME */
 
 	/* If not a raw ip address, try nameserver */
-	if (!isdigit((unsigned char)*machine) ||
+	if (!isdigit((unsigned char) *machine) ||
 #			ifdef HAVE_INET_ATON
-	    !inet_aton(machine, &defaddr))
+	    !inet_aton(machine, &defaddr)
 #			else
-	    (long)(defaddr.s_addr = (long) inet_addr (machine)) == -1)
+	    (long)(defaddr.s_addr = (long) inet_addr (machine)) == -1
 #			endif /* HAVE_INET_ATON */
+	    )
 	{
 		hp = gethostbyname (machine);
 	} else {
@@ -436,7 +441,7 @@ get_tcp_socket (
 	}
 
 	if (hp == NULL) {
-		my_fprintf (stderr, "\n%s: Unknown host.\n", machine); /* FIXME: -> lang.c */
+		my_fprintf (stderr, _("\n%s: Unknown host.\n"), machine); /* FIXME: -> lang.c */
 		return (-EHOSTUNREACH);
 	}
 
@@ -450,9 +455,9 @@ get_tcp_socket (
 #		endif /* !EXCELAN */
 
 	/*
-	 * The following is kinda gross.  The name server under 4.3
+	 * The following is kinda gross. The name server under 4.3
 	 * returns a list of addresses, each of which should be tried
-	 * in turn if the previous one fails.  However, 4.2 hostent
+	 * in turn if the previous one fails. However, 4.2 hostent
 	 * structure doesn't have this list of addresses.
 	 * Under 4.3, h_addr is a #define to h_addr_list[0].
 	 * We use this to figure out whether to include the NS specific
@@ -464,29 +469,35 @@ get_tcp_socket (
 	 * Get a socket and initiate connection -- use multiple addresses
 	 */
 	for (cp = hp->h_addr_list; cp && *cp; cp++) {
+#			if defined(__hpux) && defined(SVR4)
+		unsigned long socksize, socksizelen;
+#			endif /* __hpux && SVR4 */
+
 		if ((s = socket (hp->h_addrtype, SOCK_STREAM, 0)) < 0) {
-			perror ("socket"); /* FIXME: -> lang.c */
+			perror ("socket");
 			return (-errno);
 		}
 
 		memcpy((char *) &sock_in.sin_addr, *cp, hp->h_length);
 
+#			ifdef HAVE_INET_NTOA
 		if (x < 0)
-			my_fprintf (stderr, "Trying %s", (char *) inet_ntoa (sock_in.sin_addr)); /* FIXME: -> lang.c */
+			my_fprintf (stderr, _("Trying %s"), (char *) inet_ntoa (sock_in.sin_addr)); /* FIXME: -> lang.c */
+#			endif /* HAVE_INET_NTOA */
 
 #			if defined(__hpux) && defined(SVR4)	/* recommended by raj@cup.hp.com */
 #				define HPSOCKSIZE 0x8000
-		getsockopt(s, SOL_SOCKET, SO_SNDBUF, (caddr_t)&socksize, (caddr_t)&socksizelen);
+		getsockopt(s, SOL_SOCKET, SO_SNDBUF, /* (caddr_t) */ &socksize, /* (caddr_t) */ &socksizelen);
 		if (socksize < HPSOCKSIZE) {
 			socksize = HPSOCKSIZE;
-			setsockopt(s, SOL_SOCKET, SO_SNDBUF, (caddr_t)&socksize, sizeof(socksize));
+			setsockopt(s, SOL_SOCKET, SO_SNDBUF, /* (caddr_t) */ &socksize, sizeof(socksize));
 		}
 		socksize = 0;
 		socksizelen = sizeof(socksize);
-		getsockopt(s, SOL_SOCKET, SO_RCVBUF, (caddr_t)&socksize, (caddr_t)&socksizelen);
+		getsockopt(s, SOL_SOCKET, SO_RCVBUF, /* (caddr_t) */ &socksize, /* (caddr_t) */ &socksizelen);
 		if (socksize < HPSOCKSIZE) {
 			socksize = HPSOCKSIZE;
-			setsockopt(s, SOL_SOCKET, SO_RCVBUF, (caddr_t)&socksize, sizeof(socksize));
+			setsockopt(s, SOL_SOCKET, SO_RCVBUF, /* (caddr_t) */ &socksize, sizeof(socksize));
 		}
 #			endif /* __hpux && SVR4 */
 
@@ -494,20 +505,22 @@ get_tcp_socket (
 			break;
 
 		save_errno = errno;									/* Keep for later */
-		my_fprintf (stderr, "\nConnection to %s: ", (char *) inet_ntoa (sock_in.sin_addr)); /* FIXME: -> lang.c */
+#			ifdef HAVE_INET_NTOA
+		my_fprintf (stderr, _("\nConnection to %s: "), (char *) inet_ntoa (sock_in.sin_addr)); /* FIXME: -> lang.c */
 		perror ("");
+#			endif /* HAVE_INET_NTOA */
 		(void) s_close (s);
 	}
 
 	if (x < 0) {
-		my_fprintf (stderr, "Giving up...\n"); /* FIXME: -> lang.c */
+		my_fprintf (stderr, _("Giving up...\n")); /* FIXME: -> lang.c */
 		return (-save_errno);					/* Return the last errno we got */
 	}
-#		else	/* no name server */
+#		else
 
 #			ifdef EXCELAN
 	if ((s = socket (SOCK_STREAM, (struct sockproto *)NULL, &sock_in, SO_KEEPALIVE)) < 0) {
-		perror ("socket"); /* FIXME: -> lang.c */
+		perror ("socket");
 		return (-errno);
 	}
 
@@ -517,21 +530,21 @@ get_tcp_socket (
 	sock_in.sin_port = htons (IPPORT_NNTP);
 
 	if ((sock_in.sin_addr.s_addr = rhost (&machine)) == -1) {
-		my_fprintf (stderr, "\n%s: Unknown host.\n", machine); /* FIXME: -> lang.c */
+		my_fprintf (stderr, _("\n%s: Unknown host.\n"), machine); /* FIXME: -> lang.c */
 		return (-1);
 	}
 
 	/* And connect */
 	if (connect (s, (struct sockaddr *)&sock_in) < 0) {
 		save_errno = errno;
-		perror ("connect"); /* FIXME: -> lang.c */
+		perror ("connect");
 		(void) s_close (s);
 		return (-save_errno);
 	}
 
-#			else /* not EXCELAN */
+#			else
 	if ((s = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
-		perror ("socket"); /* FIXME: -> lang.c */
+		perror ("socket");
 		return (-errno);
 	}
 
@@ -541,7 +554,7 @@ get_tcp_socket (
 
 	if (connect (s, (struct sockaddr *) &sock_in, sizeof (sock_in)) < 0) {
 		save_errno = errno;
-		perror ("connect"); /* FIXME: -> lang.c */
+		perror ("connect");
 		(void) s_close (s);
 		return (-save_errno);
 	}
@@ -549,10 +562,11 @@ get_tcp_socket (
 #			endif /* !EXCELAN */
 #		endif /* !h_addr */
 #	endif /* !TLI */
-	return (s);
+	return s;
 }
 #endif /* NNTP_ABLE && !INET6 */
 
+
 #if defined(NNTP_ABLE) && defined(INET6)
 /*
  * get_tcp6_socket -- get us a socket connected to the server.
@@ -572,20 +586,15 @@ get_tcp6_socket (
 	char *machine,
 	unsigned short port)
 {
-	char mymachine[MAXHOSTNAMELEN+1];
+	char mymachine[MAXHOSTNAMELEN + 1];
 	char myport[12];
 	int s = -1, err = -1;
 	struct addrinfo hints, *res, *res0;
 
-#	if 0 /* currently we don't know if we have snprintf */
-	snprintf(mymachine, sizeof(mymachine)-1, "%s", machine);
-	snprintf(myport, sizeof(myport)-1, "%d", port);
-#	else
-	my_strncpy(mymachine, machine, sizeof(mymachine)-1);
-	sprintf(myport, "%d", port);
-#	endif /* 0 */
-	mymachine[sizeof(mymachine)-1] = '\0';
-	myport[sizeof(myport)-1] = '\0';
+	snprintf(mymachine, sizeof(mymachine) - 1, "%s", machine);
+	snprintf(myport, sizeof(myport) - 1, "%d", port);
+	mymachine[sizeof(mymachine) - 1] = '\0';
+	myport[sizeof(myport) - 1] = '\0';
 
 /* just in case */
 #	ifdef AF_UNSPEC
@@ -601,10 +610,11 @@ get_tcp6_socket (
 /*	hints.ai_flags    = AI_CANONNAME; */
 	hints.ai_family   = ADDRFAM;
 	hints.ai_socktype = SOCK_STREAM;
-	res = res0 = NULL;
+	res = (struct addrinfo *) 0;
+	res0 = (struct addrinfo *) 0;
 	err = getaddrinfo(mymachine, myport, &hints, &res0);
 	if (err != 0) {
-		my_fprintf (stderr, "\ngetaddrinfo: %s\n", gai_strerror(err)); /* FIXME: -> lang.c */
+		my_fprintf (stderr, "\ngetaddrinfo: %s\n", gai_strerror(err));
 		return (-1);
 	}
 	err = -1;
@@ -618,12 +628,13 @@ get_tcp6_socket (
 			break;
 		}
 	}
-	if (res0) freeaddrinfo(res0);
+	if (res0 != (struct addrinfo *) 0)
+		freeaddrinfo(res0);
 	if (err < 0) {
-		my_fprintf (stderr, "\nsocket or connect problem\n"); /* FIXME: -> lang.c */
-		return (-1);
+		my_fprintf (stderr, _("\nsocket or connect problem\n")); /* FIXME: -> lang.c */
+		return -1;
 	}
-	return(s);
+	return s;
 }
 #endif /* NNTP_ABLE && INET6 */
 
@@ -658,6 +669,7 @@ get_dnet_socket (
 		case 1:
 			node = area;
 			area = 0;
+			/* FALLTHROUGH */
 		case 2:
 			node += area*1024;
 			sdn.sdn_add.a_len = 2;
@@ -667,7 +679,7 @@ get_dnet_socket (
 			break;
 		default:
 			if ((np = getnodebyname (machine)) == NULL) {
-				my_fprintf (stderr, "%s: Unknown host.\n", machine); /* FIXME: -> lang.c */
+				my_fprintf (stderr, _("%s: Unknown host.\n"), machine); /* FIXME: -> lang.c */
 				return (-1);
 			} else {
 				memcpy((char *) sdn.sdn_add.a_addr, np->n_addr, np->n_length);
@@ -682,21 +694,21 @@ get_dnet_socket (
 	memcpy(&sdn.sdn_objname[0], "NNTP", sdn.sdn_objnamel);
 
 	if ((s = socket (AF_DECnet, SOCK_STREAM, 0)) < 0) {
-		nerror ("socket"); /* FIXME: -> lang.c */
+		nerror ("socket");
 		return (-1);
 	}
 
 	/* And then connect */
 
 	if (connect (s, (struct sockaddr *) &sdn, sizeof (sdn)) < 0) {
-		nerror ("connect"); /* FIXME: -> lang.c */
+		nerror ("connect");
 		close (s);
-		return (-1);
+		return -1;
 	}
 
-	return (s);
+	return s;
 #	else
-	return (-1);
+	return -1;
 #	endif /* NNTP_ABLE */
 }
 #endif /* DECNET */
@@ -705,7 +717,6 @@ get_dnet_socket (
 /*
  * u_put_server -- send data to the server. Do not flush output.
  */
-
 #ifndef VMS
 #	ifdef NNTP_ABLE
 void
@@ -730,8 +741,8 @@ u_put_server (
  *			Closes connection if things are not right.
  *
  *	Note:	This routine flushes the buffer each time
- *			it is called.  For large transmissions
- *			(i.e., posting news) don't use it.  Instead,
+ *			it is called. For large transmissions
+ *			(i.e., posting news) don't use it. Instead,
  *			do the fprintf's yourself, and then a final
  *			fflush.
  */
@@ -749,11 +760,10 @@ put_server (
 	s_puts (string, nntp_wr_fp);
 	s_puts ("\r\n", nntp_wr_fp);
 	(void) s_flush (nntp_wr_fp);
-
-	return;
 }
 #	endif /* NNTP_ABLE */
 
+
 /*
  * Reconnect to server after a timeout, reissue last command to
  * get us back into the pre-timeout state
@@ -772,9 +782,9 @@ reconnect (
 	if (!tinrc.auto_reconnect)
 		ring_bell ();
 
-	DEBUG_IO((stderr, "\nServer timed out, trying reconnect # %d\n", retry));
+	DEBUG_IO((stderr, _("\nServer timed out, trying reconnect # %d\n"), retry));
 
-	if (!tinrc.auto_reconnect && prompt_yn (cLINES, txt_reconnect_to_news_server, TRUE) != 1)
+	if (!tinrc.auto_reconnect && prompt_yn (cLINES, _(txt_reconnect_to_news_server), TRUE) != 1)
 		tin_done(EXIT_SUCCESS);		/* user said no to reconnect */
 
 	clear_message ();
@@ -787,13 +797,13 @@ reconnect (
 		 * Re-establish our current group and resend last command
 		 */
 		if (glob_group != (char *) 0) {
-			DEBUG_IO((stderr, "Rejoin current group\n"));
+			DEBUG_IO((stderr, _("Rejoin current group\n")));
 			sprintf (last_put, "GROUP %s", glob_group);
 			put_server (last_put);
 			s_gets (last_put, NNTP_STRLEN, nntp_rd_fp);
-			DEBUG_IO((stderr, "Read (%s)\n", last_put));
+			DEBUG_IO((stderr, _("Read (%s)\n"), last_put));
 		}
-		DEBUG_IO((stderr, "Resend last command (%s)\n", buf));
+		DEBUG_IO((stderr, _("Resend last command (%s)\n"), buf));
 		put_server (buf);
 		return 0;
 	}
@@ -834,7 +844,7 @@ get_server (
 	while (nntp_rd_fp == NULL || s_gets (string, size, nntp_rd_fp) == (char *) 0) {
 
 #		ifdef DEBUG
-		if (errno != 0 && errno != EINTR)	/*	I'm sure this will only confuse end users*/
+		if (errno != 0 && errno != EINTR)	/* I'm sure this will only confuse end users */
 			perror_message("get_server()");
 #		endif /* DEBUG */
 
@@ -845,19 +855,18 @@ get_server (
 		 * when user is quitting tin if tinrc.auto_reconnect is false.
 		 */
 		if (strncmp(last_put, "QUIT", 4)) {
-			retry = reconnect(retry);			/* Will abort when out of tries */
+			retry = reconnect(retry);		/* Will abort when out of tries */
 			reconnected_in_last_get_server = TRUE;
 		} else {
 			/*
 			 * Use standard NNTP closing message and response code if user is
 			 * quitting tin and leave loop.
 			 */
-			strncpy(string, txt_nntp_ok_goodbye, size - 2);
-			strcat(string, cCRLF);	/* tin_fgets() needs CRLF */
+			strncpy(string, _(txt_nntp_ok_goodbye), size - 2);
+			strcat(string, cCRLF);		/* tin_fgets() needs CRLF */
 			break;
 		}
 	}
-
 	return string;
 }
 #	endif /* NNTP_ABLE */
@@ -883,7 +892,7 @@ close_server (
 	if (nntp_wr_fp == NULL || nntp_rd_fp == NULL)
 		return;
 
-	my_fputs("Disconnecting from server...\n", stdout); /* FIXME: -> lang.c */
+	my_fputs(_("Disconnecting from server...\n"), stdout); /* FIXME: -> lang.c */
 	nntp_command("QUIT", OK_GOODBYE, NULL);
 
 	(void) s_fclose (nntp_wr_fp);
@@ -894,6 +903,7 @@ close_server (
 #	endif /* NNTP_ABLE */
 #endif /* !VMS */
 
+
 #ifdef DEBUG
 /*
  * NNTP strings for get_respcode()
@@ -910,174 +920,174 @@ nntp_respcode (
 			text = "";
 			break;
 		case INF_HELP:
-			text = "100  Help text on way";
+			text = _("100  Help text on way");
 			break;
 		case INF_AUTH:
-			text = "180  Authorization capabilities";
+			text = _("180  Authorization capabilities");
 			break;
 		case INF_DEBUG:
-			text = "199  Debug output";
+			text = _("199  Debug output");
 			break;
 		case OK_CANPOST:
-			text = "200  Hello; you can post";
+			text = _("200  Hello; you can post");
 			break;
 		case OK_NOPOST:
-			text = "201  Hello; you can't post";
+			text = _("201  Hello; you can't post");
 			break;
 		case OK_SLAVE:
-			text = "202  Slave status noted";
+			text = _("202  Slave status noted");
 			break;
 		case OK_GOODBYE:
-			text = "205  Closing connection";
+			text = _("205  Closing connection");
 			break;
 		case OK_GROUP:
-			text = "211  Group selected";
+			text = _("211  Group selected");
 			break;
 		/* case OK_MOTD: */
 		case OK_GROUPS:
-			text = "215  Newsgroups follow";
+			text = _("215  Newsgroups follow");
 			break;
 		case OK_XINDEX:
-			text = "218  Group index file follows";
+			text = _("218  Group index file follows");
 			break;
 		case OK_ARTICLE:
-			text = "220  Article (head & body) follows";
+			text = _("220  Article (head & body) follows");
 			break;
 		case OK_HEAD:
-			text = "221  Head follows";
+			text = _("221  Head follows");
 			break;
 		case OK_BODY:
-			text = "222  Body follows";
+			text = _("222  Body follows");
 			break;
 		case OK_NOTEXT:
-			text = "223  No text sent -- stat, next, last";
+			text = _("223  No text sent -- stat, next, last");
 			break;
 		case OK_NEWNEWS:
-			text = "230  New articles by message-id follow";
+			text = _("230  New articles by message-id follow");
 			break;
 		case OK_NEWGROUPS:
-			text = "231  New newsgroups follow";
+			text = _("231  New newsgroups follow");
 			break;
 		case OK_XFERED:
-			text = "235  Article transferred successfully";
+			text = _("235  Article transferred successfully");
 			break;
 		case OK_POSTED:
-			text = "240  Article posted successfully";
+			text = _("240  Article posted successfully");
 			break;
 		case OK_AUTHSYS:
-			text = "280  Authorization system ok";
+			text = _("280  Authorization system ok");
 			break;
 		case OK_AUTH:
-			text = "281  Authorization (user/pass) ok";
+			text = _("281  Authorization (user/pass) ok");
 			break;
 		case OK_BIN:
-			text = "282  binary data follows";
+			text = _("282  binary data follows");
 			break;
 		case OK_SPLIST:
-			text = "283  spooldir list follows";
+			text = _("283  spooldir list follows");
 			break;
 		case OK_SPSWITCH:
-			text = "284  Switching to a different spooldir";
+			text = _("284  Switching to a different spooldir");
 			break;
 		case OK_SPNOCHANGE:
-			text = "285  Still using same spooldir";
+			text = _("285  Still using same spooldir");
 			break;
 		case OK_SPLDIRCUR:
-			text = "286  Current spooldir";
+			text = _("286  Current spooldir");
 			break;
 		case OK_SPLDIRAVL:
-			text = "287  Available spooldir";
+			text = _("287  Available spooldir");
 			break;
 		case OK_SPLDIRERR:
-			text = "288  Unavailable spooldir or invalid entry";
+			text = _("288  Unavailable spooldir or invalid entry");
 			break;
 		case CONT_XFER:
-			text = "335  Continue to send article";
+			text = _("335  Continue to send article");
 			break;
 		case CONT_POST:
-			text = "340  Continue to post article";
+			text = _("340  Continue to post article");
 			break;
 		case NEED_AUTHINFO:
-			text = "380  authorization is required";
+			text = _("380  authorization is required");
 			break;
 		case NEED_AUTHDATA:
-			text = "381  <type> authorization data required";
+			text = _("381  <type> authorization data required");
 			break;
 		case ERR_GOODBYE:
-			text = "400  Have to hang up for some reason";
+			text = _("400  Have to hang up for some reason");
 			break;
 		case ERR_NOGROUP:
-			text = "411  No such newsgroup";
+			text = _("411  No such newsgroup");
 			break;
 		case ERR_NCING:
-			text = "412  Not currently in newsgroup";
+			text = _("412  Not currently in newsgroup");
 			break;
 		case ERR_XINDEX:
-			text = "418  No index file for this group";
+			text = _("418  No index file for this group");
 			break;
 		case ERR_NOCRNT:
-			text = "420  No current article selected";
+			text = _("420  No current article selected");
 			break;
 		case ERR_NONEXT:
-			text = "421  No next article in this group";
+			text = _("421  No next article in this group");
 			break;
 		case ERR_NOPREV:
-			text = "422  No previous article in this group";
+			text = _("422  No previous article in this group");
 			break;
 		case ERR_NOARTIG:
-			text = "423  No such article in this group";
+			text = _("423  No such article in this group");
 			break;
 		case ERR_NOART:
-			text = "430  No such article at all";
+			text = _("430  No such article at all");
 			break;
 		case ERR_GOTIT:
-			text = "435  Already got that article, don't send";
+			text = _("435  Already got that article, don't send");
 			break;
 		case ERR_XFERFAIL:
-			text = "436  Transfer failed";
+			text = _("436  Transfer failed");
 			break;
 		case ERR_XFERRJCT:
-			text = "437  Article rejected, don't resend";
+			text = _("437  Article rejected, don't resend");
 			break;
 		case ERR_NOPOST:
-			text = "440  Posting not allowed";
+			text = _("440  Posting not allowed");
 			break;
 		case ERR_POSTFAIL:
-			text = "441  Posting failed";
+			text = _("441  Posting failed");
 			break;
 		case ERR_NOAUTH:
-			text = "480  authorization required for command";
+			text = _("480  authorization required for command");
 			break;
 		case ERR_AUTHSYS:
-			text = "481  Authorization system invalid";
+			text = _("481  Authorization system invalid");
 			break;
 		case ERR_AUTHREJ:
-			text = "482  Authorization data rejected";
+			text = _("482  Authorization data rejected");
 			break;
 		case ERR_INVALIAS:
-			text = "483  Invalid alias on spooldir cmd";
+			text = _("483  Invalid alias on spooldir cmd");
 			break;
 		case ERR_INVNOSPDIR:
-			text = "484  No spooldir file found";
+			text = _("484  No spooldir file found");
 			break;
 		case ERR_COMMAND:
-			text = "500  Command not recognized";
+			text = _("500  Command not recognized");
 			break;
 		case ERR_CMDSYN:
-			text = "501  Command syntax error";
+			text = _("501  Command syntax error");
 			break;
 		case ERR_ACCESS:
-			text = "502  Access to server denied";
+			text = _("502  Access to server denied");
 			break;
 		/* case ERR_MOTD: */
 		case ERR_FAULT:
-			text = "503  Program fault, command not performed";
+			text = _("503  Program fault, command not performed");
 			break;
 		case ERR_AUTHBAD:
-			text = "580  Authorization Failed";
+			text = _("580  Authorization Failed");
 			break;
 		default:
-			text = "Unknown NNTP response code";
+			text = _("Unknown NNTP response code");
 			break;
 	}
 	return (text);
