@@ -8,13 +8,39 @@
  *              ~/.tin/newsrctable and returns the real hostname
  *              and the name of the newsrc file for a given
  *              alias of the server.
- *  Copyright : (c) Copyright 1996-99 by Sven Paulus
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1996-2002 Sven Paulus <sven@tin.org>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Sven Paulus.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -22,10 +48,10 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 #ifndef VERSION_H
-#	include  "version.h"
+#	include "version.h"
 #endif /* !VERSION_H */
 #ifndef MENUKEYS_H
-#	include  "menukeys.h"
+#	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
 /*
@@ -47,7 +73,7 @@ write_newsrctable_file (
 	if ((fp = fopen(local_newsrctable_file, "w")) == (FILE *) 0)
 		return;
 
-	fprintf(fp, txt_nrctbl_info, tin_progname, VERSION);
+	fprintf(fp, _(txt_nrctbl_info), PRODUCT, VERSION);
 	fclose(fp);
 }
 
@@ -127,7 +153,7 @@ get_newsrcname (
 						do_cpy = TRUE;
 					}
 
-					if ((line_entry_counter ==1) && ((!strcasecmp(line_entry, "default")) || (!strcmp(line_entry,"*")))) {
+					if ((line_entry_counter == 1) && ((!strcasecmp(line_entry, "default")) || (!strcmp(line_entry, "*")))) {
 						found = 2;
 						do_cpy = TRUE;
 					}
@@ -144,9 +170,8 @@ get_newsrcname (
 			char tmp_newsrc[PATH_LEN];
 			int error = 0;
 
-			if (!strfpath (name_found, tmp_newsrc, sizeof (tmp_newsrc),
-				homedir, (char *) 0, (char *) 0, (char *) 0)) {
-					my_fprintf (stderr, "couldn't expand %s\n", name_found);
+			if (!strfpath (name_found, tmp_newsrc, sizeof (tmp_newsrc), NULL)) {
+					my_fprintf (stderr, _("couldn't expand %s\n"), name_found);
 					error = 1;
 			} else {
 				if (tmp_newsrc[0] == '/')
@@ -161,43 +186,43 @@ get_newsrcname (
 			if (!error) {
 			/* FIXME - write a global permssion check routine */
 				if (access (dir, X_OK)) {
-					my_fprintf (stderr, txt_error_no_enter_permission, dir);
+					my_fprintf (stderr, _(txt_error_no_enter_permission), dir);
 					error = 1;
 				} else if (access (newsrc_name, F_OK)) {
-					my_fprintf (stderr, txt_error_no_such_file, newsrc_name);
+					my_fprintf (stderr, _(txt_error_no_such_file), newsrc_name);
 					error = 2;
 				} else if (access (dir, R_OK)) {
-					my_fprintf (stderr, txt_error_no_read_permission, dir);
+					my_fprintf (stderr, _(txt_error_no_read_permission), dir);
 					error = 1;
 				} else if (access (newsrc_name, R_OK)) {
-					my_fprintf (stderr, txt_error_no_read_permission, newsrc_name);
+					my_fprintf (stderr, _(txt_error_no_read_permission), newsrc_name);
 					error = 1;
 				} else if (access (dir, W_OK)) {
-					my_fprintf (stderr, txt_error_no_write_permission, dir);
+					my_fprintf (stderr, _(txt_error_no_write_permission), dir);
 					error = 1;
 				} else if (access (newsrc_name, W_OK)) {
-					my_fprintf (stderr, txt_error_no_write_permission, newsrc_name);
+					my_fprintf (stderr, _(txt_error_no_write_permission), newsrc_name);
 					error = 1;
 				}
 			}
 			if (error) {
 				char ch;
-				char default_ch = iKeyNrctblAlternative;
+				char default_ch = map_to_local (iKeyNrctblAlternative, &menukeymap.nrctbl_create);
 
 				do {
 					/* very ugly code, but curses is not initialized yet */
 					if (error >= 2) {
-						default_ch = iKeyNrctblCreate;
-						printf("%s%c\b", txt_nrctbl_create, default_ch);
+						default_ch = map_to_local (iKeyNrctblCreate, &menukeymap.nrctbl_create);
+						printf("%s%c\b", _(txt_nrctbl_create), default_ch);
 					} else
-						printf("%s%c\b", txt_nrctbl_default, default_ch);
+						printf("%s%c\b", _(txt_nrctbl_default), default_ch);
 
 					if ((ch = (char) ReadCh ()) == '\r' || ch == '\n')
 						ch = default_ch;
-				} while (!strchr ("\033acdq", ch));
+				} while (!strchr (menukeymap.nrctbl_create.localkeys, ch));
 				printf("%c\n", ch);
 
-				switch(ch) {
+				switch (map_to_default (ch, &menukeymap.nrctbl_create)) {
 					case iKeyNrctblCreate:
 						/* FIXME this doesn't check if we could create the file */
 						return TRUE;
