@@ -6,11 +6,36 @@
  *  Updated   : 1999-07-17
  *  Notes     : Routines to make reading news locally (ie. /var/spool/news)
  *              or via NNTP transparent
- *  Copyright : (c) Copyright 1991-99 by Iain Lea & Rich Skrenta
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea, Rich Skrenta.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 
@@ -25,7 +50,6 @@
 /*
  * local prototypes
  */
-static int base_comp (t_comptype *p1, t_comptype *p2);
 #if 0 /* currently unused */
 	static FILE * open_xhdr_fp (char *header, long min, long max);
 #endif /* 0 */
@@ -41,7 +65,8 @@ long head_next;
 
 char *nntp_server = (char *)0;
 static char txt_xover_string[] = "XOVER";
-char *txt_xover = txt_xover_string;
+static char *txt_xover = txt_xover_string;
+
 
 /*
  * Open a connection to the NNTP server. Authenticate if necessary or
@@ -73,16 +98,16 @@ nntp_open (
 		nntp_server = getserverbyfile (NNTP_SERVER_FILE);
 
 	if (nntp_server == (char *) 0) {
-		error_message (txt_cannot_get_nntp_server_name);
-		error_message (txt_server_name_in_file_env_var, NNTP_SERVER_FILE);
+		error_message (_(txt_cannot_get_nntp_server_name));
+		error_message (_(txt_server_name_in_file_env_var), NNTP_SERVER_FILE);
 		return -EHOSTUNREACH;
 	}
 
-	if (INTERACTIVE) {
+	if (!batch_mode) {
 		if (nntp_tcp_port != 119)
-			wait_message (0, txt_connecting_port, nntp_server, nntp_tcp_port);
+			wait_message (0, _(txt_connecting_port), nntp_server, nntp_tcp_port);
 		else
-			wait_message (0, txt_connecting, nntp_server);
+			wait_message (0, _(txt_connecting), nntp_server);
 	}
 
 #	ifdef DEBUG
@@ -90,7 +115,7 @@ nntp_open (
 #	endif /* DEBUG */
 
 	ret = server_init (nntp_server, NNTP_TCP_NAME, nntp_tcp_port, line);
-DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
+	DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
 
 	if (!batch_mode && ret >= 0 && cmd_line)
 		my_fputc ('\n', stdout);
@@ -105,15 +130,16 @@ DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
 		 * ret > 0 : NNTP response code
 		 *
 		 * According to the ietf-nntp mailinglist:
-		 *  200 you may (try to) do anything
-		 *  201 you may not POST
-		 *  202 you may not IHAVE
-		 *  203 you may not do EITHER
-		 *  All unrecognised 200 series codes should be assumed as success.
-		 *  All unrecognised 300 series codes should be assumed as notice to continue.
-		 *  All unrecognised 400 series codes should be assumed as temporary error.
-		 *  All unrecognised 500 series codes should be assumed as error.
+		 *   200 you may (try to) do anything
+		 *   201 you may not POST
+		 *   202 you may not IHAVE
+		 *   203 you may not do EITHER
+		 *   All unrecognised 200 series codes should be assumed as success.
+		 *   All unrecognised 300 series codes should be assumed as notice to continue.
+		 *   All unrecognised 400 series codes should be assumed as temporary error.
+		 *   All unrecognised 500 series codes should be assumed as error.
 		 */
+
 		case OK_CANPOST:
 		case OK_NOIHAVE:
 #	ifndef NO_POSTING
@@ -129,12 +155,12 @@ DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
 		default:
 			if (ret >= 200 && ret <= 299) {
 #	ifndef NO_POSTING
-				 can_post = TRUE;
+				can_post = TRUE;
 #	endif /* !NO_POSTING */
 				break;
 			}
 			if (ret < 0) {
-				error_message (txt_failed_to_connect_to_server, nntp_server);
+				error_message (_(txt_failed_to_connect_to_server), nntp_server);
 			} else {
 				error_message (line);
 			}
@@ -143,7 +169,7 @@ DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
 	if (!is_reconnect) {
 		/* remove leading whitespace and save server's initial response */
 		linep = line;
-		while (isspace((int)*linep))
+		while (isspace((int) *linep))
 			linep++;
 
 		STRCPY(bug_nntpserver1, linep);
@@ -156,13 +182,30 @@ DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
 #	ifdef DEBUG
 	debug_nntp ("nntp_open", "mode reader");
 #	endif /* DEBUG */
-DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
+	DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 	put_server ("MODE READER");
 
+	/*
+	 * According to the latest NNTP draft (Jan 2002), MODE READER may only
+	 * return the following response codes:
+	 *
+	 *   200 (OK_CANPOST)     Hello, you can post
+	 *   201 (OK_NOPOST)      Hello, you can't post
+	 *  (202 (OK_NOIHAVE)     discussed on the itef mailinglist)
+	 *  (203 (OK_NOPOSTIHAVE) discussed on the itef mailinglist)
+	 *   400 (ERR_GOODBYE)    Service temporarily unavailable
+	 *   502 (ERR_ACCESS)     Service unavailable
+	 *
+	 * However, there may be old servers out there that do not implement this
+	 * command and therefore return ERR_COMMAND (500). Unfortunately there
+	 * are some new servers out there (i.e. INN 2.4.0 (20020220 prerelease)
+	 * which do return ERR_COMMAND if they are feed only servers.
+	 */
+
 	ret = get_respcode(line);
 	switch (ret) {
 		case OK_CANPOST:
-		case OK_NOIHAVE:
+/*		case OK_NOIHAVE: */
 #	ifndef NO_POSTING
 			can_post = TRUE;
 #	endif /* !NO_POSTING */
@@ -170,7 +213,7 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 			break;
 
 		case OK_NOPOST:
-		case OK_NOPOSTIHAVE:
+/*		case OK_NOPOSTIHAVE: */
 			can_post = FALSE;
 			sec = TRUE;
 			break;
@@ -187,9 +230,8 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 	}
 
 	/*
-	 * NOTE: Latest NNTP draft (Aug 1998) states that LIST EXTENSIONS should
-	 *       (not SHOULD, however) be used to find out what commands are
-	 *       supported.
+	 * NOTE: Latest NNTP draft (Jan 2002) states that LIST EXTENSIONS should
+	 *       be used to find out what commands are supported.
 	 *
 	 * TODO: Implement LIST EXTENSIONS here. Get this list before issuing
 	 *       authentication because the authentication method required may be
@@ -214,7 +256,7 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 		ret = get_respcode (line);
 		switch (ret) {
 			case OK_CANPOST:
-			case OK_NOIHAVE:
+/*			case OK_NOIHAVE: */
 #	ifndef NO_POSTING
 				can_post = TRUE;
 #	endif /* !NO_POSTING */
@@ -222,7 +264,7 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 				break;
 
 			case OK_NOPOST:
-			case OK_NOPOSTIHAVE:
+/*			case OK_NOPOSTIHAVE: */
 				can_post = FALSE;
 				sec = TRUE;
 				break;
@@ -242,11 +284,11 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 	if (!is_reconnect) {
 		/* Inform user if he cannot post */
 		if (!can_post)
-			wait_message(0, "%s\n", txt_cannot_post);
+			wait_message(0, "%s\n", _(txt_cannot_post));
 
 		/* Remove leading white space and save server's second response */
 		linep = line;
-		while (isspace((int)*linep))
+		while (isspace((int) *linep))
 			linep++;
 
 		STRCPY(bug_nntpserver2, linep);
@@ -265,7 +307,7 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 			j = atoi (get_val ("COLUMNS", "80"));
 			chr1 = my_strdup ((sec ? bug_nntpserver2 : bug_nntpserver1));
 
-			if (((int)strlen (chr1)) >= j) {
+			if (((int) strlen (chr1)) >= j) {
 				chr2 = chr1 + strlen (chr1) - 1;
 				while (chr2 - chr1 >= j)
 					chr2--;
@@ -282,8 +324,7 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 
 	/*
 	 * Check if NNTP supports XOVER or OVER (successor of XOVER as of latest
-	 * NNTP Draft (Aug 1999)) command
-	 * ie, we _don't_ get an ERR_COMMAND
+	 * NNTP Draft (Jan 2002) command; ie, we _don't_ get an ERR_COMMAND
 	 *
 	 * TODO: Don't try (X)OVER if listed in LIST EXTENSIONS.
 	 */
@@ -299,16 +340,15 @@ DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
 			/* TODO issue warning if old index files found ? */
 		} else {
 			if (!is_reconnect) {
-				wait_message(2, txt_no_xover_support);
+				wait_message(2, _(txt_no_xover_support));
 			}
 		}
 	}
 
 #	if 0 /* TODO */
-	/* if we're using -n, check for XGTITLE */
+	/* if we're using -n, check for LIST NEWSGROUPS <wildmat> */
 	if (newsrc_active && !list_active) { /* -n */
-		if (!nntp_command("XGTITLE", ERR_COMMAND, NULL))
-			xgtitle_supported = TRUE;
+		/* code goes here */
 	}
 #	endif /* 0 */
 
@@ -335,24 +375,24 @@ nntp_close (
 #endif /* NNTP_ABLE */
 }
 
-/*
- *  Get a response code from the server.
- *  Returns:
- *    +ve NNTP return code
- *    -1  on an error or user abort. We don't differentiate.
- *  If 'message' is not NULL, then any trailing text after the response
- *  code is copied into it.
- *  Does not perform authentication if required; use get_respcode()
- *  instead.
- */
 
+/*
+ * Get a response code from the server.
+ * Returns:
+ * 	+ve NNTP return code
+ * 	-1  on an error or user abort. We don't differentiate.
+ * If 'message' is not NULL, then any trailing text after the response
+ * code is copied into it.
+ * Does not perform authentication if required; use get_respcode()
+ * instead.
+ */
 int
 get_only_respcode (
 	char *message)
 {
+	int respcode;
 #ifdef NNTP_ABLE
 	char *ptr, *end;
-	int respcode;
 
 	ptr = tin_fgets (FAKE_NNTP_FP, FALSE);
 
@@ -383,31 +423,29 @@ DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
 	if (message != NULL)				/* Pass out the rest of the text */
 		strcpy(message, end);
 
-	return respcode;
-#else
-	return 0;
 #endif /* NNTP_ABLE */
+	return respcode;
 }
 
-/*
- *  Get a response code from the server.
- *  Returns:
- *    +ve NNTP return code
- *    -1  on an error
- *  If 'message' is not NULL, then any trailing text after the response
- *	 code is copied into it.
- *  Performs authentication if required and repeats the last command if
- *  necessary after a timeout.
- */
 
+/*
+ * Get a response code from the server.
+ * Returns:
+ * 	+ve NNTP return code
+ * 	-1  on an error
+ * If 'message' is not NULL, then any trailing text after the response
+ *	code is copied into it.
+ * Performs authentication if required and repeats the last command if
+ * necessary after a timeout.
+ */
 int
 get_respcode (
 	char *message)
 {
+	int respcode = 0;
 #ifdef NNTP_ABLE
 	char savebuf[NNTP_STRLEN];
 	char *ptr, *end;
-	int respcode;
 
 	respcode = get_only_respcode (message);
 	if ((respcode == ERR_NOAUTH) || (respcode == NEED_AUTHINFO)) {
@@ -433,18 +471,16 @@ get_respcode (
 				strcpy(message, end);
 
 		} else {
-			error_message (txt_auth_failed, ERR_ACCESS);
-/*			return -1;*/
+			error_message (_(txt_auth_failed), ERR_ACCESS);
+			/*	return -1; */
 			tin_done (EXIT_FAILURE);
 		}
 	}
-
-	return respcode;
-#else
-	return 0;
 #endif /* NNTP_ABLE */
+	return respcode;
 }
 
+
 #ifdef NNTP_ABLE
 /*
  * Do an NNTP command. Send command to server, and read the reply.
@@ -466,7 +502,7 @@ DEBUG_IO((stderr, "nntp_command (%s)\n", command));
 	put_server (command);
 
 	if (!bool_equal(dangerous_signal_exit, TRUE))
-		if ((/* respcode = */ get_respcode (message)) != success) {
+		if ((get_respcode (message)) != success) {
 #	ifdef DEBUG
 			debug_nntp (command, "NOT_OK");
 #	endif /* DEBUG */
@@ -481,10 +517,10 @@ DEBUG_IO((stderr, "nntp_command (%s)\n", command));
 }
 #endif /* NNTP_ABLE */
 
+
 /*
  * Open the news active file locally or send the LIST command
  */
-
 FILE *
 open_news_active_fp (
 	void)
@@ -497,10 +533,10 @@ open_news_active_fp (
 		return (fopen (news_active_file, "r"));
 }
 
+
 /*
  * Open the NEWSLIBDIR/overview.fmt file locally or send LIST OVERVIEW.FMT
  */
-
 FILE *
 open_overview_fmt_fp (
 	void)
@@ -523,12 +559,12 @@ open_overview_fmt_fp (
 #endif /* NNTP_ABLE */
 }
 
+
 /*
  * Open the active.times file locally or send the NEWGROUPS command
  *
  * NEWGROUPS yymmdd hhmmss
  */
-
 FILE *
 open_newgroups_fp (
 	int idx)
@@ -558,6 +594,7 @@ open_newgroups_fp (
 		return (fopen (active_times_file, "r"));
 }
 
+
 /*
  * Get a list of default groups to subscribe to
  */
@@ -574,6 +611,7 @@ open_subscription_fp (
 		return (fopen (subscriptions_file, "r"));
 }
 
+
 #ifdef HAVE_MH_MAIL_HANDLING
 /*
  * Open the mail active file locally
@@ -585,6 +623,7 @@ open_mail_active_fp (
 	return fopen (mail_active_file, mode);
 }
 
+
 /*
  *  Open mail groups description file locally
  */
@@ -596,6 +635,7 @@ open_mailgroups_fp (
 }
 #endif /* HAVE_MH_MAIL_HANDLING */
 
+
 /*
  * If reading via NNTP the newsgroups file will be saved to ~/.tin/newsgroups
  * so that any subsequent rereads on the active file will not have to waste
@@ -619,12 +659,13 @@ open_newsgroups_fp (
 			read_local_newsgroups_file = FALSE;
 		}
 #	if 0 /* TODO */
-		if (xgtitle_supported && newsrc_active
+		if (list_newsgroups_wildmat_supported && newsrc_active
 		    && !list_active
-		    && num_active < some_usefull_limit) {
+		    && num_active < some_useful_limit) {
 			for (i = 0; i < num_active; i++) {
-				sprintf(buff, "XGTITLE %s", active[i].name);
-				nntp_command(buff, OK_LIST, NULL));
+				sprintf(buff, "LIST NEWSGROUPS %s", active[i].name);
+				nntp_command(buff, OK_LIST, NULL);
+			}
 		} else
 #	endif /* 0 */
 		return (nntp_command ("LIST NEWSGROUPS", OK_GROUPS, NULL));
@@ -633,34 +674,34 @@ open_newsgroups_fp (
 		return fopen (newsgroups_file, "r");
 }
 
+
 /*
  * Open a group NOV/XOVER file
  */
-
 FILE *
 open_xover_fp (
 	struct t_group *psGrp,
-	const char *pcMode,
+	const char *mode,
 	long lMin,
 	long lMax)
 {
 #ifdef NNTP_ABLE
-	if (read_news_via_nntp && xover_supported && *pcMode == 'r' && psGrp->type == GROUP_TYPE_NEWS) {
-		char acLine[NNTP_STRLEN];
+	if (read_news_via_nntp && xover_supported && *mode == 'r' && psGrp->type == GROUP_TYPE_NEWS) {
+		char line[NNTP_STRLEN];
 
-		sprintf (acLine, "%s %ld-%ld", txt_xover, lMin, lMax);
-		return(nntp_command (acLine, OK_XOVER, NULL));
+		sprintf (line, "%s %ld-%ld", txt_xover, lMin, lMax);
+		return (nntp_command (line, OK_XOVER, NULL));
 	} else {
 #endif /* NNTP_ABLE */
 		char *pcNovFile;
 
-		pcNovFile = pcFindNovFile (psGrp, (*pcMode == 'r' ? R_OK : W_OK));
+		pcNovFile = find_nov_file (psGrp, (*mode == 'r' ? R_OK : W_OK));
 #ifdef DEBUG
 		if (debug)
 			error_message ("READ file=[%s]", pcNovFile);
 #endif /* DEBUG */
 		if (pcNovFile != (char *) 0)
-			return fopen (pcNovFile, pcMode);
+			return fopen (pcNovFile, mode);
 
 		return (FILE *) 0;
 #ifdef NNTP_ABLE
@@ -675,15 +716,15 @@ open_xover_fp (
 t_bool
 stat_article (
 	long art,
-	char *group_path)
+	const char *group_path)
 {
 	char buf[NNTP_STRLEN];
-	int i;
+	struct t_group currgrp;
 
-	i = my_group[cur_groupnum];
+	currgrp = CURR_GROUP;
 
 #ifdef NNTP_ABLE
-	if (read_news_via_nntp && active[i].type == GROUP_TYPE_NEWS) {
+	if (read_news_via_nntp && currgrp.type == GROUP_TYPE_NEWS) {
 		sprintf (buf, "STAT %ld", art);
 		return(nntp_command (buf, OK_NOTEXT, NULL) != NULL);
 	} else
@@ -691,13 +732,17 @@ stat_article (
 	{
 		struct stat sb;
 
-		joinpath (buf, active[i].spooldir, group_path);
+		joinpath (buf, currgrp.spooldir, group_path);
 		sprintf (&buf[strlen (buf)], "/%ld", art);
 
 		return (stat (buf, &sb) != -1);
 	}
 }
 
+
+/*
+ * Open an article for reading just the header
+ */
 FILE *
 open_art_header (
 	long art)
@@ -736,152 +781,37 @@ open_art_header (
 #endif /* NNTP_ABLE */
 }
 
-#ifdef NNTP_ABLE
-/*
- * Copies an article from the nntp socket and writes it to a temp file.
- * Returns open descriptor to this file. We have to do this in order
- * to do header and MIME parsing, and other functions that use seek()
- */
-static FILE *
-get_article (
-	FILE *art_fp,
-	int lines)
-{
-	FILE *fp;
-	char *ptr;
-	char tempfile[PATH_LEN];
-	int count = 0;
-#	if defined(HAVE_FDOPEN) && defined(HAVE_MKSTEMP)
-	int fd = -1;
-#	endif /* HAVE_FDOPEN && HAVE_MKSTEMP */
-	struct stat sb;
-
-	sprintf (tempfile, "%stin_nntpXXXXXX", TMPDIR);
-#	if defined(HAVE_FDOPEN) && defined(HAVE_MKSTEMP)
-	if ((fd = my_mktemp (tempfile)) == -1) {
-		perror_message (txt_cannot_create_uniq_name);
-		return (FILE *) 0;
-	}
-	if ((fp = fdopen (fd, "w")) == (FILE *) 0) {
-#	else
-	mktemp (tempfile);
-	if ((fp = fopen (tempfile, "w")) == (FILE *) 0) {
-#	endif /* HAVE_FDOPEN && HAVE_MKSTEMP */
-		perror_message (txt_article_cannot_open, tempfile);
-		return (FILE *) 0;
-	}
-
-	while ((ptr = tin_fgets(art_fp, FALSE)) != NULL) {
-		fputs (ptr, fp);
-		fputs ("\n", fp);		/* The one case where we do need the \n */
-								/* as rfc1521_decode() still expects this */
-		/*
-		 * Use the default message if one hasn't been supplied
-		 * Body search is currently the only function that has a different message
-		 */
-
-		if (lines && ++count % MODULO_COUNT_NUM == 0)
-			show_progress((*mesg=='\0') ? txt_reading_article : mesg, count, lines);
-
-	}
-
-	if (ferror(fp) || tin_errno) {
-		fclose(fp);
-		if (!tin_errno) {
-			tin_errno = 1;
-			error_message(txt_filesystem_full, tempfile);
-		}
-#	if defined(M_AMIGA) || defined(WIN32)
-		log_unlink(fp, tempfile);
-#	else
-		unlink (tempfile);
-#	endif /* M_AMIGA || WIN32 */
-		return (FILE *) 0;
-	}
-
-	fclose(fp);
-
-	/*
-	 * Grab the correct filesize now that it's been closed
-	 */
-	note_size = ((stat (tempfile, &sb) < 0) ? 0 : sb.st_size);
-
-	if ((fp = fopen (tempfile, "r")) == (FILE *) 0) {	/* Reopen for just reading */
-		perror_message (txt_article_cannot_reopen, tempfile);
-		return (FILE *) 0;
-	}
-
-
-	/*
-	 * It is impossible to delete an open file on the Amiga or Win32. So we keep a
-	 * copy of the file name and delete it when finished instead.
-	 */
-#	if defined(M_AMIGA) || defined(WIN32)
-	log_unlink(fp, tempfile);
-#	else
-	unlink (tempfile);
-#	endif /* M_AMIGA || WIN32 */
-
-	return fp;
-}
-#endif /* NNTP_ABLE */
 
 /*
- * Open a mail/news article. If via NNTP, despool it to local disk. We need the
- * article local for later. Run it through RFC1521 decode.
- * If lines != 0 then show progress meter.
+ * Open a mail/news article. Do MIME decoding if necessary.
  * Return:
  *		A pointer to the open postprocessed file
  *		NULL pointer if article read fails in some way
  */
-
 FILE *
 open_art_fp (
-	char *group_path,
-	long art,
-	int lines,
-	t_bool rfc1521decode)
+	const char *group_path,
+	long art)
 {
 	char buf[NNTP_STRLEN];
-	struct stat sb;
-	FILE *fp, *art_fp;
+	FILE *art_fp;
 
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && CURR_GROUP.type == GROUP_TYPE_NEWS) {
-		FILE *nntp_fp;
-
-		sprintf (buf, "ARTICLE %ld", art);
-		if ((nntp_fp = nntp_command (buf, OK_ARTICLE, NULL)) == NULL)
+		snprintf (buf, sizeof(buf) - 1, "ARTICLE %ld", art);
+		if ((art_fp = nntp_command (buf, OK_ARTICLE, NULL)) == NULL)
 			return (FILE *) 0;
-
-		art_fp = get_article (nntp_fp, lines);
 	} else {
 #endif /* NNTP_ABLE */
 		joinpath (buf, CURR_GROUP.spooldir, group_path);
 		sprintf (&buf[strlen (buf)], "/%ld", art);
-		/*
-		 * Get the correct file size. This is done in get_article() for
-		 * the NNTP case. TODO - fix this
-		 */
-		note_size = ((stat (buf, &sb) == -1) ? 0 : sb.st_size);
 
 		art_fp = fopen (buf, "r");
 #ifdef NNTP_ABLE
 	}
 #endif /* NNTP_ABLE */
 
-	/*
-	 * Do a bit of 1521 decoding, if appropriate.
-	 * If art_fp=NULL, then it returns NULL
-	 */
-	if (rfc1521decode) {
-		fp = rfc1521_decode(art_fp);
-		if (fp != art_fp)
-			note_size = 0;
-	} else
-		fp = art_fp;
-
-	return fp;
+	return art_fp;
 }
 
 
@@ -891,8 +821,8 @@ open_art_fp (
  */
 static int
 base_comp (
-	t_comptype *p1,
-	t_comptype *p2)
+	t_comptype p1,
+	t_comptype p2)
 {
 	const long *a = (const long *) p1;
 	const long *b = (const long *) p2;
@@ -921,19 +851,19 @@ base_comp (
  * Grow the arts[] and bitmaps as needed.
  * NB: the output will be sorted on artnum
  *
- * top_base is one past top.
+ * grpmenu.max is one past top.
  * Returns total number of articles in group, or -1 on error
  */
 long
 setup_hard_base (
 	struct t_group *group,
-	char *group_path)
+	const char *group_path)
 {
 	char buf[NNTP_STRLEN];
 	long art;
 	long total = 0;
 
-	top_base = 0;
+	grpmenu.max = 0;
 
 	/*
 	 * If reading with NNTP, issue a LISTGROUP
@@ -964,10 +894,10 @@ setup_hard_base (
 #	endif /* DEBUG */
 
 			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
-				if (top_base >= max_art)
+				if (grpmenu.max >= max_art)
 					expand_art ();
 
-				base[top_base++] = atoi (ptr);
+				base[grpmenu.max++] = atoi (ptr);
 			}
 
 			if (tin_errno)
@@ -991,7 +921,7 @@ setup_hard_base (
 			if (nntp_command(buf, OK_GROUP, line) == NULL)
 				return(-1);
 
-			if (sscanf (line,"%ld %ld %ld", &count, &start, &last) != 3)
+			if (sscanf (line, "%ld %ld %ld", &count, &start, &last) != 3)
 				return(-1);
 
 			total = count;
@@ -1000,9 +930,9 @@ setup_hard_base (
 				count = last - start;
 
 			while (start <= last) {
-				if (top_base >= max_art)
+				if (grpmenu.max >= max_art)
 					expand_art();
-				base[top_base++] = start++;
+				base[grpmenu.max++] = start++;
 			}
 		}
 #endif /* NNTP_ABLE */
@@ -1016,7 +946,7 @@ setup_hard_base (
 		joinpath (buf, group->spooldir, group_path);
 
 		if (access (buf, R_OK) != 0) {
-			error_message(txt_not_exist);
+			error_message(_(txt_not_exist));
 			return (-1);
 		}
 
@@ -1025,19 +955,19 @@ setup_hard_base (
 				art = atol (e->d_name);
 				if (art >= 1) {
 					total++;
-					if (top_base >= max_art)
+					if (grpmenu.max >= max_art)
 						expand_art ();
-					base[top_base++] = art;
+					base[grpmenu.max++] = art;
 				}
 			}
 			CLOSEDIR(d);
-			qsort ((char *) base, (size_t)top_base, sizeof (long), base_comp);
+			qsort ((char *) base, (size_t)grpmenu.max, sizeof (long), base_comp);
 		}
 	}
 
-	if (top_base) {
-		if (base[top_base-1] > group->xmax)
-			group->xmax = base[top_base-1];
+	if (grpmenu.max) {
+		if (base[grpmenu.max - 1] > group->xmax)
+			group->xmax = base[grpmenu.max - 1];
 		expand_bitmap (group, base[0]);
 	}
 
@@ -1052,7 +982,7 @@ vGet1GrpArtInfo (
 	long lMinOld = grp->xmin;
 	long lMaxOld = grp->xmax;
 
-	vGrpGetArtInfo (grp->spooldir, grp->name, grp->type, &grp->count, &grp->xmax, &grp->xmin);
+	group_get_art_info (grp->spooldir, grp->name, grp->type, &grp->count, &grp->xmax, &grp->xmin);
 
 	if (grp->newsrc.num_unread > grp->count) {
 #ifdef DEBUG
@@ -1078,98 +1008,93 @@ vGet1GrpArtInfo (
  *  Return 0, or -error
  */
 int
-vGrpGetArtInfo (
-	char *pcSpoolDir,
-	char *pcGrpName,
-	int iGrpType,
-	long *plArtCount,
-	long *plArtMax,
-	long *plArtMin)
+group_get_art_info (
+	char *tin_spooldir,
+	char *groupname,
+	int grouptype,
+	long *art_count,
+	long *art_max,
+	long *art_min)
 {
-	DIR *tDirFile;
-	DIR_BUF *tFile;
-	char acBuf[NNTP_STRLEN];
-	long lArtNum;
+	DIR *dir;
+	DIR_BUF *direntry;
+	char buf[NNTP_STRLEN];
+	long artnum;
 #ifdef M_AMIGA
-	long lArtMin;
-	long lArtMax;
+	long artmin;
+	long artmax;
 
-	lArtMin = *plArtMin;
-	lArtMax = *plArtMax;
+	artmin = *art_min;
+	artmax = *art_max;
 #endif /* M_AMIGA */
 
-	if (read_news_via_nntp && iGrpType == GROUP_TYPE_NEWS) {
+	if (read_news_via_nntp && grouptype == GROUP_TYPE_NEWS) {
 #ifdef NNTP_ABLE
-		char acLine[NNTP_STRLEN];
+		char line[NNTP_STRLEN];
 
-		sprintf (acBuf, "GROUP %s", pcGrpName);
+		sprintf (buf, "GROUP %s", groupname);
 #	ifdef DEBUG
-		debug_nntp ("vGrpGetArtInfo", acBuf);
+		debug_nntp ("group_get_art_info", buf);
 #	endif /* DEBUG */
-		put_server (acBuf);
+		put_server (buf);
 
-		switch (get_respcode(acLine)) {
+		switch (get_respcode(line)) {
 
 			case OK_GROUP:
-				if (sscanf (acLine, "%ld %ld %ld", plArtCount, plArtMin, plArtMax) != 3)
-					error_message("Invalid response to GROUP command, %s", acLine);
+				if (sscanf (line, "%ld %ld %ld", art_count, art_min, art_max) != 3)
+					error_message(_("Invalid response to GROUP command, %s"), line);
 				break;
 
 			case ERR_NOGROUP:
-				*plArtCount = 0;
-				*plArtMin = 1;
-				*plArtMax = 0;
+				*art_count = 0;
+				*art_min = 1;
+				*art_max = 0;
 				return(-ERR_NOGROUP);
 
 			case ERR_ACCESS:
-				error_message (cCRLF "%s", acLine);
+				error_message (cCRLF "%s", line);
 				tin_done (NNTP_ERROR_EXIT);
 				/* keep lint quiet: */
-				/* FALLTHROUGH */
+				/* NOTREACHED */
+				break;
+
 			default:
 #	ifdef DEBUG
-				debug_nntp ("NOT_OK", acLine);
+				debug_nntp ("NOT_OK", line);
 #	endif /* DEBUG */
 				return(-1);
 		}
 #else
-		my_fprintf(stderr, "Unreachable ?\n");
+		my_fprintf(stderr, _("Unreachable ?\n"));
 		return 0;
 #endif /* NNTP_ABLE */
 	} else {
 #ifdef M_AMIGA
-		if (!lArtMin)
-			*plArtMin = 1;
-		*plArtMax = lArtMax;
-		*plArtCount = lArtMax - *plArtMin + 1;
+		if (!artmin)
+			*art_min = 1;
+		*art_max = artmax;
+		*art_count = artmax - *art_min + 1;
 #else
-		*plArtCount = 0;
-		*plArtMin = 1;
-		*plArtMax = 0;
+		*art_count = 0;
+		*art_min = 1;
+		*art_max = 0;
 
-		vMakeGrpPath (pcSpoolDir, pcGrpName, acBuf);
+		make_base_group_path (tin_spooldir, groupname, buf);
 
-/* TODO - Surely this is spurious, the opendir will fail anyway */
-/*		  unless there is some subtle permission check if tin is suid news? */
-#	if 0
-		if (access (acBuf, R_OK) != 0)
-			return(-1);
-#	endif /* 0 */
-
-		if ((tDirFile = opendir (acBuf)) != (DIR *) 0) {
-			while ((tFile = readdir (tDirFile)) != (DIR_BUF *) 0) {
-				lArtNum = atol(tFile->d_name); /* should be '\0' terminated... */
-				if (lArtNum >= 1) {
-					if (lArtNum > *plArtMax) {
-						*plArtMax = lArtNum;
-						if (*plArtMin == 0)
-							*plArtMin = lArtNum;
-					} else if (lArtNum < *plArtMin)
-						*plArtMin = lArtNum;
-					(*plArtCount)++;
+		if ((dir = opendir (buf)) != (DIR *) 0) {
+			while ((direntry = readdir (dir)) != (DIR_BUF *) 0) {
+				artnum = atol(direntry->d_name); /* should be '\0' terminated... */
+				if (artnum >= 1) {
+					if (artnum > *art_max) {
+						*art_max = artnum;
+						if (*art_min == 0)
+							*art_min = artnum;
+					} else if (artnum < *art_min)
+						*art_min = artnum;
+					(*art_count)++;
 				}
 			}
-			CLOSEDIR(tDirFile);
+			CLOSEDIR(dir);
 		} else
 			return(-1);
 #endif /* M_AMIGA */
