@@ -3,15 +3,41 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 1994-12-21
+ *  Updated   : 2001-07-22
  *  Notes     : signal handlers for different modes and window resizing
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -21,6 +47,9 @@
 #ifndef included_trace_h
 #	include "trace.h"
 #endif /* !included_trace_h */
+#ifndef VERSION_H
+#	include "version.h"
+#endif /* !VERSION_H */
 
 
 #if defined (VMS) && defined (SIGBUS)
@@ -34,7 +63,6 @@
 /*
  * Needed for resizing under an xterm
  */
-
 #ifdef HAVE_TERMIOS_H
 #	include <termios.h>
 #else
@@ -88,18 +116,20 @@ static void _CDECL signal_handler (SIG_ARGS);
 
 int signal_context = cMain;
 int need_resize = cNo;
+int NOTESLINES;			/* # lines of useable area */
 
+
+#ifndef __LCLINT__ /* lclint doesn't like it */
 static const struct {
 	int code;
 	const char *name;
 } signal_list[] = {
-#ifdef SIGINT
+#	ifdef SIGINT
 	{ SIGINT,	"SIGINT" },	/* ctrl-C */
-#endif /* SIGINT */
-#ifdef SIGQUIT
+#	endif /* SIGINT */
+#	ifdef SIGQUIT
 	{ SIGQUIT,	"SIGQUIT " },	/* ctrl-\ */
-#endif /* SIGQUIT */
-#ifndef WIN32
+#	endif /* SIGQUIT */
 #	ifdef SIGILL
 	{ SIGILL,	"SIGILL" },	/* illegal instruction */
 #	endif /* SIGILL */
@@ -112,40 +142,41 @@ static const struct {
 #	ifdef SIGSEGV
 	{ SIGSEGV,	"SIGSEGV" },	/* segmentation violation */
 #	endif /* SIGSEGV */
-#endif /* !WIN32 */
-#ifdef SIGPIPE
+#	ifdef SIGPIPE
 	{ SIGPIPE,	"SIGPIPE" },	/* broken pipe */
-#endif /* SIGPIPE */
-#ifdef SIGCHLD
+#	endif /* SIGPIPE */
+#	ifdef SIGCHLD
 	{ SIGCHLD,	"SIGCHLD" },	/* death of a child process */
-#endif /* SIGCHLD */
-#ifdef SIGPWR
+#	endif /* SIGCHLD */
+#	ifdef SIGPWR
 	{ SIGPWR,	"SIGPWR" },	/* powerfail */
-#endif /* SIGPWR */
-#ifdef SIGTSTP
+#	endif /* SIGPWR */
+#	ifdef SIGTSTP
 	{ SIGTSTP,	"SIGTSTP" },	/* terminal-stop */
-#endif /* SIGTSTP */
-#ifdef SIGHUP
+#	endif /* SIGTSTP */
+#	ifdef SIGHUP
 	{ SIGHUP,	"SIGHUP" },	/* hang up */
-#endif /* SIGHUP */
-#ifdef SIGUSR1
+#	endif /* SIGHUP */
+#	ifdef SIGUSR1
 	{ SIGUSR1,	"SIGUSR1" },	/* User-defined signal 1 */
-#endif /* SIGUSR1 */
-#ifdef SIGTERM
+#	endif /* SIGUSR1 */
+#	ifdef SIGTERM
 	{ SIGTERM,	"SIGTERM" },	/* termination */
-#endif /* SIGTERM */
-#ifdef SIGWINCH
+#	endif /* SIGTERM */
+#	ifdef SIGWINCH
 	{ SIGWINCH,	"SIGWINCH" },	/* window-size change */
-#endif /* SIGWINCH */
+#	endif /* SIGWINCH */
 };
+#endif /* !__LCLINT__ */
 
-#	ifdef HAVE_NESTED_PARAMS
-RETSIGTYPE (*sigdisp(int signum, RETSIGTYPE (_CDECL *func)(SIG_ARGS)))(SIG_ARGS)
-#	else
-RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
+
+#ifdef HAVE_NESTED_PARAMS
+	RETSIGTYPE (*sigdisp(int signum, RETSIGTYPE (_CDECL *func)(SIG_ARGS)))(SIG_ARGS)
+#else
+	RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
 	int signum;
 	RETSIGTYPE (_CDECL *func)(SIG_ARGS);
-#	endif /* HAVE_NESTED_PARAMS */
+#endif /* HAVE_NESTED_PARAMS */
 {
 #ifdef HAVE_POSIX_JC
 #	define RESTORE_HANDLER(x, y)
@@ -159,7 +190,6 @@ RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
 #	endif /* SA_RESTART */
 	if (sigaction (signum, &sa, &osa) < 0)
 		return SIG_ERR;
-
 	return (osa.sa_handler);
 #else
 #	define RESTORE_HANDLER(x, y)  signal (x, y)
@@ -167,6 +197,7 @@ RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
 #endif /* HAVE_POSIX_JC */
 }
 
+
 /*
  * Block/unblock SIGWINCH/SIGTSTP restarting syscalls
  */
@@ -174,7 +205,7 @@ void
 allow_resize (
 	t_bool allow)
 {
-#	ifdef HAVE_POSIX_JC
+#ifdef HAVE_POSIX_JC
 	struct sigaction sa, osa;
 
 	sa.sa_handler = signal_handler;
@@ -193,12 +224,14 @@ allow_resize (
 #endif /* HAVE_POSIX_JC */
 }
 
+
 static const char *
 signal_name (
 	int code)
 {
 	size_t n;
 	const char *name = "unknown";
+
 	for (n = 0; n < SIZEOF(signal_list); n++) {
 		if (signal_list[n].code == code) {
 			name = signal_list[n].name;
@@ -208,6 +241,7 @@ signal_name (
 	return name;
 }
 
+
 /*
  * Rescale the display buffer and redraw the contents according to
  * the current context
@@ -224,19 +258,19 @@ handle_resize (
 
 	if (cLINES < MIN_LINES_ON_TERMINAL || cCOLS < MIN_COLUMNS_ON_TERMINAL) {
 		ring_bell ();
-		wait_message(3, txt_screen_too_small_exiting);
+		wait_message(3, _(txt_screen_too_small_exiting), tin_progname);
 		tin_done (EXIT_FAILURE);
 	}
 
-	TRACE(("handle_resize(%d:%d)", signal_context, repaint))
+	TRACE(("handle_resize(%d:%d)", signal_context, repaint));
 
 	if (!repaint)
 		return;
 
 #	ifdef USE_CURSES
 #		ifdef HAVE_RESIZETERM
-/*	resizeterm(cLINES+1, cCOLS+1); */
-	resizeterm(cLINES+1, cCOLS);
+	resizeterm(cLINES + 1, cCOLS);
+	my_retouch();					/* seems necessary if win size unchanged */
 #		else
 	my_retouch();
 #		endif /* HAVE_RESIZETERM */
@@ -244,33 +278,27 @@ handle_resize (
 
 	switch (signal_context) {
 		case cArt:
+			ClearScreen ();
 			show_art_msg (glob_group);
 			break;
 		case cConfig:
 			refresh_config_page (-1);
 			break;
-		case cHelp:
-#	ifdef USE_CURSES
-			display_info_page (TRUE);
-#	else
-			display_info_page();
-#	endif /* USE_CURSES*/
+		case cFilter:
+			refresh_filter_menu ();
+			break;
+		case cInfopager:
+			display_info_page (0);
 			break;
 		case cGroup:
-			ClearScreen ();
-			show_group_page ();
-			break;
-		case cPage:
-			ClearScreen ();
-			redraw_page (glob_group, glob_respnum);
-			break;
 		case cSelect:
-			ClearScreen ();
-			show_selection_page ();
-			break;
 		case cThread:
 			ClearScreen ();
-			show_thread_page ();
+			currmenu->redraw();
+			break;
+		case cPage:
+			resize_article (TRUE, &pgart);
+			draw_page (glob_group, 0);
 			break;
 		case cMain:
 			break;
@@ -279,19 +307,20 @@ handle_resize (
 #endif /* SIGWINCH || SIGTSTP */
 }
 
+
 #ifdef SIGTSTP
 static void
 handle_suspend (
 	void)
 {
-	TRACE(("handle_suspend(%d)", signal_context))
+	TRACE(("handle_suspend(%d)", signal_context));
 
 	set_keypad_off ();
 	if (!cmd_line)
 		set_xclick_off ();
 
 	Raw (FALSE);
-	wait_message (0, txt_suspended_message);
+	wait_message (0, _(txt_suspended_message), tin_progname);
 
 	kill (0, SIGSTOP);				/* Put ourselves to sleep */
 
@@ -307,6 +336,7 @@ handle_suspend (
 }
 #endif /* SIGTSTP */
 
+
 static void _CDECL
 signal_handler (
 	int sig)
@@ -346,39 +376,46 @@ signal_handler (
 			RESTORE_HANDLER(sig, signal_handler);
 			return;
 #endif /* SIGPIPE */
+
 #ifdef SIGTSTP
 		case SIGTSTP:
 			handle_suspend();
 			return;
 #endif /* SIGTSTP */
+
 #ifdef SIGWINCH
 		case SIGWINCH:
 			need_resize = cYes;
 			RESTORE_HANDLER (SIGWINCH, signal_handler);
 			return;
 #endif /* SIGWINCH */
+
 		default:
 			break;
 	}
 	fprintf (stderr, "\n%s: signal handler caught %s signal (%d).\n", tin_progname, signal_name(sig), sig);
+
 #if defined(SIGHUP)
 	if (sig == SIGHUP) {
 		dangerous_signal_exit = TRUE;
 		tin_done (- SIGHUP);
 	}
 #endif /* SIGHUP */
+
 #if defined(SIGUSR1)
 	if (sig == SIGUSR1) {
 		dangerous_signal_exit = TRUE;
 		tin_done (- SIGUSR1);
 	}
 #endif /* SIGUSR1 */
+
 #if defined(SIGTERM)
 	if (sig == SIGTERM) {
 		dangerous_signal_exit = TRUE;
 		tin_done (- SIGTERM);
 	}
 #endif /* SIGTERM */
+
 #if defined(SIGBUS) || defined(SIGSEGV)
 	if (
 #	ifdef SIGBUS
@@ -391,7 +428,10 @@ signal_handler (
 		sig == SIGSEGV
 #	endif /* SIGSEGV */
 		) {
-		vPrintBugAddress ();
+		/* FIXME: -> lang.c */
+		my_fprintf (stderr, _("%s %s %s (\"%s\") [%s]: send a DETAILED bug report to %s\n"),
+		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
+		my_fflush (stderr);
 	}
 #endif /* SIGBUS || SIGSEGV */
 	cleanup_tmp_files ();
@@ -405,8 +445,9 @@ signal_handler (
 #endif /* 1 */ /* apollo || HAVE_COREFILE */
 }
 
+
 /*
- * Turn on (flag==TRUE) our signal handler for TSTP and WINCH
+ * Turn on (flag != FALSE) our signal handler for TSTP and WINCH
  * Otherwise revert to the default handler
  */
 void
@@ -423,6 +464,7 @@ set_signal_catcher (
 #endif /* SIGWINCH */
 }
 
+
 void
 set_signal_handlers (
 	void)
@@ -473,6 +515,11 @@ set_win_size (
 	old_lines = *num_lines;
 	old_cols = *num_cols;
 
+#ifdef HAVE_XCURSES
+	*num_lines = LINES - 1;		/* FIXME */
+	*num_cols = COLS;
+#else	/* curses/ncurses */
+
 #ifndef USE_CURSES
 	init_screen_array (FALSE);		/* deallocate screen array */
 #endif /* !USE_CURSES */
@@ -504,15 +551,17 @@ set_win_size (
 	init_screen_array (TRUE);		/* allocate screen array for resize */
 #endif /* !USE_CURSES */
 
+#endif /* HAVE_XCURSES */
+
 	set_subj_from_size (*num_cols);
 
 	RIGHT_POS = *num_cols - 20;
 	MORE_POS  = *num_cols - 15;
 	set_noteslines (*num_lines);
-
 	return (*num_lines != old_lines || *num_cols != old_cols);
 }
 
+
 void
 set_noteslines (
 	int num_lines)
