@@ -3,15 +3,38 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 1997-12-31
+ *  Updated   : 2001-04-24
  *  Notes     :
- *  Copyright : (c) Copyright 1997-99 by Urs Janssen
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1997-2002 Urs Janssen <urs@tin.org>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -32,9 +55,9 @@ tin_ltoa (
 	int digits)
 {
 	static char buffer[256];
-	static const char power[]=" KMGT";
+	static const char power[] = " KMGT";
 	int len;
-	int i=0;
+	int i = 0;
 
 	sprintf (buffer, "%ld", value);
 	len = (int) strlen (buffer);
@@ -45,10 +68,10 @@ tin_ltoa (
 	}
 
 	if (i) {
-		while (len < (digits-1))
+		while (len < (digits - 1))
 			buffer[len++] = ' ';
 
-		buffer[digits-1] = power[i];
+		buffer[digits - 1] = power[i];
 		buffer[digits] = '\0';
 	} else
 		sprintf (buffer, "%*ld", digits, value);
@@ -63,7 +86,8 @@ tin_ltoa (
  *
  * also, strdup is not mandatory in ANSI-C
  */
-char *my_strdup (
+char *
+my_strdup (
 	const char *str)
 {
 	size_t len = strlen (str) + 1;
@@ -95,37 +119,22 @@ my_strncpy (
 }
 
 
-/* this strcpy variant removes \n and "" */
-void
-strcpynl (
-	char *to,
-	const char *from)
-{
-	while (*from && *from != '\r' && *from != '\n') {
-		if (*from == '"')
-			from++;	/* this is just plain silly */
-		else
-			*to++ = *from++;
-	}
-	*to = 0;
-}
-
-
+#ifndef HAVE_STRCASESTR
 /*
  * case-insensitive version of strstr()
  */
-char *
+const char *
 strcasestr (
-	char *haystack,
+	const char *haystack,
 	const char *needle)
 {
-	char *h;
+	const char *h;
 	const char *n;
 
 	h = haystack;
 	n = needle;
 	while (*haystack) {
-		if (tolower ((unsigned char)*h) == tolower ((unsigned char)*n)) {
+		if (tolower ((unsigned char) *h) == tolower ((unsigned char) *n)) {
 			h++;
 			n++;
 			if (!*n)
@@ -137,6 +146,7 @@ strcasestr (
 	}
 	return NULL;
 }
+#endif /* !HAVE_STRCASESTR */
 
 
 size_t
@@ -155,37 +165,6 @@ mystrcat (
 }
 
 
-void
-modifiedstrncpy (
-	char *target,
-	const char *source,
-	size_t size,
-	int decode)
-{
-	char buf[2048];
-	int count;
-	char *c;
-
-	count = sizeof(buf)-1;
-	c = buf;
-	while (*source) {
-		if (*source!= 1) {
-			*c++ = *source++;
-			if (!--count)
-				break;
-		}
-		else source++;
-	}
-	*c = 0;
-	c = (decode ? (rfc1522_decode(buf)) : buf);
-
-	while (--size)
-		*target++ = *c++;
-
-	*target = 0;
-}
-
-
 void
 str_lwr (
 	char *str)
@@ -217,7 +196,7 @@ strpbrk (
 	register char *ptr2;
 
 	for (ptr1 = str1; *ptr1 != '\0'; ptr1++) {
-		for (ptr2 = str2; *ptr2 != '\0';) {
+		for (ptr2 = str2; *ptr2 != '\0'; ) {
 			if (*ptr1 == *ptr2++)
 				return (ptr1);
 		}
@@ -257,7 +236,7 @@ strstr (
 	delta = deltaspace;
 	for (i = 0; i <= 255; i++)
 		delta[i] = p1;
-	for (p = (unsigned char *) pattern, i = p1; --i > 0;)
+	for (p = (unsigned char *) pattern, i = p1; --i > 0; )
 		delta[*p++] = i;
 
 	/*
@@ -311,7 +290,7 @@ atol (
 
 #ifndef HAVE_STRTOL
 /* fix me - put me in tin.h */
-#define DIGIT(x) (isdigit((unsigned char)x)? ((x)-'0'): (10+tolower((unsigned char)x)-'a'))
+#define DIGIT(x) (isdigit((unsigned char)x) ? ((x) - '0') : (10 + tolower((unsigned char)x) - 'a'))
 #define MBASE 36
 long
 strtol (
@@ -344,15 +323,15 @@ strtol (
 	} else if (use_base == 16)
 		if (str[0] == '0' && (str[1] == 'x' || str[1] == 'X'))
 			str += 2;
-		/*
-		 * for any base > 10, the digits incrementally following
-		 * 9 are assumed to be "abc...z" or "ABC...Z"
-		 */
-		while (isalnum ((unsigned char)*str) && (xx = DIGIT(*str)) < use_base) {
-			/* accumulate neg avoids surprises near maxint */
-			val = use_base * val - xx;
-			++str;
-		}
+	/*
+	 * for any base > 10, the digits incrementally following
+	 * 9 are assumed to be "abc...z" or "ABC...Z"
+	 */
+	while (isalnum ((unsigned char)*str) && (xx = DIGIT(*str)) < use_base) {
+		/* accumulate neg avoids surprises near maxint */
+		val = use_base * val - xx;
+		++str;
+	}
 OUT:
 	if (ptr != (char **) 0)
 		*ptr = str;
@@ -369,7 +348,7 @@ OUT:
  */
 
 /*fix me - put me in tin.h */
-#define	FOLD_TO_UPPER(a)	(toupper ((unsigned char)(a)))
+#define FOLD_TO_UPPER(a)	(toupper ((unsigned char)(a)))
 
 #ifndef HAVE_STRCASECMP
 int
@@ -405,8 +384,7 @@ strncasecmp (
 
 
 /*
- * str_trim - substitute multiple white spaces with one space
- *            and delete heading and trailing whitespace
+ * str_trim - leading and trailing whitespace
  *
  * INPUT:  string  - string to trim
  *
@@ -420,23 +398,24 @@ str_trim (
 {
 	char *rp;		/* reading string pointer */
 	char *wp;		/* writing string pointer */
-	t_bool ws = TRUE;		/* white space flag */
+	char *ls;		/* last space */
 
-	for (wp = rp = string; *rp; rp++) {		/* loop over string */
-		if (isspace((int)*rp)) {		/* is it a white space? */
-			if (!ws) {		/* was the last character not a white space? */
-				*wp++ = ' ';		/* store a blank */
-				ws = TRUE;
-			}
-		} else /* no white space */ {
-			*wp++ = *rp;		/* store the character */
-			ws = FALSE;
-		}
+	for (rp = wp = ls = string; isspace((int)*rp); rp++)		/* Skip leading space */
+		;
+
+	while (*rp) {
+		if (isspace((int)*rp)) {
+			if (ls == NULL)		/* Remember last written space */
+				ls = wp;
+		} else
+			ls = NULL;			/* It wasn't the last space */
+		*wp++ = *rp++;
 	}
-	if (ws)		/* delete trailing blank */
-		wp--;
 
-	*wp = '\0';
+	if (ls)						/* ie, there is trailing space */
+		*ls = '\0';
+	else
+		*wp = '\0';
 
 	return (string);
 }
@@ -567,7 +546,7 @@ sh_format (
 	}
 	va_end(ap);
 
-	if (result+1 >= (int) len)
+	if (result + 1 >= (int) len)
 		result = -1;
 	else
 		*dst = '\0';
@@ -599,7 +578,35 @@ my_strerror (
 	if (n >= 0 && n < sys_nerr)
 		return sys_errlist[n];
 #	endif /* HAVE_SYS_ERRLIST */
-	sprintf(temp, "Errno: %i", n);
+	sprintf(temp, "Errno: %d", n);
 	return temp;
 }
 #endif /* !HAVE_STRERROR */
+
+
+/* strrstr() based on Lars Wirzenius' <lars.wirzenius@helsinki.fi> code */
+#ifndef HAVE_STRRSTR
+char *
+strrstr (
+	const char *str,
+	const char *pat)
+{
+	size_t slen, plen;
+	const char *ptr;
+
+	if (str == (char *) 0 || pat == (char *) 0)
+		return (char *) 0;
+
+	slen = strlen(str);
+	plen = strlen(pat);
+
+	if (plen <= slen) {
+		for (ptr = str + (slen - plen); ptr > str; --ptr) {
+			if (*ptr == *pat && strncmp(ptr, pat, plen) == 0)
+				return (char *) ptr;
+		}
+	}
+
+	return (char *) 0;
+}
+#endif /* HAVE_STRRSTR */
